@@ -1,13 +1,12 @@
 //
 // See the file LICENSE for redistribution information.
 //
-// Copyright (c) 2004-2005
-//	Sleepycat Software.  All rights reserved.
+// Copyright (c) 2004,2006 Oracle.  All rights reserved.
 //
-// $Id: mdConst.java,v 1.3 2005/04/20 18:31:18 bostic Exp $
+// $Id: mdConst.java,v 1.6 2006/10/30 17:45:47 bostic Exp $
 //
 
-package com.sleepycat.dbxml.examples.gettingStarted;
+package dbxml.gettingStarted;
 
 //Simple class used to encapsulate some constants used throughout
 // these examples.
