@@ -1,13 +1,12 @@
 //
 // See the file LICENSE for redistribution information.
 //
-// Copyright (c) 2004-2005
-//	Sleepycat Software.  All rights reserved.
+// Copyright (c) 2004,2006 Oracle.  All rights reserved.
 //
-// $Id: myDb.java,v 1.9 2005/04/20 18:31:18 bostic Exp $
+// $Id: myDb.java,v 1.12 2006/10/30 17:45:47 bostic Exp $
 //
 
-package com.sleepycat.dbxml.examples.gettingStarted;
+package dbxml.gettingStarted;
 
 import java.io.*;
 import java.util.*;
