@@ -1,13 +1,12 @@
 //
 // See the file LICENSE for redistribution information.
 //
-// Copyright (c) 2004-2005
-//	Sleepycat Software.  All rights reserved.
+// Copyright (c) 2004,2006 Oracle.  All rights reserved.
 //
-// $Id: simpleContainerInEnv.java,v 1.8 2005/04/20 18:31:20 bostic Exp $
+// $Id: simpleContainerInEnv.java,v 1.12 2006/11/14 21:25:37 gmf Exp $
 //
 
-package com.sleepycat.dbxml.examples.gettingStarted;
+package dbxml.gettingStarted;
 
 import java.io.*;
 
@@ -44,5 +43,12 @@ class simpleContainerInEnv
 	XmlContainer container3 = mgr.createContainer("myContainer3");
 
 	// do work here //
+
+	// clean up
+	container1.delete();
+	container2.delete();
+	container3.delete();
+	mgr.delete();
+	env.close();
     }
 }
