@@ -1,10 +1,9 @@
 //
 // See the file LICENSE for redistribution information.
 //
-// Copyright (c) 2002-2005
-//	Sleepycat Software.  All rights reserved.
+// Copyright (c) 2002,2006 Oracle.  All rights reserved.
 //
-// $Id: DbXml.hpp,v 1.34 2005/04/05 16:43:53 bostic Exp $
+// $Id: DbXml.hpp,v 1.38 2006/10/30 17:45:47 bostic Exp $
 //
 
 #ifndef __DBXML_HPP
@@ -30,5 +29,8 @@
 #include "XmlTransaction.hpp"
 #include "XmlStatistics.hpp"
 #include "XmlInputStream.hpp"
+#include "XmlEventReader.hpp"
+#include "XmlEventWriter.hpp"
+#include "XmlEventReaderToWriter.hpp"
 
 #endif
