@@ -1,10 +1,9 @@
 //
 // See the file LICENSE for redistribution information.
 //
-// Copyright (c) 2002-2005
-//	Sleepycat Software.  All rights reserved.
+// Copyright (c) 2002,2006 Oracle.  All rights reserved.
 //
-// $Id: XmlResults.hpp,v 1.36 2005/04/05 16:43:56 bostic Exp $
+// $Id: XmlResults.hpp,v 1.40 2006/10/30 17:45:48 bostic Exp $
 //
 
 #ifndef __XMLRESULTS_HPP
@@ -12,6 +11,7 @@
 
 #include "DbXmlFwd.hpp"
 #include "XmlPortability.hpp"
+#include "XmlQueryContext.hpp"
 #include <string>
 
 namespace DbXml
@@ -43,6 +43,8 @@ public:
 	bool hasPrevious();
 	void reset();
 
+	XmlQueryContext::EvaluationType getEvaluationType() const;
+
 	/** @name Eager Results Methods */
 	size_t size() const;
 	void add(const XmlValue &value);
