@@ -1,33 +1,36 @@
 //
 // See the file LICENSE for redistribution information.
 //
-// Copyright (c) 2002-2005
-//	Sleepycat Software.  All rights reserved.
+// Copyright (c) 2002,2006 Oracle.  All rights reserved.
 //
-// $Id: Document.cpp,v 1.152 2005/05/18 22:56:08 jsnelson Exp $
+// $Id: Document.cpp,v 1.195.4.1 2008/04/08 15:06:10 gmf Exp $
 //
 
-#include "dbxml_config.h"
-#include "dbxml/XmlPortability.hpp"
+#include "DbXmlInternal.hpp"
 #include "dbxml/XmlNamespace.hpp"
 #include "dbxml/XmlException.hpp"
 #include "dbxml/XmlValue.hpp"
+#include "dbxml/XmlData.hpp"
 #include "Document.hpp"
 #include "OperationContext.hpp"
 #include "UTF8.hpp"
 #include "Value.hpp"
 #include "MetaDatum.hpp"
 #include "Manager.hpp"
+#include "ReferenceMinder.hpp"
 #include "MemBufInputStream.hpp"
+#include "nodeStore/NsDomReader.hpp"
+#include "nodeStore/NsEventReader.hpp"
+#include "nodeStore/EventReaderToWriter.hpp"
 #include "nodeStore/NsXercesDom.hpp"
 #include "nodeStore/NsDocument.hpp"
 #include "nodeStore/NsSAX2Reader.hpp"
-#include "nodeStore/NsXercesHandler.hpp"
-#include "nodeStore/NsEventGenerator.hpp"
+#include "nodeStore/NsTransientDomBuilder.hpp"
 #include "nodeStore/NsXercesIndexer.hpp"
+#include "nodeStore/NsXercesTranscoder.hpp"
 #include "nodeStore/NsWriter.hpp"
-#include "nodeStore/NsDomEventSource.hpp"
-#include "XmlInputStreamWrapper.hpp"
+#include "nodeStore/PullEventInputStream.hpp"
+#include <dbxml/XmlEventReaderToWriter.hpp>
 #include "Globals.hpp"
 
 #include "db_utils.h"
@@ -42,27 +45,24 @@
 XERCES_CPP_NAMESPACE_USE
 #endif
 
-#include <pathan/internal/XPath2MemoryManagerImpl.hpp>
-
 using namespace DbXml;
 using namespace std;
 
-Document::Document(XmlManager &db)
-	: mgr_(db),
+Document::Document(XmlManager &mgr)
+	: mgr_(mgr),
 	  container_(0),
 	  id_(0),
-	  validate_(false),
+	  documentURI_(0),
+	  validation_(DONT_VALIDATE),
 	  lazy_(NEITHER),
 	  db_flags_(0),
 	  definitiveContent_(NONE),
 	  contentModified_(false),
 	  dbtContent_(0),
 	  inputStream_(0),
+	  reader_(0),
 	  nsDocument_(0),
-	  oldNsDocument_(0),
-	  nsDomFactory_(0),
-	  memMgr_(0),
-	  document_(0)
+	  nsDomFactory_(0)
 {
 }
 
@@ -71,11 +71,54 @@ Document::~Document()
 	reset();
 }
 
+bool Document::operator==(const Document &other) const
+{
+	// if doc IDs are 0, this is a transient doc,
+	// so compare pointers; otherwise, use IDs
+	if (id_ == 0)
+		return (this == &other);
+	return (id_ == other.id_);
+}
+
+Document *Document::createCopy() const
+{
+	Document *doc = new Document(mgr_);
+	doc->setID(getID());
+	doc->setContainer(container_);
+	copyMetaData(*doc);
+	return doc;
+}
+
+bool
+Document::needsCopyForUpdate() const
+{
+	return (nsDocument_ && !contentModified_);
+}
+
+void Document::copyMetaData(Document &doc) const
+{
+	MetaData::const_iterator i;
+	for(i = metaData_.begin(); i != metaData_.end(); ++i) {
+		const Dbt *dbt = (*i)->getDbt();
+		DbtOut *ddbt = new DbtOut(*dbt);
+		MetaDatum *newmd = new MetaDatum((*i)->getName(), (*i)->getType(),
+						 &ddbt, (*i)->isModified());
+		doc.setMetaDataPtr(newmd);
+	}
+}
+
 void Document::reset()
 {
+	if(!refMinders_.empty()) {
+		RefMinders::iterator i;
+		for(i = refMinders_.begin(); i != refMinders_.end();++i)
+			(*i)->removeDocument(this);
+		refMinders_.clear();
+	}
+
 	id_ = 0;
-	identifier_ = string();
-	validate_ = false;
+	setDocumentURI(0);
+	validation_ = DONT_VALIDATE;
 
 	lazy_ = NEITHER;
 	oc_.set(0);
@@ -93,21 +136,33 @@ void Document::reset()
 	resetContentAsDbt();
 	resetContentAsInputStream();
 	resetContentAsDOM();
-	// handle reset content, if present
-	if (oldNsDocument_) {
-		NsDomFactory *fact = oldNsDocument_->getDomFactory();
-		MemoryManager *memMgr = oldNsDocument_->getMemoryManager();
-		delete oldNsDocument_;
-		if (fact)
-			delete fact;
-		if (memMgr)
-			delete memMgr;
-	}
+	resetContentAsEventReader();
 	// hold reference on Container until done
 	setContainer(0);
 }
 
-void Document::setMetaDataFromContainer(TransactedContainer *container, const ID &id, Transaction *txn, u_int32_t flags)
+void Document::setDocumentURI(const XMLCh *uri)
+{
+	if(documentURI_)
+		Globals::defaultMemoryManager->deallocate(documentURI_);
+	documentURI_ = NsUtil::nsStringDup(Globals::defaultMemoryManager, uri, 0);
+}
+
+void Document::addReferenceMinder(ReferenceMinder *r)
+{
+	DBXML_ASSERT(r != 0);
+	refMinders_.insert(r);
+}
+
+void Document::removeReferenceMinder(ReferenceMinder *r)
+{
+	DBXML_ASSERT(r != 0);
+	refMinders_.erase(r);
+}
+
+void Document::setMetaDataFromContainer(TransactedContainer *container,
+					const DocID &id, Transaction *txn,
+					u_int32_t flags)
 {
 	// Set up lazy meta data
 	setContainer(container);
@@ -142,7 +197,7 @@ void Document::setEagerMetaData()
 		if(container_->getContainerType() ==
 		   XmlContainer::WholedocContainer) {
 			lazy_ = NEITHER;
-			oc_.set(0);
+			//oc_.set(0);
 		}
 		else lazy_ = CONTENT;
 	}
@@ -160,8 +215,9 @@ void Document::setEagerContentAndMetaData()
 	setEagerMetaData();
 	getContentAsDbt();
 	resetContentAsDOM();
+	resetContentAsEventReader();
 	lazy_ = NEITHER;
-	oc_.set(0);
+	//oc_.set(0);
 }
 
 void Document::setContainer(TransactedContainer *container)
@@ -173,15 +229,15 @@ void Document::setContainer(TransactedContainer *container)
 		container_ = container;
 		if(container_) {
 			container_->acquire();
-			validate_ = container_->getDoValidation();
-		}
-		else {
-			validate_ = false;
+			validation_ = (container_->getDoValidation()) ? VALIDATE :
+				DONT_VALIDATE;
+		} else {
+			validation_ = DONT_VALIDATE;
 		}
 	}
 }
 
-TransactedContainer *Document::getContainer()
+TransactedContainer *Document::getContainer() const
 {
 	return container_;
 }
@@ -197,6 +253,12 @@ void Document::setMetaData(const Name &name, const XmlValue &value, bool modifie
 		throw XmlException(XmlException::INVALID_VALUE,
 				   "setMetaData expects a typed value");
 		break;
+	case XmlValue::BINARY: {
+		DbtOut *ddbt = new DbtOut(value.asBinary().getDbt());
+		setMetaDataPtr(new MetaDatum(name, value.getType(),
+					     &ddbt, modified));
+		return;
+	}
 	default:
 		// one of the atomic types
 		break;
@@ -324,9 +386,21 @@ const MetaDatum *Document::getMetaDataPtr(const Name &name) const
 	}
 	if(r == 0 && lazy_ == BOTH) {
 		// If we can't find it, then try looking it up in the database
-		int err = container_->getDocumentDB()->
-			getMetaData(oc_, container_->getDictionaryDB(), name,
-				    id_, const_cast<Document*>(this), db_flags_);
+		int err = 0;
+		DbtOut *data = new DbtOut();
+		try {
+			XmlValue::Type type;
+			err = container_->getDocumentDB()->
+				getMetaData(oc_, container_->getDictionaryDB(), name,
+					id_, type, data, db_flags_);
+			if(err == 0) const_cast<Document*>(this)->
+					     setMetaData(name, type, &data, /*modified*/false);
+			delete data; // If not consumed by setThisFromDbt
+		}
+		catch(...) {
+			delete data; // If not consumed by setThisFromDbt
+			throw;
+		}
 
 		if(err == DB_NOTFOUND) {
 			// Add a dummy MetaDatum, to avoid doing the database lookup again
@@ -368,13 +442,6 @@ void Document::clearModified(const Name &name)
 void Document::setContentAsDbt(DbtOut **value, bool setOnly) // Note: Consumes Dbt.
 {
 
-	if (!setOnly && isNodeDocument() && oldNsDocument_) {
-		delete *value;
-		// cannot safely reset DOM twice!
-		throw XmlException(
-			XmlException::INVALID_VALUE,
-			"Cannot setContentAsDbt this node storage document");
-	}
 	if(*value != dbtContent_) {
 		resetContentAsDbt();
 		dbtContent_ = *value;
@@ -388,7 +455,8 @@ void Document::setContentAsDbt(DbtOut **value, bool setOnly) // Note: Consumes D
 	contentModified_ = true;
 
 	resetContentAsInputStream();
-	resetContentAsDOM(true);
+	resetContentAsDOM();
+	resetContentAsEventReader();
 }
 
 const Dbt *Document::getContentAsDbt() const
@@ -397,6 +465,7 @@ const Dbt *Document::getContentAsDbt() const
 	case DBT: break;
 	case INPUTSTREAM: stream2dbt(); break;
 	case DOM: dom2dbt(); break;
+	case READER: reader2dbt(); break;
 	case NONE: id2dbt(); break;
 	}
 
@@ -411,13 +480,6 @@ void Document::resetContentAsDbt() const
 
 void Document::setContentAsInputStream(XmlInputStream **adopted_str)
 {
-	if (isNodeDocument() && oldNsDocument_) {
-		delete *adopted_str;
-		// cannot safely reset DOM twice!
-		throw XmlException(
-			XmlException::INVALID_VALUE,
-			"Cannot setContentAsDbt this node storage document");
-	}
 	resetContentAsInputStream();
 	inputStream_ = *adopted_str;
 	*adopted_str = 0;
@@ -426,7 +488,21 @@ void Document::setContentAsInputStream(XmlInputStream **adopted_str)
 	contentModified_ = true;
 
 	resetContentAsDbt();
-	resetContentAsDOM(true);
+	resetContentAsDOM();
+	resetContentAsEventReader();
+}
+
+void Document::setContentAsEventReader(XmlEventReader &reader)
+{
+	resetContentAsEventReader();
+	reader_ = &reader;
+
+	definitiveContent_ = READER;
+	contentModified_ = true;
+
+	resetContentAsDbt();
+	resetContentAsDOM();
+	resetContentAsInputStream();
 }
 
 // the object is donated to caller.  This is because
@@ -438,6 +514,7 @@ XmlInputStream *Document::getContentAsInputStream() const
 	case DBT: dbt2stream(); break;
 	case INPUTSTREAM: break;
 	case DOM: dom2stream(); break;
+	case READER: reader2stream(); break;
 	case NONE: id2stream(); break;
 	}
 
@@ -455,97 +532,97 @@ void Document::resetContentAsInputStream() const
 	inputStream_ = 0;
 }
 
+void Document::resetContentAsEventReader() const
+{
+	if (reader_) {
+		reader_->close();
+		reader_ = 0;
+	}
+}
+
 void Document::createNsObjects() const
 {
 	resetContentAsDOM();
-	nsDomFactory_ = new NsXDOMFactory(getMemoryManager(), true);
-	nsDocument_ = new NsDocument(getMemoryManager());
-	nsDocument_->initDOM(nsDomFactory_);
+	DictionaryDatabase *ddb = (container_ ? container_->getDictionaryDB() :
+				   ((Manager&)mgr_).getDictionary());
+	nsDomFactory_ = new NsXDOMFactory(Globals::defaultMemoryManager);
+	nsDocument_ = new NsDocument(this, Globals::defaultMemoryManager);
+	nsDocument_->initDOM(nsDomFactory_, id_, ddb);
+	nsDocument_->setDocumentUri(documentURI_);
 }
 
 void Document::initNsObjects() const
 {
-	assert(nsDocument_ && container_);
+	DBXML_ASSERT(nsDocument_ && container_);
 	nsDocument_->initDoc(
 		oc_.txn(),
-		(NsDocumentDatabase*)container_->getDocumentDB(),
-		id_, db_flags_, false);
-}
-
-MemoryManager *Document::getMemoryManager() const
-{
-	if(memMgr_ == 0) {
-		memMgr_ = new XPath2MemoryManagerImpl;
-	}
-	return memMgr_;
+		container_->getDocumentDB()->getNsDocumentDatabase(),
+		container_->getDictionaryDB(),
+		id_, db_flags_);
 }
 
 void Document::createBlankDOM() const
 {
 	createNsObjects();
-	NsXercesHandler handler(getMemoryManager(), nsDocument_, nsDomFactory_);
-	handler.startDocument();
-	handler.endDocument();
-	document_ = nsDomFactory_->getDocumentNode();
+	NsTransientDomBuilder builder(Globals::defaultMemoryManager,
+		nsDocument_, nsDomFactory_);
+	builder.startDocument(0);
+	builder.endDocument();
+}
+
+void Document::changeContentToDOM() const
+{
+	switch(definitiveContent_) {
+	case DBT: dbt2dom(oc_.txn()); break;
+	case INPUTSTREAM: stream2dom(lazy_ != NEITHER ? oc_.txn() : 0); break;
+	case DOM: break;
+	case READER:
+	{
+		reader2stream();
+		stream2dom((lazy_ != NEITHER ? oc_.txn() : 0), true);
+		break;
+	}
+	case NONE: id2dom(); break;
+	}
 }
 
 XERCES_CPP_NAMESPACE_QUALIFIER DOMDocument *Document::getContentAsDOM() const
 {
-	switch(definitiveContent_) {
-	case DBT: dbt2dom(lazy_ != NEITHER ? oc_.txn() : 0); break;
-	case INPUTSTREAM: stream2dom(lazy_ != NEITHER ? oc_.txn() : 0); break;
-	case DOM: break;
-	case NONE: id2dom(); break;
-	}
+	changeContentToDOM();
 
-	if(document_ == 0) {
-		// There's no content already set, so create a
-		// blank DOMDocument
+	if(nsDocument_ == 0) {
 		createBlankDOM();
-		definitiveContent_ = DOM;
-		contentModified_ = false;
 	}
 
-	return document_;
+	return nsDomFactory_->getDocumentNode(nsDocument_);
 }
 
-DOMElement *Document::getElement(const nid_t *nid) const
+DOMElement *Document::getElement(const NsNid *nid) const
 {
-	if(container_->getContainerType() != XmlContainer::NodeContainer) return 0;
-	getContentAsDOM();
+	changeContentToDOM();
 	return NsXDOMFactory::
 		CastToDOMElement(nsDocument_->getDomElement(nid));
 }
 
 void Document::setContentAsDOM(XERCES_CPP_NAMESPACE_QUALIFIER DOMDocument *doc)
 {
-	if(doc != document_) {
+	DOMDocument *documentNode = getContentAsDOM();
+	if(doc != documentNode) {
 		DOMNode *child;
-		if(document_ == 0) {
-			createBlankDOM();
-		} else {
-			if (isNodeDocument() && oldNsDocument_) {
-				// cannot safely reset DOM -- it may be referenced
-				// by Query results.
-				throw XmlException(
-					XmlException::INVALID_VALUE,
-					"Cannot setContentAsDOM a valid node storage document");
-			}
-			// Remove our document's children
-			while((child = document_->getFirstChild()) != 0) {
-				document_->removeChild(child);
-				child->release();
-			}
+		// Remove our document's children
+		while((child = documentNode->getFirstChild()) != 0) {
+			documentNode->removeChild(child);
+			child->release();
 		}
 
 		child = doc->getFirstChild();
 		while(child != 0) {
 			// allow importNode to fail with null for
 			// certain nodes that don't affect the info set
-			DOMNode *cnode = document_->importNode(child,
-							       /*deep*/true);
+			DOMNode *cnode = documentNode->
+				importNode(child, /*deep*/true);
 			if (cnode)
-				document_->appendChild(cnode);
+				documentNode->appendChild(cnode);
 			child = child->getNextSibling();
 		}
 	}
@@ -557,39 +634,161 @@ void Document::setContentAsDOM(XERCES_CPP_NAMESPACE_QUALIFIER DOMDocument *doc)
 	resetContentAsInputStream();
 }
 
-void Document::resetContentAsDOM(bool saveOldDoc) const
+void Document::domModified() const
 {
-	if (saveOldDoc && nsDocument_) {
-		oldNsDocument_ = nsDocument_;
-	} else {
-		if (nsDocument_)
-			delete nsDocument_; // must precede nsDomFactory_ del
-		if (nsDomFactory_)
-			delete nsDomFactory_;
-		delete memMgr_;
-	}
-	nsDocument_ = 0;
-	nsDomFactory_ = 0;
-	memMgr_ = 0;
-	document_ = 0;
+	definitiveContent_ = DOM;
+	contentModified_ = true;
+
+	resetContentAsDbt();
+	resetContentAsInputStream();
 }
 
-NsEventSource8 *Document::getContentAsSAX(
+void Document::resetContentAsDOM() const
+{
+	if (nsDocument_)
+		delete nsDocument_; // must precede nsDomFactory_ del
+	if (nsDomFactory_)
+		delete nsDomFactory_;
+	nsDocument_ = 0;
+	nsDomFactory_ = 0;
+}
+
+EventReader *Document::getElementAsReader(DOMElement *element) const
+{
+	NsDomElement *el = (NsDomElement*)((NsDomNode*)element->
+		getInterface(_nsDomString));
+
+	if(container_ == 0 ||
+		container_->getContainerType() == XmlContainer::WholedocContainer ||
+		isContentModified()) {
+		return new NsDomReader(el);
+	} else {
+		DBXML_ASSERT(el->getNsDocument()->getDocId() == id_);
+		// Otherwise get the data directly from the database
+		return new NsEventReader(
+			oc_.txn(),
+			container_->getDocumentDB()->getNsDocumentDatabase(),
+			container_->getDictionaryDB(),
+			id_, db_flags_, Globals::defaultMemoryManager, el->getNodeId());
+	}
+}
+
+NsPushEventSource *Document::getContentAsEventSource(
 	Transaction *txn,
 	bool needsValidation,
-	NsEventTranslator *adopted_translator) const
+	bool nodeEvents,
+	bool useID) const
 {
-	switch(definitiveContent_) {
-	case DBT: return dbt2sax(txn, needsValidation, adopted_translator);
-	case INPUTSTREAM: return stream2sax(txn,
-					    needsValidation,
-					    adopted_translator);
-	case DOM: return dom2sax(txn, needsValidation, adopted_translator);
-	case NONE: return id2sax(needsValidation, adopted_translator);
+	Content src = (useID ? NONE : definitiveContent_);
+	switch (src) {
+	case DBT: return dbt2events(txn, needsValidation, nodeEvents);
+	case INPUTSTREAM: return stream2events(txn, needsValidation,
+					       nodeEvents);
+	case DOM: return dom2events(txn, needsValidation, nodeEvents);
+	case READER:
+	{
+		DBXML_ASSERT(reader_);
+		EventReaderToWriter *r2w = new EventReaderToWriter(
+			*reader_,
+			true, //owned
+			false); //not internal
+		reader_ = 0;
+		definitiveContent_ = NONE;
+		return r2w;
+	}
+	case NONE: return id2events(txn, needsValidation, nodeEvents);
 	}
 	return 0;
 }
 
+//
+// Event translations...
+//
+XmlEventReader &Document::getContentAsEventReader() const
+{
+	switch(definitiveContent_) {
+	case DBT: dbt2reader(); break;
+	case INPUTSTREAM: stream2reader(); break;
+	case DOM: dom2reader(); break;
+	case READER: break;
+	case NONE: id2reader(); break;
+	}
+
+	XmlEventReader *ret = reader_;
+	reader_ = 0;
+	if(definitiveContent_ == READER) {
+		definitiveContent_ = NONE;
+	}
+	DBXML_ASSERT(ret);
+	return *ret;
+}
+
+// write content to the writer, then close it
+void Document::getContentAsEventWriter(XmlEventWriter &writer)
+{
+	// DBT or stream: parse directly to writer
+	// DOM or READER: get XmlEventReader, use EventReaderToWriter.
+	switch(definitiveContent_) {
+	case DOM:
+		dom2reader();
+		break;
+	case READER:
+		break;
+	case NONE:
+		if(container_->getContainerType() ==
+		   XmlContainer::NodeContainer)
+			id2reader();
+		break;
+#if 1
+        // TBD: see below -- try to avoid intermediate reader
+        // by writing parse events directly to the writer
+	case DBT: dbt2reader(); break;
+	case INPUTSTREAM: stream2reader(); break;
+#endif		
+	default:
+		break;
+	}
+	XmlEventReader *reader = reader_;
+	if (reader)  {
+		reader_ = 0;
+		if(definitiveContent_ == READER)
+			definitiveContent_ = NONE;
+		XmlEventReaderToWriter r2w(*reader, writer, true);
+		r2w.start();
+	} else {
+		// TBD: implement this -- it allows
+		// the parse events to be written directly to the
+		// writer, rather than using an intermediate reader
+		// need to parse to events
+		switch(definitiveContent_) {
+		case DBT:
+			dbt2stream();
+			break;
+		case INPUTSTREAM:
+			break;
+		case NONE:
+			id2stream();
+			break;
+		default:
+			DBXML_ASSERT(0);
+			break;
+		}
+		// parse stream into writer
+		DBXML_ASSERT(inputStream_ != 0);
+		ScopedPtr<NsPushEventSource>
+			pushSource(stream2events(
+					   (lazy_ != NEITHER ? oc_.txn() : 0),
+					   false, // needs validation
+					   false)); // nodeEvents
+#if 0
+		// GMF TBD
+		NsEventHandlerToWriter h2w(writer);
+		pushSource->setNsEventHandler(&h2w);
+		pushSource->start();
+#endif		
+	}
+}
+
 bool Document::isContentModified() const
 {
 	return contentModified_;
@@ -611,33 +810,43 @@ void Document::id2dbt() const
 	}
 }
 
-void Document::id2dom() const
+void Document::id2dom(bool includeEntityInfo) const
 {
-	if(lazy_ != NEITHER && document_ == 0) {
+	if(lazy_ != NEITHER && nsDocument_ == 0) {
 		if(container_->getContainerType() == XmlContainer::NodeContainer) {
 			createNsObjects();
 			initNsObjects();
-			if (nsDocument_->hasContent()) {
-				nsDocument_->getDocumentNode();
-				document_ = nsDomFactory_->getDocumentNode();
-				definitiveContent_ = DOM;
-				contentModified_ = false;
-			}
+			definitiveContent_ = DOM;
+			contentModified_ = false;
 		} else {
 			id2dbt();
 			dbt2stream();
-			stream2dom(oc_.txn());
+			stream2dom(oc_.txn(), includeEntityInfo);
 		}
 	}
 }
 
 void Document::id2stream() const
 {
-	// TBD write a stream output from a NLS
-	// document to an InputStream - jpcs [#11037]
 	if(lazy_ != NEITHER && inputStream_ == 0) {
-		id2dbt();
-		dbt2stream();
+		if(container_->getContainerType() == XmlContainer::NodeContainer) {
+			resetContentAsInputStream();
+			inputStream_ = new PullEventInputStream(
+				new EventReaderToWriter(
+					*(new NsEventReader(
+					  oc_.txn(),
+					  container_->getDocumentDB()->
+					  getNsDocumentDatabase(),
+					  container_->getDictionaryDB(),
+					  id_, db_flags_,
+					  Globals::defaultMemoryManager)),
+					true,
+					true /* owns reader */)
+				);
+		} else {
+			id2dbt();
+			dbt2stream();
+		}
 	}
 }
 
@@ -648,9 +857,6 @@ void Document::dbt2stream() const
 			return; // no-content doc
 		resetContentAsInputStream();
 
-		// We pass in this, so that the MemBufInputStream
-		// can hold a reference to us. This way, the document
-		// won't be deleted before the stream has been deleted.
 		inputStream_ =
 			new MemBufInputStream(
 				(const char*)dbtContent_->get_data(),
@@ -659,33 +865,34 @@ void Document::dbt2stream() const
 	}
 }
 
-void Document::dbt2dom(Transaction *txn) const
+void Document::dbt2dom(Transaction *txn, bool includeEntityInfo) const
 {
 	dbt2stream();
-	stream2dom(txn);
+	stream2dom(txn, includeEntityInfo);
 }
 
 void Document::dom2dbt() const
 {
 	if(dbtContent_ == 0) {
-		assert(document_);
+		DBXML_ASSERT(nsDocument_);
 		// if the user has modified the DOM, in place,
 		// and called setContentAsDOM(), it will be noted
 		// as contentModified_, so generate output from the DOM.
 		// Otherwise, generate it directly from the database
 		if(lazy_ == NEITHER || isContentModified()) {
 			// Make output writer
-			BufferNsStream8 output;
-			NsWriter writer;
-			writer.setStream(&output);
+			BufferNsStream output;
+			NsWriter writer(&output);
 
 			// Make event generator
-			NsDomEventSource8 event_gen(nsDocument_->getDocumentNode());
-			event_gen.setNsEventHandler(&writer);
-			event_gen.start();
+			NsDomReader reader(nsDocument_->getDocumentNode());
+			writer.writeFromReader(reader);
 
 			resetContentAsDbt();
-			dbtContent_ = new DbtOut(output.buffer.getBuffer(), output.buffer.getOccupancy());
+			// directly donate memory from Buffer to DbtOut
+			dbtContent_ = new DbtOut();
+			dbtContent_->setNoCopy(output.buffer.donateBuffer(),
+					       output.buffer.getOccupancy());
 		} else {
 			id2dbt();
 		}
@@ -696,28 +903,61 @@ void Document::dom2dbt() const
 
 void Document::dom2stream() const
 {
-	dom2dbt();
-	dbt2stream();
+	if(inputStream_ == 0) {
+		DBXML_ASSERT(nsDocument_);
+
+		// if the user has modified the DOM, in place,
+		// and called setContentAsDOM(), it will be noted
+		// as contentModified_, so generate output from the DOM.
+		// Otherwise, generate it directly from the database
+		if(lazy_ == NEITHER || isContentModified()) {
+			resetContentAsInputStream();
+			inputStream_ = new PullEventInputStream(
+				new EventReaderToWriter(
+					*(new NsDomReader(
+						  nsDocument_->getDocumentNode())),
+					true, // internal
+					true /* owns reader */));
+		} else {
+			id2stream();
+		}
+	}
 }
 
-void Document::stream2dom(Transaction *txn) const
+void Document::stream2dom(Transaction *txn, bool includeEntityInfo) const
 {
-	if(document_ == 0) {
-		assert(inputStream_);
+	if(nsDocument_ == 0) {
+		if(!inputStream_) // no content doc...
+			return;
 
-		// NLS DOM
+		//
+		// This is the path used for creating DOM for querying
+		// for wholedoc containers, and also for node containers
+		// if the current document's definitive format is DBT or
+		// stream.
+		//
 		createNsObjects();
+		// TBD: figure out if this is for query, and pass that
+		// info along, to suppress attempts to add to dictionary.
+		// For now, false is safe.
+		nsDocument_->initDOMParse(txn, db_flags_, false /* TBD */);
 		try {
 			unsigned int pflags = 0;
-			if (!validate_)
+			if (validation_ == WF_ONLY)
+				pflags |=
+				  NsParserEventSource::NS_PARSER_WF_ONLY;
+			else if (validation_ != VALIDATE)
 				pflags |=
 				  NsParserEventSource::NS_PARSER_WELL_FORMED;
 			NsSAX2Reader parser(mgr_, txn, pflags);
-			NsXercesHandler handler(getMemoryManager(), nsDocument_, nsDomFactory_);
-			parser.setNsEventHandler(&handler);
+			NsTransientDomBuilder builder(
+				Globals::defaultMemoryManager,
+				nsDocument_,
+				nsDomFactory_,
+				includeEntityInfo);
+			parser.setNsEventHandler(&builder);
 
 			parser.parse(&inputStream_);
-			document_ = nsDomFactory_->getDocumentNode();
 		} catch (XMLException &e) {
 			// shouldn't happen -- parser catches Xerces exceptions
 			throw XmlException(XmlException::DOM_PARSER_ERROR,
@@ -737,7 +977,7 @@ void Document::stream2dom(Transaction *txn) const
 void Document::stream2dbt() const
 {
 	if(dbtContent_ == 0) {
-		assert(inputStream_);
+		DBXML_ASSERT(inputStream_);
 
 		char tmp[TMP_BUF_SIZE];
 		Buffer buffer(0, 16 * 1024); // 16k buffer, to start
@@ -760,70 +1000,81 @@ void Document::stream2dbt() const
 	}
 }
 
-NsEventSource8 *Document::id2sax(bool needsValidation,
-				 NsEventTranslator *adopted_translator) const
+NsPushEventSource *Document::id2events(Transaction *txn, bool needsValidation,
+				       bool nodeEvents) const
 {
-	NsEventSource8 *result = 0;
+	NsPushEventSource *result = 0;
 	if(lazy_ != NEITHER) {
-		if(container_->getContainerType() == XmlContainer::NodeContainer &&
-		   adopted_translator == 0) {
-			result = new
-				NsEventGenerator(oc_.txn(), (NsDocumentDatabase*)
-						 container_->getDocumentDB(),
-						 id_, db_flags_, getMemoryManager());
+		if(container_->getContainerType() == XmlContainer::NodeContainer
+		   && !needsValidation) {
+			NsDocumentDatabase *ddb = container_->getDocumentDB()->
+				getNsDocumentDatabase();
+			result = new EventReaderToWriter(
+				*(new NsEventReader(
+					  txn, ddb,
+					  container_->getDictionaryDB(),
+					  id_, db_flags_,
+					  Globals::defaultMemoryManager)),
+				true, // internal
+				true /* owns reader */);
 		} else {
-			id2dbt();
-			dbt2stream();
-			return stream2sax(oc_.txn(), needsValidation,
-					  adopted_translator);
+			id2stream();
+			return stream2events(txn, needsValidation,
+					     nodeEvents);
 		}
 	}
 	return result;
 }
 
-NsEventSource8 *Document::dbt2sax(Transaction *txn,
-				  bool needsValidation,
-				  NsEventTranslator *adopted_translator) const
+NsPushEventSource *Document::dbt2events(Transaction *txn, bool needsValidation,
+					bool nodeEvents) const 
 {
 	dbt2stream();
-	return stream2sax(txn, needsValidation, adopted_translator);
+	return stream2events(txn, needsValidation, nodeEvents);
 }
 
-NsEventSource8 *Document::dom2sax(Transaction *txn,
-				  bool needsValidation,
-				  NsEventTranslator *adopted_translator) const
+NsPushEventSource *Document::dom2events(Transaction *txn, bool needsValidation,
+					bool nodeEvents) const 
 {
-	assert(document_);
-	assert(nsDocument_);
-	// if adopted_translater is set, must parse the document,
+	DBXML_ASSERT(nsDocument_);
+	// may need to parse the document
 	// in order to (1) validate if necessary, and (2) get it
 	// into a node storage container.
-	if (adopted_translator == 0) {
-		return new NsDomEventSource8(nsDocument_->getDocumentNode());
+	if (!needsValidation) {
+		return 	new EventReaderToWriter(
+			*(new NsDomReader(
+				  nsDocument_->getDocumentNode())),
+			true, //internal
+			true /* owns reader */);
 	} else {
-		dom2dbt();
-		dbt2stream();
-		return stream2sax(txn, needsValidation, adopted_translator);
+		dom2stream();
+		return stream2events(txn, needsValidation, nodeEvents);
 	}
 }
 
-NsEventSource8 *Document::stream2sax(Transaction *txn,
-				     bool needsValidation,
-				     NsEventTranslator *adopted_translator) const
+NsPushEventSource *Document::stream2events(Transaction *txn,
+					   bool needsValidation,
+					   bool nodeEvents) const
 {
-	if (!inputStream_) {
-		// delete donated translator
-		if (adopted_translator)
-			delete adopted_translator;
-		return 0; // no-content doc
-	}
+	if (!inputStream_) return 0; // no-content doc
 
-	if(adopted_translator == 0) {
-		adopted_translator = new NsXercesIndexer();
+	NsDocumentDatabase *ddb = 0;
+	if (container_)
+		ddb = container_->getDocumentDB()->
+			getNsDocumentDatabase();
+	NsEventTranslator *translator;
+	if(nodeEvents || ddb != 0) {
+		DBXML_ASSERT(container_);
+		translator = new NsXercesTranscoder(txn, container_, id_, 0,
+						    Globals::defaultMemoryManager);
+	} else {
+		translator = new NsXercesIndexer();
 	}
 
 	unsigned int pflags = 0;
-	if (!validate_)
+	if (validation_ == WF_ONLY)
+		pflags |= NsParserEventSource::NS_PARSER_WF_ONLY;
+	else if (validation_ == DONT_VALIDATE)
 		pflags |= NsParserEventSource::NS_PARSER_WELL_FORMED;
 	else if (needsValidation)
 		pflags |= NsParserEventSource::NS_PARSER_MUST_VALIDATE;
@@ -834,9 +1085,85 @@ NsEventSource8 *Document::stream2sax(Transaction *txn,
 		definitiveContent_ = NONE;
 	}
 
-	return new NsEventSourceTranslator8(pes, adopted_translator);
+	return new NsPushEventSourceTranslator(pes, translator);
 }
 
+// XmlEventReader methods
+
+//
+// Generate an XmlEventReader from the id
+//
+void Document::id2reader() const
+{
+	if(lazy_ != NEITHER && reader_ == 0) {
+		DBXML_ASSERT(container_);
+		if(container_->getContainerType() ==
+		   XmlContainer::NodeContainer) {
+			reader_ = new NsEventReader(
+				oc_.txn(),
+				container_->getDocumentDB()->
+				getNsDocumentDatabase(),
+				container_->getDictionaryDB(),
+				id_, db_flags_,
+				Globals::defaultMemoryManager);
+		} else {
+			// wholedoc requires parsing into dom, which serves
+			// as the source for the events to be read
+			// The returned reader is not part of the document object
+			id2dom(true);
+			dom2reader();
+		}
+	}
+}
+
+void Document::dbt2reader() const
+{
+	// txn may be needed to lookup IDs in dictionary w/o deadlock
+	dbt2dom(oc_.txn(), true);
+	dom2reader();
+}
+
+void Document::dom2reader() const
+{
+	DBXML_ASSERT(nsDocument_);
+	reader_ = new NsDomReader(nsDocument_->getDocumentNode());
+}
+
+void Document::stream2reader() const
+{
+	stream2dom(oc_.txn(), true);
+	dom2reader();
+}
+
+void Document::reader2stream() const
+{
+	DBXML_ASSERT(reader_);
+	resetContentAsInputStream();
+
+	inputStream_ = new PullEventInputStream(
+		new EventReaderToWriter(*reader_, true, true));
+	if (definitiveContent_ == READER)
+		definitiveContent_ = INPUTSTREAM;
+	reader_ = 0;
+}
+
+void Document::reader2dbt() const
+{
+	DBXML_ASSERT(reader_);
+	// write from reader to NsWriter
+	BufferNsStream output;
+	NsWriter writer(&output);
+	EventReaderToWriter r2e(*reader_, writer, true);
+	reader_ = 0;
+	resetContentAsDbt();
+	r2e.start();
+	dbtContent_ = new DbtOut();
+	dbtContent_->setNoCopy(output.buffer.donateBuffer(),
+			       output.buffer.getOccupancy());
+	if (definitiveContent_ == READER)
+		definitiveContent_ = DBT;
+}
+	
 MetaData::const_iterator Document::metaDataBegin() const {
 	return metaData_.begin();
 }
@@ -863,7 +1190,7 @@ bool MetaDataIterator::next(std::string &uri, std::string &name, XmlValue &value
 	if(i_ == document_->metaDataEnd()) {
 		uri= "";
 		name= "";
-		value = Value::create();
+		value = XmlValue();
 		return false;
 	}
 	else {
