@@ -1,24 +1,25 @@
 //
 // See the file LICENSE for redistribution information.
 //
-// Copyright (c) 2002-2005
-//	Sleepycat Software.  All rights reserved.
+// Copyright (c) 2002,2006 Oracle.  All rights reserved.
 //
-// $Id: DocumentDatabase.cpp,v 1.22 2005/04/05 16:44:00 bostic Exp $
+// $Id: DocumentDatabase.cpp,v 1.47.2.1.2.1 2008/04/08 15:06:10 gmf Exp $
 //
 
 #include <sstream>
 
 #include <dbxml/XmlException.hpp>
-#include "DocumentDatabase.hpp"
-#include "ScopedDbt.hpp"
 #include "Container.hpp"
+#include "DocumentDatabase.hpp"
+#include "Manager.hpp"
+#include "ScopedDbt.hpp"
 #include "OperationContext.hpp"
 #include "Document.hpp"
 #include "Cursor.hpp"
 #include "UpdateContext.hpp"
-#include "nodeStore/NsXercesIndexer.hpp"
+#include "NameID.hpp"
 #include "db_utils.h"
+#include "nodeStore/NsEvent.hpp"
 
 using namespace DbXml;
 using namespace std;
@@ -30,34 +31,62 @@ DocumentDatabase::DocumentDatabase(DbEnv *env, Transaction *txn,
 				   u_int32_t pageSize, u_int32_t flags,
 				   int mode)
 	: environment_(env),
-	  containerName_(name),
+	  name_(name),
+	  type_(type),
 	  content_(env, name, "content_", document_name,
 		   pageSize, flags & DB_XA_CREATE),
-	  secondary_(env, name, document_name, 0, pageSize,
-		     flags & DB_XA_CREATE),
-	  type_(type)
+	  secondary_(env, name, document_name, pageSize,
+		     flags & DB_XA_CREATE)
 {
 	flags &= ~DB_XA_CREATE;
+	open(txn, flags, mode);
+}
+
+DocumentDatabase::DocumentDatabase(DbEnv *env, const std::string &name,
+				   XmlContainer::ContainerType type)
+	: environment_(env),
+	  name_(name),
+	  type_(type),
+	  content_(env, name, "content_", document_name, 0, 0),
+	  secondary_(env, name, document_name, 0, 0)
+{
+	open(0, 0, 0);
+}
+
+void
+DocumentDatabase::open(Transaction *txn, u_int32_t flags, int mode)
+{
 	int err = 0;
 	try {
 		// Open the Db objects
-		if (type == XmlContainer::WholedocContainer) {
+		if (type_ == XmlContainer::WholedocContainer) {
 			err = content_.open(txn, DB_BTREE, flags, mode);
 		}
 		if (err == 0)
-			err = secondary_.open(txn, false, flags, mode);
+			err = secondary_.open(txn, /*duplicates*/false,
+					      flags, mode);
 	} catch (DbException &e) {
-		if(e.get_errno() == EEXIST) {
-			throw XmlException(XmlException::CONTAINER_EXISTS,
-					   e.what());
-		} else {
-			throw XmlException(e);
-		}
+		err = e.get_errno();
+		// fall through
 	}
-	if(err == EEXIST) {
-		throw XmlException(XmlException::CONTAINER_EXISTS,
-				   db_strerror(err));
-	} else if(err != 0) {
+	catch (XmlException &xe) {
+		if (txn)
+			txn->abort();
+		throw;
+	}
+	if (err != 0) {
+		if (txn)
+			txn->abort();
+		string msg = name_;
+		if (err == EEXIST) {
+			msg += ": container exists";
+			throw XmlException(
+				XmlException::CONTAINER_EXISTS, msg);
+		} else if (err == ENOENT) {
+			msg += ": container file not found, or not a container";
+			throw XmlException(XmlException::CONTAINER_NOT_FOUND,
+					   msg);
+		}
 		throw XmlException(err);
 	}
 }
@@ -77,8 +106,8 @@ int DocumentDatabase::getContent(OperationContext &context,
 		document->getID().setDbtFromThis(context.key());
 		// NOTE: db_.get() does not throw on DB_NOTFOUND, but
 		// try/catch anyway...
-		err = content_.getDb().get(context.db_txn(),
-					   &context.key(), data, flags);
+		err = content_.get(context.txn(),
+			&context.key(), data, flags);
 	} catch (...) {
 		delete data;
 		throw; // re-throw
@@ -96,10 +125,10 @@ int DocumentDatabase::getContent(OperationContext &context,
 
 // object is donated to caller
 int DocumentDatabase::createDocumentCursor(
-	Transaction *txn, scoped_ptr<DocumentCursor> &cursor) const
+	Transaction *txn, ScopedPtr<DocumentCursor> &cursor, u_int32_t flags) const
 {
 	DocumentDatabaseCursor *newCursor =
-		new DocumentDatabaseCursor(txn, content_);
+		new DocumentDatabaseCursor(txn, content_, flags);
 	cursor.reset(newCursor);
 	return 0;
 }
@@ -117,46 +146,24 @@ unsigned long DocumentDatabase::getNumberOfPages() const
 // Remove the ID -- cannot "remove" an ID.  They are not
 // reusable, but implement the call, just in case that ever
 // changes.
-int DocumentDatabase::deleteID(OperationContext &context, const ID& oldId)
+int DocumentDatabase::deleteID(OperationContext &context, const DocID& oldId)
 {
 	return 0; // this is a no-op; IDs are never reused
 }
 
-int DocumentDatabase::addContentAndIndex(Document &document,
-					 UpdateContext &context,
-					 Container &container,
-					 KeyStash &stash)
+
+int DocumentDatabase::addContent(Document &document, UpdateContext &context)
 {
-	int err = 0;
 	OperationContext &oc = context.getOperationContext();
-
-	// If this document's content is a stream, convert it to a dbt,
-	// because streams can only be used once... - jpcs
-	if(document.getDefinitiveContent() == Document::INPUTSTREAM) {
-		document.getContentAsDbt();
-	}
-
-	// Generate a unique ID for the Document, if not done by caller
-	if (document.getID() == 0) {
-		err = container.getConfigurationDB()->
-			generateID(oc.txn(), document.getIDToSet());
-		if (err)
-			return err;
-	}
 	
-	// Index the document
-	context.getIndexer().indexDocument(context.getIndexSpecification(),
-					   document, true, stash);
-
 	Dbt *dbt = (Dbt*) document.getContentAsDbt();
-	// Allow no-content documents; they have no entry in
-	// the content database
+	// Allow no-content documents
+	int err = 0;
 	if (dbt && (dbt->get_size() != 0)) {
 		document.getID().setDbtFromThis(oc.key());
-		err = content_.getDb().put(oc.db_txn(), &oc.key(),
-					   dbt, 0);
+		err = content_.put(oc.txn(), &oc.key(),
+			dbt, 0);
 	}
-	if(err == 0) document.setContentModified(false);
 	return err;
 }
 
@@ -165,7 +172,7 @@ int DocumentDatabase::updateContentAndIndex(Document &new_document,
 					    KeyStash &stash)
 {
 	OperationContext &oc = context.getOperationContext();
-	ID id = new_document.getID();
+	DocID id = new_document.getID();
 	Indexer &indexer = context.getIndexer();
 	IndexSpecification &index = context.getIndexSpecification();
 	int err = 0;
@@ -176,8 +183,7 @@ int DocumentDatabase::updateContentAndIndex(Document &new_document,
 	// Check to see if the old document exists, first
 	// If ID is non-zero, let's trust it.  If not, get by name.
 	// Retrieve the old document
-	XmlDocument old_document = indexer.getContainer()->
-		getManager().createDocument();
+	XmlDocument old_document;
 
 	if (id == 0) {
 		// will throw if name is bad or doc doesn't exist
@@ -206,9 +212,17 @@ int DocumentDatabase::updateContentAndIndex(Document &new_document,
 	// Index the new document
 	indexer.indexMetaData(index, new_document, stash, true);
 	if(new_document.isContentModified()) {
-		scoped_ptr<NsEventSource8> source(new_document.getContentAsSAX(oc.txn(), true));
-		indexer.indexContent(index, id,
-				     source.get(), stash);
+		ScopedPtr<NsPushEventSource>
+			source(new_document.
+			       getContentAsEventSource(
+				       oc.txn(), /*needsValidation*/true,
+				       indexer.getContainer()->getIndexNodes()));
+		if (source.get()) {
+			indexer.initIndexContent(index, id,
+						 source.get(), stash,
+						 true);
+			source.get()->start();
+		}
 	}
 
 	// Set the modified flags of the old document to the same as the
@@ -229,18 +243,25 @@ int DocumentDatabase::updateContentAndIndex(Document &new_document,
 	delete_index.set(Index::INDEXER_DELETE);
 	indexer.indexMetaData(delete_index, old_document, stash, true);
 	if(new_document.isContentModified()) {
-		scoped_ptr<NsEventSource8>
-			source(((Document&)old_document).getContentAsSAX(oc.txn(), false));
-		indexer.indexContent(delete_index, id,
-				     source.get(), stash);
+		ScopedPtr<NsPushEventSource>
+			source(((Document&)old_document).
+			       getContentAsEventSource(
+				       oc.txn(),
+				       /*needsValidation*/false,
+				       indexer.getContainer()->getIndexNodes()));
+		if (source.get()) {
+			indexer.initIndexContent(delete_index, id,
+						 source.get(), stash,
+						 false);
+			source.get()->start();
+		}
 	}
 
 	// Update the content
 	if(new_document.isContentModified()) {
 		OperationContext &oc = context.getOperationContext();
 		id.setDbtFromThis(oc.key());
-		err = content_.getDb().put(
-			oc.db_txn(), &oc.key(),
+		err = content_.put(oc.txn(), &oc.key(),
 			(Dbt*)new_document.getContentAsDbt(), 0);
 	}
 
@@ -254,20 +275,55 @@ int DocumentDatabase::removeContentAndIndex(const Document &document,
 					    UpdateContext &context,
 					    KeyStash &stash)
 {
+	Indexer &indexer = context.getIndexer();
+	OperationContext &oc = context.getOperationContext();
+
 	// Index the document
 	IndexSpecification &index = context.getIndexSpecification();
 	index.set(Index::INDEXER_DELETE);
-	context.getIndexer().indexDocument(index, document, false, stash);
+
+	indexer.indexMetaData(index, document, stash, /*checkModified*/false);
+	ScopedPtr<NsPushEventSource>
+		source(document.getContentAsEventSource(
+			       oc.txn(), /*needsValidation*/false,
+			       indexer.getContainer()->getIndexNodes()));
+	if (source.get()) {
+		indexer.initIndexContent(index, document.getID(),
+					 source.get(), stash,
+					 false);
+		source.get()->start();
+	}
 
 	// Delete the content
-	OperationContext &oc = context.getOperationContext();
 	deleteID(oc, document.getID()); // a no-op
 	document.getID().setDbtFromThis(oc.key());
-	int err = content_.getDb().del(oc.db_txn(), &oc.key(), 0);
+	int err = content_.del(oc.txn(), &oc.key(), 0);
+	if (err == DB_NOTFOUND)
+		err = 0; // no-content doc
 	
 	return err;
 }
 
+int DocumentDatabase::reindex(const IndexSpecification &is, const Document &document,
+	UpdateContext &context, KeyStash &stash)
+{
+	Indexer &indexer = context.getIndexer();
+	OperationContext &oc = context.getOperationContext();
+
+	// Index the document
+	indexer.indexMetaData(is, document, stash, /*checkModified*/false);
+	ScopedPtr<NsPushEventSource> source(document.
+		getContentAsEventSource(oc.txn(), /*needsValidation*/false,
+					indexer.getContainer()->getIndexNodes()));
+	if (source.get()) {
+		indexer.initIndexContent(is, document.getID(),
+					 source.get(), stash, false);
+		source.get()->start();
+	}
+
+	return 0;
+}
+
 int DocumentDatabase::getAllMetaData(OperationContext &context,
 				     DictionaryDatabase *dictionary,
 				     Document *document,
@@ -275,30 +331,33 @@ int DocumentDatabase::getAllMetaData(OperationContext &context,
 {
 	int err = 0;
 	u_int32_t orig_flags = flags;
-	const ID &did = document->getID();
+	const DocID &did = document->getID();
 
 	//
 	// Read each of the meta-data items from the document secondary
 	// database.  Content is no longer considered metadata
 	//
-	Cursor cursor(environment_,
-		      const_cast<SecondaryDatabase*>(&secondary_)->getDb(),
-		      context.txn(), CURSOR_READ);
+	Cursor cursor(const_cast<SecondaryDatabase&>(secondary_),
+		context.txn(), CURSOR_READ, flags);
+	flags = DB_CURSOR_GET_MASK(flags);
+	orig_flags = DB_CURSOR_GET_MASK(flags);
 	flags |= DB_SET_RANGE;
 	bool done = false;
 	while (!done) {
 		did.setDbtFromThis(context.key());
 		DbtIn none;
 		none.set_flags(DB_DBT_PARTIAL); // Don't pull back the data.
-		err = cursor.get(&context.key(), &none, flags);
+		// if DB_RMW set, don't get locks on this get, wait for a match
+		err = cursor.get(&context.key(), &none, flags & (~DB_RMW));
 		if (err == 0) {
-			ID db_did, db_nid;
+			DocID db_did;
+			NameID db_nid;
 			XmlValue::Type type;
 			MetaDatum::decodeKeyDbt(context.key(), db_did,
 						db_nid, type);
 			if (did == db_did) {
 				Name name;
-				err = dictionary->lookupName(context, db_nid, name);
+				err = dictionary->lookupNameFromID(context, db_nid, name);
 				if(err == 0 && !document->containsMetaData(name)) {
 					DbtOut *data = new DbtOut();
 					err = cursor.get(&context.key(), data,
@@ -324,36 +383,30 @@ int DocumentDatabase::getAllMetaData(OperationContext &context,
 }
 
 int DocumentDatabase::getMetaData(OperationContext &context,
-				  DictionaryDatabase *dictionary,
-				  const Name &name,
-				  const ID &did, Document *document,
-				  u_int32_t flags) const
+	DictionaryDatabase *dictionary, const Name &name,
+	const DocID &did, XmlValue::Type &type, Dbt *metadata,
+	u_int32_t flags) const
 {
-	ID nid;
-	int err = dictionary->lookupName(context, name,
-					 nid, /*define=*/false);
+	NameID nid;
+	int err = dictionary->lookupIDFromName(context, name,
+					       nid, /*define=*/false);
 	if(err == 0) {
-		Cursor cursor(
-			environment_,
-			const_cast<SecondaryDatabase*>(&secondary_)->getDb(),
-			context.txn(), CURSOR_READ);
+		Cursor cursor(const_cast<SecondaryDatabase&>(secondary_),
+			context.txn(), CURSOR_READ, flags);
+		u_int32_t origFlags = DB_CURSOR_GET_MASK(flags);
 		MetaDatum::setKeyDbt(did, nid, XmlValue::NONE, context.key());
 		DbtIn none;
 		none.set_flags(DB_DBT_PARTIAL); // Don't pull back the data.
-		err = cursor.get(&context.key(), &none, flags | DB_SET_RANGE);
+		err = cursor.get(&context.key(), &none,
+				 (flags | DB_SET_RANGE) & ~DB_RMW);
 		if (err == 0) {
-			ID db_did, db_nid;
-			XmlValue::Type type;
+			DocID db_did;
+			NameID db_nid;
 			MetaDatum::decodeKeyDbt(context.key(), db_did,
 						db_nid, type);
 			if(db_did == did && db_nid == nid) {
-				DbtOut *data = new DbtOut();
-				err = cursor.get(&context.key(), data, DB_CURRENT);
-				if(err == 0)
-					document->setMetaData(
-						name, type,
-						&data, /*modified*/false);
-				delete data; // If not consumed by setThisFromDbt
+				err = cursor.get(&context.key(), metadata,
+						 origFlags|DB_CURRENT);
 			} else {
 				return DB_NOTFOUND;
 			}
@@ -371,10 +424,10 @@ int DocumentDatabase::addMetaData(OperationContext &oc,
 	MetaData::const_iterator end = document.metaDataEnd();
 	MetaData::const_iterator i;
 	for (i = document.metaDataBegin(); err == 0 && i != end; ++i) {
-		ID nid;
-		err = dictionary->lookupName(oc,
-					     (*i)->getName(),
-					     nid, /*define=*/true);
+		NameID nid;
+		err = dictionary->lookupIDFromName(oc,
+						   (*i)->getName(),
+						   nid, /*define=*/true);
 		if(err == 0) {
 			DbtIn value;
 			MetaDatum::setKeyDbt(document.getID(),
@@ -399,13 +452,13 @@ int DocumentDatabase::updateMetaData(OperationContext &oc,
 	int err = 0;
 	MetaData::const_iterator end = document.metaDataEnd();
 	MetaData::const_iterator i;
-	std::vector<ID> toRemove;
+	std::vector<NameID> toRemove;
 	for(i = document.metaDataBegin(); err == 0 && i != end; ++i) {
 		if((*i)->isModified()) {
-			ID nid;
-			err = dictionary->lookupName(oc,
-						     (*i)->getName(),
-						     nid, /*define=*/true);
+			NameID nid;
+			err = dictionary->lookupIDFromName(oc,
+							   (*i)->getName(),
+							   nid, /*define=*/true);
 			if(err == 0) {
 				if ((*i)->isRemoved())
 					toRemove.push_back(nid);
@@ -435,9 +488,9 @@ int DocumentDatabase::updateMetaData(OperationContext &oc,
 }
 
 static bool
-idInList(const std::vector<ID> &list, const ID &id)
+idInList(const std::vector<NameID> &list, const NameID &id)
 {
-	std::vector<ID>::const_iterator it = list.begin();
+	std::vector<NameID>::const_iterator it = list.begin();
 	while (it != list.end()) {
 		if (*it == id)
 			return true;
@@ -450,10 +503,10 @@ idInList(const std::vector<ID> &list, const ID &id)
 // if toRemove is non-null, it specifies a list of Name IDs
 // to remove; otherwise remove all metadata for the target document
 int DocumentDatabase::removeMetaData(OperationContext &oc,
-				     const ID &id,
-				     std::vector<ID> *toRemove)
+				     const DocID &id,
+				     std::vector<NameID> *toRemove)
 {
-	Cursor cursor(environment_, secondary_.getDb(), oc.txn(),
+	Cursor cursor(const_cast<SecondaryDatabase&>(secondary_), oc.txn(),
 		      CURSOR_WRITE);
 
 	DbtIn none;
@@ -461,29 +514,34 @@ int DocumentDatabase::removeMetaData(OperationContext &oc,
 
 	id.setDbtFromThis(oc.key());
 
-	ID db_id;
+	DocID db_id;
 	int err = cursor.get(&oc.key(), &none, DB_SET_RANGE);
-	while(err == 0) {
-		if (toRemove) {
-			ID nm_id;
-			XmlValue::Type type;
-			MetaDatum::decodeKeyDbt(oc.key(), db_id, nm_id, type);
-			if ((id == db_id) && idInList(*toRemove, nm_id))
-				cursor.del(0);
-		} else {
-			db_id.setThisFromDbt(oc.key());
-			if (id == db_id)
-				cursor.del(0);
+	try {
+		while(err == 0) {
+			if (toRemove) {
+				NameID nm_id;
+				XmlValue::Type type;
+				MetaDatum::decodeKeyDbt(oc.key(), db_id, nm_id, type);
+				if ((id == db_id) && idInList(*toRemove, nm_id))
+					cursor.del(0);
+			} else {
+				db_id.setThisFromDbt(oc.key());
+				if (id == db_id)
+					cursor.del(0);
+			}
+			if (id != db_id) // done with document?
+				break;
+			err = cursor.get(&oc.key(), &none, DB_NEXT);
 		}
-		if (id != db_id) // done with document?
-			break;
-		err = cursor.get(&oc.key(), &none, DB_NEXT);
+	} catch (...) {
+		cursor.close();
+		throw;
 	}
 
 	if(err == DB_NOTFOUND) {
 		err = 0;
 	}
-
+	cursor.close();
 	return err;
 }
 
@@ -492,8 +550,7 @@ int DocumentDatabase::dump(DbEnv *env, const std::string &name,
 			   std::ostream *out)
 {
 	DbWrapper content(env, name, "content_", document_name, 0, 0);
-	SecondaryDatabase secondary(env, name, document_name, 0,
-				    0, 0);
+	SecondaryDatabase secondary(env, name, document_name, 0, 0);
 	int err = 0;
 	if (type == XmlContainer::WholedocContainer) {
 		err = Container::writeHeader(content.getDatabaseName(), out);
@@ -513,7 +570,7 @@ int DocumentDatabase::load(DbEnv *env, const std::string &name,
 			   std::istream *in, unsigned long *lineno)
 {
 	DbWrapper content(env, name, "content_", document_name, 0, 0);
-	SecondaryDatabase secondary(env, name, document_name, 0, 0, 0);
+	SecondaryDatabase secondary(env, name, document_name, 0, 0);
 
 	int err = 0;
 	if (type == XmlContainer::WholedocContainer) {
@@ -548,7 +605,7 @@ int DocumentDatabase::verify(DbEnv *env, const std::string &name,
 			     std::ostream *out, u_int32_t flags)
 {
 	DbWrapper content(env, name, "content_", document_name, 0, 0);
-	SecondaryDatabase secondary(env, name, document_name, 0, 0, 0);
+	SecondaryDatabase secondary(env, name, document_name, 0, 0);
 
 	int err = 0;
 	if (type == XmlContainer::WholedocContainer) {
@@ -566,25 +623,22 @@ int DocumentDatabase::verify(DbEnv *env, const std::string &name,
 }
 
 DocumentDatabaseCursor::DocumentDatabaseCursor(Transaction *txn,
-					       DbWrapper &db) :
-	cursor_(db.getEnvironment(), db.getDb(), txn, CURSOR_READ),
+					       DbWrapper &db, u_int32_t flags) :
+	cursor_(db, txn, CURSOR_READ, flags),
+	cursorFlags_(DB_CURSOR_GET_MASK(flags)),
 	done_(false) {
 	data_.set_flags(DB_DBT_PARTIAL); // only want keys (for now)
 }
 
-int DocumentDatabaseCursor::next(ID &id) {
+int DocumentDatabaseCursor::next(DocID &id) {
 	int err = 0;
 	if (done_) {
 		id = 0;
 	} else {
-		err = cursor_.get(&key_, &data_, DB_NEXT_NODUP); // no throw
-		if (err == 0) {
-			// skip DbSequence key -- it's the only one
-			// with len > 4. Can only recurse once
-			if (key_.get_size() > sizeof(u_int32_t))
-				return next(id);
+		err = cursor_.get(&key_, &data_,
+				  cursorFlags_|DB_NEXT_NODUP); // no throw
+		if (err == 0)
 			id.setThisFromDbt(key_);
-		}
 		if (err == DB_NOTFOUND || err == DB_KEYEMPTY) {
 			err = 0;
 			done_ = true;
@@ -594,26 +648,181 @@ int DocumentDatabaseCursor::next(ID &id) {
 	return err;
 }
 
-void DocumentDatabase::upgrade(int saved_version, int current_version)
+// upgrade from 2.1 (version 4) to 2.2 (version 5)
+// Modify keys for both document secondary and content (if wholedoc storage)
+// secondary key: docId, nameId, type
+// content key: docId
+// for both:
+// 1.  open old DB and create a new database
+// 2.  for each record:
+//    o unmarshal old format; marshal in new, and put into new DB
+// 3.  remove old DB
+// 4.  rename new DB to correct name
+static int upgrade22(const std::string &name, Manager &mgr)
 {
-	if (saved_version == VERSION_20) {
-		// byte-swap name ids in document metadata keys.
-		// iterate through all records in secondary.
-		// the key is: docId,nodeId
-		Dbt key;
-		Dbt data;
-		Cursor curs(environment_, secondary_.getDb(),
-			    (Transaction*)0, DbXml::CURSOR_WRITE);
-		int ret = 0;
-		while ((ret = curs.get(&key, &data, DB_NEXT)) == 0) {
-			u_int32_t *p = reinterpret_cast<u_int32_t*>(key.get_data());
-			curs.del(0);
-			++p;
-			M_32_SWAP(*p);
-			int putRet = curs.put(&key, &data, DB_KEYFIRST);
-			if (putRet != 0) {
-				std::cerr <<"put returned error: " << putRet << std::endl;
+	int err = 0;
+	SecondaryDatabase secondary(mgr.getDbEnv(), name,
+				    document_name, 0, 0);
+	err = secondary.open(0, /*duplicates*/false, 0, 0);
+	SecondaryDatabase newSec(mgr.getDbEnv(), name,
+				 "newdoc", secondary.getPageSize(), 0);
+	err = newSec.open(0, /*duplicates*/false, DB_CREATE|DB_EXCL, 0);
+	// the key is: docId,nodeId,type
+	Dbt key;
+	Dbt data;
+	Cursor curs(secondary, (Transaction*)0, DbXml::CURSOR_READ);
+	int ret = 0;
+	while ((err == 0) &&
+	       (ret = curs.get(&key, &data, DB_NEXT)) == 0) {
+		// decode key using pre-2.2 code
+		DocID did;
+		NameID nid;
+		XmlValue::Type type;
+		u_int32_t *p = (u_int32_t *)key.get_data();
+		u_int32_t id1, id2;
+		id1 = *p++;
+		id2 = *p++;
+		if (Manager::isBigendian()) {
+			M_32_SWAP(id1);
+			M_32_SWAP(id2);  
+		}
+		did = id1;            
+		nid = id2;
+		type = (XmlValue::Type) * ((unsigned char*)p);
+		// encode using 2.2. code
+		DbtOut newKey;
+		MetaDatum::setKeyDbt(did, nid, type, newKey);
+		err = newSec.put(0, &newKey, &data, 0);
+	}
+	curs.close();
+	secondary.close(0);
+	newSec.close(0);
+	if (err == 0) {
+		// remove old db, rename new one
+		std::string dbname("secondary_");
+		std::string tname("secondary_");
+		dbname += document_name;
+		tname += "newdoc";
+		err = mgr.getDbEnv()->dbremove(0, name.c_str(),
+					       dbname.c_str(), 0);
+		if (err == 0)
+			err = mgr.getDbEnv()->dbrename(0, name.c_str(),
+						       tname.c_str(),
+						       dbname.c_str(), 0);
+		if (err != 0)
+			return err;
+		try {
+			DbWrapper content(mgr.getDbEnv(), name,
+					  "content_",
+					  document_name, 0, 0);
+			err = content.open(0, DB_BTREE, 0, 0);
+			if (err != 0) {
+				// ignore ENOENT
+				if (err == ENOENT)
+					err = 0;
+				return err;
+			}
+			DbWrapper newContent(mgr.getDbEnv(), name,
+					     "newcontent_", document_name,
+					     content.getPageSize(), 0);
+			err = newContent.open(0, DB_BTREE,
+					      DB_CREATE|DB_EXCL, 0);
+			if (err != 0)
+				return err;
+			// the key is: docId
+			Dbt key;
+			Dbt data;
+			Cursor curs1(content, (Transaction*)0, DbXml::CURSOR_READ);
+			int ret = 0;
+			while ((err == 0) &&
+			       (ret = curs1.get(&key, &data, DB_NEXT)) == 0) {
+				// decode/encode key, rewrite
+				u_int32_t *p = (u_int32_t *)key.get_data();
+				u_int32_t id1;
+				id1 = *p;
+				if (Manager::isBigendian())
+					M_32_SWAP(id1);
+				DocID id(id1);
+				DbtOut newKey;
+				id.setDbtFromThis(newKey);
+				err = newContent.put(0, &newKey, &data, 0);
+			}
+			curs1.close();
+			content.close(0);
+			newContent.close(0);
+			if (err == 0) {
+				std::string dbname("content_");
+				std::string tname("newcontent_");
+				dbname += document_name;
+				tname += document_name;
+				// remove old db, rename new one
+				err = mgr.getDbEnv()->dbremove(
+					0, name.c_str(), dbname.c_str(), 0);
+				if (err == 0)
+					err = mgr.getDbEnv()->dbrename(
+						0, name.c_str(), tname.c_str(),
+						dbname.c_str(), 0);
 			}
 		}
+		catch (DbException &de) {} // ignore
 	}
+	return err;
+}
+
+// determine if the name ID needs swapping.
+// If the container was created big-endian, then yes.
+// This is detected by finding *little-endian* values
+// that are very large. Note that this could have issues
+// if there is a *HUGE* number of metadata names, which
+// is very unlikely
+static bool needsSwap(u_int32_t val)
+{
+	bool be = Manager::isBigendian();
+	if (be) {
+		if (val <= 0x00010000)
+			return true;
+	} else {
+		if (val >= 0x00010000)
+			return true;
+	}
+	return false;
+	
+}
+
+// static
+void DocumentDatabase::upgrade(const std::string &name, Manager &mgr,
+			       int old_version, int current_version)
+{
+	int err = 0;
+	DBXML_ASSERT(old_version < current_version);
+	// 2.0 -> 2.1 upgrade, if necessary, must be first
+	if (old_version == VERSION_20){
+		SecondaryDatabase secondary(mgr.getDbEnv(), name,
+					    document_name, 0, 0);
+		err = secondary.open(0, /*duplicates*/false, 0, 0);
+		// byte-swap name ids in document metadata keys.
+		// iterate through all records in secondary.
+		// the key is: docId,nodeId,type
+		Dbt key;
+		Dbt data;
+		Cursor curs(secondary, (Transaction*)0, DbXml::CURSOR_WRITE);
+		int ret = 0;
+		while ((err == 0) &&
+		       (ret = curs.get(&key, &data, DB_NEXT)) == 0) {
+			u_int32_t *p = reinterpret_cast<u_int32_t*>(key.get_data());
+			++p;
+			// swap happens on second int, not first
+			if (!needsSwap(*p))
+				break;
+			M_32_SWAP(*p);
+			curs.del(0);
+			err = curs.put(&key, &data, DB_KEYFIRST);
+		}
+	}
+	if (old_version < VERSION_22 && err == 0)
+		err = upgrade22(name, mgr);
+	if (err != 0)
+		throw XmlException(
+			XmlException::DATABASE_ERROR,
+			"Unexpected error upgrading Document DB");
 }
