@@ -1,10 +1,9 @@
 //
 // See the file LICENSE for redistribution information.
 //
-// Copyright (c) 2002-2005
-//	Sleepycat Software.  All rights reserved.
+// Copyright (c) 2002,2006 Oracle.  All rights reserved.
 //
-// $Id: Globals.hpp,v 1.10 2005/04/05 16:44:01 bostic Exp $
+// $Id: Globals.hpp,v 1.19 2006/10/30 17:45:52 bostic Exp $
 //
 
 #ifndef __GLOBALS_HPP
@@ -15,14 +14,22 @@
 
 #include <xercesc/util/XMLUni.hpp>
 #include <xercesc/framework/MemoryManager.hpp>
-#include <xercesc/validators/datatype/DatatypeValidatorFactory.hpp>
+#include "nodeStore/NsTypes.hpp"
 
 class DbEnv;
+class DatatypeLookup;
+class DocumentCache;
 
 namespace DbXml
 {
 
-class NullValue;
+class SimpleMemoryManager : public XER_NS MemoryManager
+{
+public:
+	SimpleMemoryManager() {}
+	void *allocate(size_t size);
+	void deallocate(void *p);
+};
 
 class Globals
 {
@@ -32,24 +39,17 @@ public:
 	/// Called by Manager::terminate()
 	static void terminate();
 
-	static XERCES_CPP_NAMESPACE_QUALIFIER DatatypeValidatorFactory
-	&getDatatypeValidatorFactory() {
-		return *validatorFactory_;
+	static const DatatypeLookup *getDatatypeLookup() {
+		return datatypeLookup_;
 	}
 
 	static bool isInitialized() { return (refCount_ != 0); }
 
-	/// The singleton null value
-	static NullValue *nullValue;
-
 	/// The map of index names to values
 	typedef std::map<std::string, unsigned long> NameToNumber;
 	static NameToNumber *indexMap;
 
-	static XERCES_CPP_NAMESPACE_QUALIFIER DatatypeValidatorFactory
-	*validatorFactory_;
-	static XERCES_CPP_NAMESPACE_QUALIFIER MemoryManager
-	*defaultMemoryManager_;
+	static XERCESC::MemoryManager *defaultMemoryManager;
 
 	/// Mutex for the globals
 	static void *mutex_;
@@ -61,6 +61,9 @@ public:
 	static bool isBigendian_;
 private:
 	static void initializeXmlPlatform();
+
+	static DocumentCache *documentCache_;
+	static DatatypeLookup *datatypeLookup_;
 };
 
 }
