@@ -1,35 +1,32 @@
 //
 // See the file LICENSE for redistribution information.
 //
-// Copyright (c) 2002-2005
-//	Sleepycat Software.  All rights reserved.
+// Copyright (c) 2002,2006 Oracle.  All rights reserved.
 //
-// $Id: IndexSpecification.cpp,v 1.95 2005/04/20 18:31:24 bostic Exp $
+// $Id: IndexSpecification.cpp,v 1.106.2.1 2006/12/19 15:22:03 gmf Exp $
 //
 
-#include "dbxml_config.h"
-#include "dbxml/XmlPortability.hpp"
+#include "DbXmlInternal.hpp"
 #include "dbxml/XmlException.hpp"
 #include "dbxml/XmlNamespace.hpp"
 #include "IndexSpecification.hpp"
 #include "SyntaxManager.hpp"
 #include "tokenizer.hpp"
 #include "Name.hpp"
-#include "Container.hpp"
 #include "ScopedDbt.hpp"
-#include "Cursor.hpp"
 #include "Value.hpp"
 #include "Globals.hpp"
+#include "Container.hpp"
 
 #include <map>
 #include <algorithm>
 
-using namespace std;
-using namespace DbXml;
-
 const char * DbXml::metaDataName_uri_name = "name:http://www.sleepycat.com/2002/dbxml";
 const char * DbXml::metaDataName_uri_root = "root:http://www.sleepycat.com/2002/dbxml";
 
+using namespace std;
+using namespace DbXml;
+
 // Index
 
 Index::Index()
@@ -56,7 +53,7 @@ Index::Index(const std::string &s)
 
 bool Index::set(const std::string &s)
 {
-	bool r = true;
+	bool valid = true;
 	index_ = 0;
 	size_t start = 0;
 	size_t end = 0;
@@ -71,7 +68,7 @@ bool Index::set(const std::string &s)
 			if (syntax != 0) {
 				index_ |= syntax->getType();
 			} else {
-				r = false; // ERROR
+				valid = false; // ERROR
 			}
 		} else {
 			index_ |= i->second;
@@ -83,13 +80,13 @@ bool Index::set(const std::string &s)
 			}
 		}
 	}
-	if (r) {
+	if (valid) {
 		// The indexing strategy didn't contain any unknown words, but
 		// did we end up with an indexing strategy that makes sense?
 		//
-		r = isValidIndex();
+		valid = isValidIndex();
 	}
-	return r;
+	return valid;
 }
 
 bool Index::set(unsigned long index)
@@ -212,25 +209,6 @@ unsigned char Index::getKeyPrefix() const
 		 ((index_&KEY_MASK) >> 8));
 }
 
-size_t Index::getKeySize() const
-{
-	size_t n = sizeof(unsigned char); // prefix
-	switch (getPath()) {
-	case PATH_NONE:
-		break;
-	case PATH_NODE:
-		n += sizeof(u_int32_t);
-		break;
-	case PATH_EDGE:
-		n += sizeof(u_int32_t);
-		n += sizeof(u_int32_t);
-		break;
-	default:
-		break;
-	}
-	return n;
-}
-
 std::string Index::asString() const
 {
 	string s;
@@ -322,7 +300,6 @@ void IndexVector::enableIndex(const IndexVector &iv)
 
 Index IndexVector::getIndex(const Index::Type &test, const Index::Type &mask) const
 {
-        bool found = false;
         const_iterator i;
         for(i = begin(); i != end(); ++i) {
                 if(i->equalsMask(test, mask)) {
@@ -518,6 +495,20 @@ const IndexVector *IndexSpecification::getIndexOrDefault(const char *uriname) co
 	return &defaultIndex_;
 }
 
+// convenience method to see if there are content indexes (element and/
+// or attribute) in the specification
+bool IndexSpecification::isContentIndexed() const
+{
+	return (isIndexed(Index::NODE_ATTRIBUTE, Index::NODE_MASK) ||
+		isIndexed(Index::NODE_ELEMENT, Index::NODE_MASK));
+}
+
+// return true if there is a default index on metdata
+bool IndexSpecification::isMetaDataDefaultIndex() const
+{
+	return defaultIndex_.isEnabled(Index::NODE_METADATA, Index::NODE_MASK);
+}
+
 bool IndexSpecification::isIndexed(Index::Type test, Index::Type mask) const
 {
 	if(defaultIndex_.isEnabled(test, mask)) return true;
@@ -544,6 +535,10 @@ const IndexSpecification::NameList IndexSpecification::getIndexed(Index::Type te
 void IndexSpecification::enableIndex(const char *uriname, const Index &index)
 {
 	IndexVector *iv = &defaultIndex_;
+	// null uriname is default index
+	// check for bad name (empty string)
+	if (uriname && !*uriname)
+		throw XmlException(XmlException::INVALID_VALUE, "Illegal index name (empty string)");
 	if(uriname != 0) {
 		IndexMap::iterator i = indexMap_.find(uriname);
 		if (i != indexMap_.end()) {
@@ -575,6 +570,10 @@ void IndexSpecification::enableIndex(const char *uriname, const std::string &ind
 
 void IndexSpecification::enableIndex(const char *uriname, const IndexVector &iv)
 {
+	// check for bad name (empty string)
+	if (uriname && !*uriname)
+		throw XmlException(XmlException::INVALID_VALUE, "Illegal index name (empty string)");
+	
 	IndexMap::iterator i = indexMap_.find(uriname);
 	if (i != indexMap_.end()) {
 		i->second->enableIndex(iv);
@@ -703,7 +702,6 @@ bool IndexSpecification::find(const std::string &uri, const std::string &name, s
 	bool r = false;
 	Name n(uri, name);
 	string s(n.getURIName());
-	IndexVector *iv = 0;
 	IndexMap::iterator i = indexMap_.find(s.c_str());
 	if (i != indexMap_.end() && i->second->isIndexed()) {
 		index = i->second->asString();
@@ -728,6 +726,7 @@ int IndexSpecification::read(const ConfigurationDatabase *config, Transaction *t
 			clear();
 			const char *p = (const char *)tmpBuffer_.getBuffer();
 			if(p != 0) {
+				// Default index
 				const char *index = p;
 				p += strlen(p) + 1;
 				enableIndex(0, index);
@@ -747,7 +746,83 @@ int IndexSpecification::read(const ConfigurationDatabase *config, Transaction *t
 	return err;
 }
 
+int IndexSpecification::upgrade(DbWrapper &config_db, int old_version, int current_version)
+{
+	int err = 0;
+	if (old_version < VERSION_23) {
+		// Need to remove indices of type anyURI, QName, NOTATION,
+		// replacing them with equivalent string indices
+		static const char *keyValue = "index";
+		static size_t keyLength = strlen(keyValue) + 1;
+
+		DbtIn key((void*)keyValue, keyLength);
+		DbtOut value;
+		err = config_db.get(0, &key, &value, 0);
+
+		clear();
+		if (err == 0) {
+			const char *p = (const char *)value.get_data();
+			if(p != 0) {
+				// Default index
+				const char *index = p;
+				p += strlen(p) + 1;
+				upgradeEnableIndex(0, index);
+			}
+			while (p != 0 && *p != '\0') {
+				const char *uriname = p;
+				p += strlen(p) + 1;
+				const char *index = p;
+				p += strlen(p) + 1;
+				upgradeEnableIndex(uriname, index);
+			}
+		}
+		else if (err == DB_NOTFOUND) {
+			err = 0;
+		}
+
+		if(err == 0) {
+			writeToBuffer();
+			DbtIn new_value(buffer_.getBuffer(), buffer_.getOccupancy());
+			err = config_db.put(0, &key, &new_value, 0);
+		}
+	}
+	return err;
+}
+
+void IndexSpecification::upgradeEnableIndex(const char *uriname, const std::string &indexString)
+{
+	// Enable the index strategy.
+	tokenizer tok(indexString, ", ");
+	std::string indexspec;
+	Index index;
+	while(tok.next(indexspec) == 0) {
+		if(!index.set(indexspec) &&
+			index.equalsMask(Index::KEY_EQUALITY, Index::KEY_MASK) &&
+			index.equalsMask(Index::SYNTAX_NONE, Index::SYNTAX_MASK)) {
+			// Need to remove indices of type anyURI, QName, NOTATION,
+			// replacing them with equivalent string indices
+			if(indexspec.find(AtomicTypeValue::getValueTypeString(XmlValue::ANY_URI)) != string::npos) {
+				index.set(Syntax::STRING, Index::SYNTAX_MASK);
+			}
+			else if(indexspec.find(AtomicTypeValue::getValueTypeString(XmlValue::QNAME)) != string::npos) {
+				index.set(Syntax::STRING, Index::SYNTAX_MASK);
+			}
+			else if(indexspec.find(AtomicTypeValue::getValueTypeString(XmlValue::NOTATION)) != string::npos) {
+				index.set(Syntax::STRING, Index::SYNTAX_MASK);
+			}
+		}
+		enableIndex(uriname, index);
+	}
+}
+
+
 int IndexSpecification::write(ConfigurationDatabase *config, Transaction *txn) const
+{
+	writeToBuffer();
+	return config->putConfigurationItem(txn, "index", buffer_);
+}
+
+void IndexSpecification::writeToBuffer() const
 {
 	if (buffer_.getOccupancy() == 0) {
 		string index = defaultIndex_.asString();
@@ -766,7 +841,6 @@ int IndexSpecification::write(ConfigurationDatabase *config, Transaction *txn) c
 		char null = '\0';
 		buffer_.write(&null, sizeof(null));
 	}
-	return config->putConfigurationItem(txn, "index", buffer_);
 }
 
 std::string IndexSpecification::asString() const
