@@ -1,10 +1,9 @@
 //
 // See the file LICENSE for redistribution information.
 //
-// Copyright (c) 2002-2005
-//	Sleepycat Software.  All rights reserved.
+// Copyright (c) 2002,2006 Oracle.  All rights reserved.
 //
-// $Id: KeyStatistics.hpp,v 1.29 2005/04/05 16:44:02 bostic Exp $
+// $Id: KeyStatistics.hpp,v 1.32 2006/10/30 17:45:52 bostic Exp $
 //
 
 #ifndef __KEYSTATISTICS_HPP
@@ -12,6 +11,8 @@
 
 #include <string>
 
+#include "nodeStore/NsTypes.hpp"
+
 namespace DbXml
 {
 class DbtOut;
@@ -28,6 +29,9 @@ public:
 	void setThisFromDbt(const DbtOut &dbt);
 	void setDbtFromThis(DbtOut &dbt) const;
 
+	int marshal(xmlbyte_t *ptr, bool count) const;
+	void unmarshal(const xmlbyte_t *ptr);
+
 	std::string asString() const;
 
 	int32_t numIndexedKeys_;
