@@ -1,41 +1,35 @@
 //
 // See the file LICENSE for redistribution information.
 //
-// Copyright (c) 2002-2005
-//	Sleepycat Software.  All rights reserved.
+// Copyright (c) 2002,2006 Oracle.  All rights reserved.
 //
-// $Id: QueryExecutionContext.cpp,v 1.3 2005/04/05 16:44:02 bostic Exp $
+// $Id: QueryExecutionContext.cpp,v 1.11 2006/11/17 18:18:25 jsnelson Exp $
 //
 
+#include "DbXmlInternal.hpp"
 #include "QueryExecutionContext.hpp"
 #include <float.h>
 
 using namespace DbXml;
 using namespace std;
 
-QueryExecutionContext::QueryExecutionContext(const Container &container, QueryContext &context, bool debugging)
-	: container(container),
-    statsReadCache_(container),
-    idsReadCache_(container),
-    context(context),
-    costToFilterPointer(0),
-    debugging_(debugging)
+QueryExecutionContext::QueryExecutionContext(QueryContext &context, bool debugging)
+	: container_(0),
+	  context(context),
+	  dc_(0),
+	  costToFilterPointer(0),
+	  debugging_(debugging)
 {}
 
 QueryExecutionContext::~QueryExecutionContext()
 {}
 
-const Container &QueryExecutionContext::getContainer() const
-{
-	return container;
-}
-
 void QueryExecutionContext::setCostToFilterFunction(CostFunctionPointer cfp)
 {
 	costToFilterPointer = cfp;
 }
 
-double QueryExecutionContext::costToFilter(const IDS::SharedPtr &ids) const
+double QueryExecutionContext::costToFilter(const IDS::Ptr &ids) const
 {
 	double r = DBL_MAX; // Very costly... keep refining candidate set.
 	if (costToFilterPointer != 0) {
@@ -44,6 +38,15 @@ double QueryExecutionContext::costToFilter(const IDS::SharedPtr &ids) const
 	return r;
 }
 
+double QueryExecutionContext::costToFilter(const IndexData::Ptr &data) const
+{
+	double r = DBL_MAX; // Very costly... keep refining candidate set.
+	if (costToFilterPointer != 0) {
+		r = (*costToFilterPointer)((!data ? 0 : data->size()));
+	}
+	return r;
+}
+
 string QueryExecutionContext::toString() const
 {
 	return "QueryExecutionContext"; // jcm - could provide a description for debugging
