@@ -1,15 +1,14 @@
 //
 // See the file LICENSE for redistribution information.
 //
-// Copyright (c) 2002-2005
-//	Sleepycat Software.  All rights reserved.
+// Copyright (c) 2002,2006 Oracle.  All rights reserved.
 //
-// $Id: ReferenceCounted.cpp,v 1.12 2005/04/05 16:44:02 bostic Exp $
+// $Id: ReferenceCounted.cpp,v 1.17 2006/10/30 17:45:53 bostic Exp $
 //
 
-#include "dbxml_config.h"
-#include "dbxml/XmlPortability.hpp"
+#include "DbXmlInternal.hpp"
 #include "ReferenceCounted.hpp"
+#include "Globals.hpp"
 #include <xercesc/util/PlatformUtils.hpp>
 
 #if defined(XERCES_HAS_CPP_NAMESPACE)
@@ -19,7 +18,8 @@
 using namespace DbXml;
 
 ReferenceCountedProtected::ReferenceCountedProtected()
-	: mutex_(XMLPlatformUtils::makeMutex())
+	: count_(0),
+	  mutex_(XMLPlatformUtils::makeMutex(Globals::defaultMemoryManager))
 {}
 
 ReferenceCountedProtected::~ReferenceCountedProtected()
