@@ -1,13 +1,12 @@
 //
 // See the file LICENSE for redistribution information.
 //
-// Copyright (c) 2004-2005
-//	Sleepycat Software.  All rights reserved.
+// Copyright (c) 2004,2006 Oracle.  All rights reserved.
 //
-// $Id: StdInInputStream.cpp,v 1.4 2005/04/05 16:44:03 bostic Exp $
+// $Id: StdInInputStream.cpp,v 1.7 2006/10/30 17:45:53 bostic Exp $
 //
 
-#include "dbxml/XmlPortability.hpp"
+#include "DbXmlInternal.hpp"
 #include "StdInInputStream.hpp"
 #include <xercesc/framework/StdInInputSource.hpp>
 
