@@ -1,22 +1,21 @@
 //
 // See the file LICENSE for redistribution information.
 //
-// Copyright (c) 2002-2005
-//	Sleepycat Software.  All rights reserved.
+// Copyright (c) 2002,2006 Oracle.  All rights reserved.
 //
-// $Id: UpdateContext.cpp,v 1.14 2005/04/05 16:44:03 bostic Exp $
+// $Id: UpdateContext.cpp,v 1.19 2006/10/30 17:45:53 bostic Exp $
 //
 
-#include "dbxml_config.h"
-#include "dbxml/XmlPortability.hpp"
+#include "DbXmlInternal.hpp"
 #include "UpdateContext.hpp"
+#include "Manager.hpp"
 
 using namespace DbXml;
 
 UpdateContext::UpdateContext(XmlManager &db)
 	: db_(db),
 	  applyChanges_(true),
-	  indexer_()
+	  indexer_(((Manager&)db).getImplicitTimezone())
 {}
 
 UpdateContext::~UpdateContext()
@@ -24,7 +23,7 @@ UpdateContext::~UpdateContext()
 
 void UpdateContext::init(Transaction *txn, Container *container)
 {
-	indexer_.reset(txn, container);
+	indexer_.resetContext(container, &oc_);
 	is_.read(container->getConfigurationDB(), txn, /*lock=*/false);
 	oc_.set(txn);
 }
