@@ -1,14 +1,12 @@
 //
 // See the file LICENSE for redistribution information.
 //
-// Copyright (c) 2002-2005
-//	Sleepycat Software.  All rights reserved.
+// Copyright (c) 2002,2006 Oracle.  All rights reserved.
 //
-// $Id: VariableBindings.cpp,v 1.19 2005/04/20 18:31:29 bostic Exp $
+// $Id: VariableBindings.cpp,v 1.23 2006/10/30 17:45:53 bostic Exp $
 //
 
-#include "dbxml_config.h"
-#include "dbxml/XmlPortability.hpp"
+#include "DbXmlInternal.hpp"
 #include "dbxml/XmlException.hpp"
 #include "VariableBindings.hpp"
 
@@ -30,29 +28,20 @@ VariableBindings &VariableBindings::operator = (const VariableBindings &o)
 }
 
 bool VariableBindings::getVariableValue(const std::string &name,
-					XmlValue &value) const
+					XmlResults &value) const
 {
 	Values::const_iterator i = values_.find(name);
 	if (i != values_.end()) {
 		value = i->second;
+		value.reset();
 	} else {
-		value = XmlValue();
+		value = XmlResults();
 	}
 	return !value.isNull();
 }
 
 void VariableBindings::setVariableValue(const std::string &name,
-					const XmlValue &value)
+					const XmlResults &value)
 {
-  // prevent the type of a value from changing
-	Values::const_iterator i = values_.find(name);
-	if (i != values_.end()) {
-		if(i->second.getType() != value.getType()) {
-			throw XmlException(
-				XmlException::INVALID_VALUE,
-				"The type of a variable value cannot change");
-		}
-	}
-
 	values_[name] = value;
 }
