@@ -1,10 +1,9 @@
 //
 // See the file LICENSE for redistribution information.
 //
-// Copyright (c) 2002-2005
-//	Sleepycat Software.  All rights reserved.
+// Copyright (c) 2002,2006 Oracle.  All rights reserved.
 //
-// $Id: VariableBindings.hpp,v 1.16 2005/04/05 16:44:03 bostic Exp $
+// $Id: VariableBindings.hpp,v 1.19 2006/10/30 17:45:53 bostic Exp $
 //
 
 #ifndef __VARIABLEBINDINGS_HPP
@@ -12,7 +11,7 @@
 
 #include <string>
 #include <map>
-#include "dbxml/XmlValue.hpp"
+#include <dbxml/XmlResults.hpp>
 
 namespace DbXml
 {
@@ -20,7 +19,7 @@ namespace DbXml
 class VariableBindings
 {
 public:
-	typedef std::map<std::string, XmlValue> Values;
+	typedef std::map<std::string, XmlResults> Values;
 	VariableBindings()
 	{}
 	;
@@ -30,8 +29,8 @@ public:
 	VariableBindings(const VariableBindings &);
 	VariableBindings & operator = (const VariableBindings &);
 
-	bool getVariableValue(const std::string &name, XmlValue &value) const;
-	void setVariableValue(const std::string &name, const XmlValue &value);
+	bool getVariableValue(const std::string &name, XmlResults &value) const;
+	void setVariableValue(const std::string &name, const XmlResults &value);
 	const Values &getValues() const
 	{
 		return values_;
