@@ -1,14 +1,12 @@
 //
 // See the file LICENSE for redistribution information.
 //
-// Copyright (c) 2002-2005
-//	Sleepycat Software.  All rights reserved.
+// Copyright (c) 2002,2006 Oracle.  All rights reserved.
 //
-// $Id: XmlContainer.cpp,v 1.122 2005/04/20 18:31:29 bostic Exp $
+// $Id: XmlContainer.cpp,v 1.142 2006/11/15 21:07:10 gmf Exp $
 //
 
-#include "dbxml_config.h"
-#include "dbxml/XmlPortability.hpp"
+#include "DbXmlInternal.hpp"
 #include "dbxml/XmlException.hpp"
 #include "dbxml/XmlContainer.hpp"
 #include "dbxml/XmlUpdateContext.hpp"
@@ -16,6 +14,8 @@
 #include "dbxml/XmlTransaction.hpp"
 #include "dbxml/XmlResults.hpp"
 #include "dbxml/XmlStatistics.hpp"
+#include "dbxml/XmlIndexLookup.hpp"
+#include "dbxml/XmlNamespace.hpp"
 #include "TransactedContainer.hpp"
 #include "UpdateContext.hpp"
 #include "QueryExpression.hpp"
@@ -23,15 +23,20 @@
 #include "Manager.hpp"
 #include "Document.hpp"
 #include "Results.hpp"
-#include "Util.hpp"
+#include "IndexLookup.hpp"
+#include "nodeStore/NsEventWriter.hpp"
 
-#include <pathan/PathanPlatformUtils.hpp>
-#include <pathan/PathanException.hpp>
-#include <pathan/XPath2MemoryManager.hpp>
+#include <xqilla/utils/XQillaPlatformUtils.hpp>
+#include <xqilla/exceptions/XQillaException.hpp>
+#include <xqilla/framework/XPath2MemoryManager.hpp>
 
 #include <db_cxx.h>
 #include <cerrno>
 
+#if defined(XERCES_HAS_CPP_NAMESPACE)
+XERCES_CPP_NAMESPACE_USE
+#endif
+
 using namespace DbXml;
 using namespace std;
 
@@ -96,6 +101,11 @@ XmlManager &XmlContainer::getManager()
 	return container_->getManager();
 }
 
+u_int32_t XmlContainer::getFlags() const
+{
+	return container_->getFlags();
+}
+	
 void XmlContainer::setIndexSpecification(XmlTransaction &txn, 
 					 const XmlIndexSpecification &index,
 					 XmlUpdateContext &context)
@@ -309,6 +319,18 @@ XmlContainer::ContainerType XmlContainer::getContainerType() const
 	return container_->getContainerType();
 }
 
+bool XmlContainer::getIndexNodes() const
+{
+	CHECK_POINTER;
+	return container_->getIndexNodes();
+}
+
+u_int32_t XmlContainer::getPageSize() const
+{
+	CHECK_POINTER;
+	return container_->getPageSize();
+}
+
 std::string XmlContainer::putDocument(const std::string &name,
 				      XmlInputStream *input,
 				      XmlUpdateContext &context,
@@ -336,11 +358,64 @@ std::string XmlContainer::putDocument(XmlTransaction &txn,
 	return document.getName();
 }
 
+std::string XmlContainer::putDocument(const std::string &name,
+				      XmlEventReader &reader,
+				      XmlUpdateContext &context,
+				      u_int32_t flags)
+{
+	CHECK_POINTER;
+	XmlDocument document = container_->getManager().createDocument();
+	document.setName(name);
+	document.setContentAsEventReader(reader);
+	putDocument(document, context, flags);
+	return document.getName();
+}
+
+std::string XmlContainer::putDocument(XmlTransaction &txn,
+				      const std::string &name,
+				      XmlEventReader &reader,
+				      XmlUpdateContext &context,
+				      u_int32_t flags)
+{
+	CHECK_POINTER;
+	XmlDocument document = container_->getManager().createDocument();
+	document.setName(name);
+	document.setContentAsEventReader(reader);
+	putDocument(txn, document, context, flags);
+	return document.getName();
+}
+
+
+XmlEventWriter & XmlContainer::putDocumentAsEventWriter(
+	XmlDocument &document,
+	XmlUpdateContext &context,
+	u_int32_t flags)
+{
+	CHECK_POINTER;
+	container_->checkFlags(Log::misc_flag_info, "putDocumentAsEventWriter()",
+			       flags, DBXML_GEN_NAME);
+	return container_->addDocumentAsEventWriter(0, document, context, flags);
+}
+
+XmlEventWriter & XmlContainer::putDocumentAsEventWriter(
+	XmlTransaction &txn,
+	XmlDocument &document,
+	XmlUpdateContext &context,
+	u_int32_t flags)
+{
+	CHECK_POINTER;
+	container_->checkFlags(Log::misc_flag_info,
+			       "putDocumentAsEventWriter()",
+			       flags, DBXML_GEN_NAME);
+	return container_->addDocumentAsEventWriter(txn, document, context, flags);
+}
+
 void XmlContainer::putDocument(XmlTransaction &txn, XmlDocument &document,
 			       XmlUpdateContext &context, u_int32_t flags)
 {
 	CHECK_POINTER;
-	container_->checkFlags(Log::misc_flag_info, "putDocument()", flags, DBXML_GEN_NAME);
+	container_->checkFlags(Log::misc_flag_info, "putDocument()", flags,
+			       DBXML_GEN_NAME);
 	int err = 0;
 	try {
 		err = container_->addDocument(txn, document, context, flags);
@@ -356,7 +431,8 @@ void XmlContainer::putDocument(XmlDocument &document,
 			       u_int32_t flags)
 {
 	CHECK_POINTER;
-	container_->checkFlags(Log::misc_flag_info, "putDocument()", flags, DBXML_GEN_NAME);
+	container_->checkFlags(Log::misc_flag_info, "putDocument()", flags,
+			       DBXML_GEN_NAME|DBXML_WELL_FORMED_ONLY);
 	int err = 0;
 	try {
 		err = container_->addDocument(0, document, context, flags);
@@ -395,7 +471,10 @@ string XmlContainer::putDocument(const std::string &name,
 static void throwDocNotFound(const std::string &name)
 {
 	ostringstream s;
-	s << "Document not found: " << name;
+	std::string tname(name);
+	if (tname.size() == 0)
+		tname = "<empty name>";
+	s << "Document not found: " << tname;
 	throw XmlException(XmlException::DOCUMENT_NOT_FOUND, 
 			   s.str().c_str());
 
@@ -406,8 +485,9 @@ XmlDocument XmlContainer::getDocument(XmlTransaction &txn,
 {
 	CHECK_POINTER;
 	container_->checkFlags(Log::misc_flag_info, "getDocument()", flags,
-			       DB_DIRTY_READ|DB_RMW|DBXML_LAZY_DOCS);
-	XmlDocument r = container_->getManager().createDocument();
+			       DB_READ_COMMITTED|DB_READ_UNCOMMITTED|DB_RMW|
+			       DBXML_LAZY_DOCS|DB_TXN_SNAPSHOT);
+	XmlDocument r;
 	int err = 0;
 	try {
 		OperationContext oc(txn);
@@ -429,8 +509,10 @@ XmlDocument XmlContainer::getDocument(XmlTransaction &txn,
 XmlDocument XmlContainer::getDocument(const std::string &name, u_int32_t flags)
 {
 	CHECK_POINTER;
-	container_->checkFlags(Log::misc_flag_info, "getDocument()", flags, DB_DIRTY_READ|DB_RMW|DBXML_LAZY_DOCS);
-	XmlDocument r = container_->getManager().createDocument();
+	container_->checkFlags(Log::misc_flag_info, "getDocument()", flags,
+			       DB_READ_COMMITTED|DB_READ_UNCOMMITTED|DB_RMW|
+			       DBXML_LAZY_DOCS|DB_TXN_SNAPSHOT);
+	XmlDocument r;
 	int err = 0;
 	try {
 		OperationContext oc;
@@ -449,6 +531,77 @@ XmlDocument XmlContainer::getDocument(const std::string &name, u_int32_t flags)
 	return r;
 }
 
+XmlValue XmlContainer::getNode(XmlTransaction &txn, 
+			       const std::string &nodeHandle,
+			       u_int32_t flags)
+{
+	CHECK_POINTER;
+	container_->checkFlags(Log::misc_flag_info, "getNode()", flags,
+			       DB_READ_COMMITTED|DB_READ_UNCOMMITTED|DB_RMW|
+			       DBXML_LAZY_DOCS|DB_TXN_SNAPSHOT);
+
+	IndexEntry ie;
+	ie.setFromNodeHandle(nodeHandle);
+
+	XmlDocument doc;
+	try {
+		OperationContext oc(txn);
+		ie.getDocID().fetchDocument(container_, oc, flags, doc, 0);
+	}
+	catch(XmlException &ex) {
+		if(ex.getExceptionCode() != XmlException::INTERNAL_ERROR) {
+			throw;
+		}
+	}
+	if(doc.isNull()) {
+		ostringstream s;
+		s << "Invalid node handle (document " << ie.getDocID().asString() << " not found)";
+
+		container_->log(Log::C_QUERY, Log::L_ERROR, s.str());
+		throw XmlException(XmlException::INVALID_VALUE, "The node handle points to a non-existent document");
+	}
+
+	DOMNode *node = 0;
+	try {
+		node = ie.fetchNode(doc);
+	}
+	catch(XmlException &ex) {
+		if(ex.getExceptionCode() != XmlException::INTERNAL_ERROR) {
+			throw;
+		}
+	}
+	if(node == 0) {
+		ostringstream s;
+		s << "Invalid node handle (element 0x";
+		ie.getNodeID().displayNid(s);
+		if(ie.isSpecified(IndexEntry::ATTRIBUTE_INDEX)) {
+			s << ", attr " << ie.getIndex();
+		}
+		if(ie.isSpecified(IndexEntry::TEXT_INDEX)) {
+			s << ", text " << ie.getIndex();
+		}
+		if(ie.isSpecified(IndexEntry::COMMENT_INDEX)) {
+			s << ", comment " << ie.getIndex();
+		}
+		if(ie.isSpecified(IndexEntry::PI_INDEX)) {
+			s << ", pi " << ie.getIndex();
+		}
+		s << " not found)";
+
+		container_->log(Log::C_QUERY, Log::L_ERROR, s.str());
+		throw XmlException(XmlException::INVALID_VALUE, "The node handle points to a non-existent node");
+	}
+
+	return new NodeValue(node, doc);
+}
+
+XmlValue XmlContainer::getNode(const std::string &nodeHandle,
+			       u_int32_t flags)
+{
+	XmlTransaction txn;
+	return getNode(txn, nodeHandle, flags);
+}
+
 void XmlContainer::deleteDocument(XmlTransaction &txn,
 				  const std::string &name,
 				  XmlUpdateContext &context)
@@ -534,9 +687,14 @@ void XmlContainer::updateDocument(XmlTransaction &txn, XmlDocument &document,
 	try {
 		err = container_->updateDocument(txn, document, context);
 	} catch (DbException &e) {
-		throw XmlException(e);
+		if (e.get_errno() == DB_NOTFOUND)
+			err = DB_NOTFOUND;
+		else
+			throw XmlException(e);
 	}
-	if (err != 0)
+	if (err == DB_NOTFOUND)
+		throwDocNotFound(document.getName());
+	else if (err != 0)
 		throw XmlException(err);
 }
 
@@ -548,37 +706,40 @@ void XmlContainer::updateDocument(XmlDocument &document,
 	try {
 		err = container_->updateDocument(0, document, context);
 	} catch (DbException &e) {
-		throw XmlException(e);
+		if (e.get_errno() == DB_NOTFOUND)
+			err = DB_NOTFOUND;
+		else
+			throw XmlException(e);
 	}
-	if (err != 0)
+	if (err == DB_NOTFOUND)
+		throwDocNotFound(document.getName());
+	else if (err != 0)
 		throw XmlException(err);
 }
 
 // common function for getAllDocuments() and getNumDocuments variants.
 // Always use lazy evaluation
-static Results *getDocs(Container *container,
+static Results *getDocs(XmlContainer &container,
 			Transaction *txn,
 			bool lazy,
 			u_int32_t flags)
 {
-	XmlQueryContext context = container->getManager().createQueryContext();
+	XmlQueryContext context = container.getManager().createQueryContext();
 	if (lazy)
 		context.setEvaluationType(XmlQueryContext::Lazy);
 	if (flags == 0)
 		flags = DBXML_LAZY_DOCS;
-	XmlValue val;
-	Name nm(Name::dbxml_colon_name);
-	Results *res = container->lookupIndex(txn, context, &nm,
-					      NULL, // parent
-					      "unique-metadata-equality-string",
-					      val, flags);
-	return res;
+
+	XmlIndexLookup il = container.getManager().createIndexLookup(container,
+		metaDataNamespace_uri, metaDataName_name,
+		"unique-metadata-equality-string");
+	return ((IndexLookup&)il).execute(txn, context, flags);
 }
 
 XmlResults XmlContainer::getAllDocuments(u_int32_t flags)
 {
 	CHECK_POINTER;
-	Results *res = getDocs(container_, NULL, true, flags);
+	Results *res = getDocs(*this, NULL, true, flags);
 	return res;
 }
 
@@ -586,14 +747,14 @@ XmlResults XmlContainer::getAllDocuments(XmlTransaction &txn,
 					 u_int32_t flags)
 {
 	CHECK_POINTER;
-	Results *res = getDocs(container_, txn, true, flags);
+	Results *res = getDocs(*this, txn, true, flags);
 	return res;
 }
 
 size_t XmlContainer::getNumDocuments()
 {
 	CHECK_POINTER;
-	Results *res = getDocs(container_, NULL, false, 0);
+	Results *res = getDocs(*this, NULL, false, 0);
 	size_t num = res->size();
 	delete res;
 	return num;
@@ -602,24 +763,27 @@ size_t XmlContainer::getNumDocuments()
 size_t XmlContainer::getNumDocuments(XmlTransaction &txn)
 {
 	CHECK_POINTER;
-	Results *res = getDocs(container_, txn, false, 0);
+	Results *res = getDocs(*this, txn, false, 0);
 	size_t num = res->size();
 	delete res;
 	return num;
 }
 
+//
+// all lookupIndex methods are deprecated in favor of
+// createIndexLookup.
+//
 XmlResults XmlContainer::lookupIndex(XmlQueryContext &context,
 				     const std::string &uri,
 				     const std::string &name,
 				     const std::string &index,
 				     const XmlValue &value, u_int32_t flags)
 {
-	CHECK_POINTER;
-	Name child(uri, name);
-	return container_->lookupIndex(NULL, // txn
-				       context, &child,
-				       NULL, // parent
-				       index, value, flags);
+	// NOTE: Operation is always EQ for compatibility interfaces
+	XmlIndexLookup il = getManager().createIndexLookup(
+		*this, uri, name, index, value,
+		XmlIndexLookup::EQ);
+	return ((IndexLookup&)il).execute(0,context, flags);
 }
 
 XmlResults XmlContainer::lookupIndex(XmlQueryContext &context,
@@ -630,13 +794,11 @@ XmlResults XmlContainer::lookupIndex(XmlQueryContext &context,
 				     const std::string &index,
 				     const XmlValue &value, u_int32_t flags)
 {
-	CHECK_POINTER;
-	Name child(uri, name);
-	Name parent(parent_uri, parent_name);
-	return container_->lookupIndex(NULL, // txn
-				       context, &child,
-				       &parent,
-				       index, value, flags);
+	// NOTE: Operation is always EQ for compatibility interfaces
+	XmlIndexLookup il = getManager().createIndexLookup(
+		*this, uri, name, index, value, XmlIndexLookup::EQ);
+	il.setParent(parent_uri, parent_name);
+	return ((IndexLookup&)il).execute(0,context, flags);
 }
 
 XmlResults XmlContainer::lookupIndex(XmlTransaction &txn,
@@ -647,12 +809,11 @@ XmlResults XmlContainer::lookupIndex(XmlTransaction &txn,
 				     const XmlValue &value,
 				     u_int32_t flags)
 {
-	CHECK_POINTER;
-	Name child(uri, name);
-	return container_->lookupIndex(txn,
-				       context, &child,
-				       NULL, // parent
-				       index, value, flags);
+	// NOTE: Operation is always EQ for compatibility interfaces
+	XmlIndexLookup il = getManager().createIndexLookup(
+		*this, uri, name, index, value, XmlIndexLookup::EQ);
+
+	return ((IndexLookup&)il).execute(txn,context, flags);
 }
 
 XmlResults XmlContainer::lookupIndex(XmlTransaction &txn,
@@ -665,13 +826,12 @@ XmlResults XmlContainer::lookupIndex(XmlTransaction &txn,
 				     const XmlValue &value,
 				     u_int32_t flags)
 {
-	CHECK_POINTER;
-	Name child(uri, name);
-	Name parent(parent_uri, parent_name);
-	return container_->lookupIndex(txn,
-				       context, &child,
-				       &parent,
-				       index, value, flags);
+	// NOTE: Operation is always EQ for compatibility interfaces
+	XmlIndexLookup il = getManager().createIndexLookup(
+		*this, uri, name, index, value, XmlIndexLookup::EQ);
+
+	il.setParent(parent_uri, parent_name);
+	return ((IndexLookup&)il).execute(txn,context, flags);
 }
 
 XmlStatistics XmlContainer::lookupStatistics(const std::string &uri,
@@ -747,25 +907,3 @@ XmlContainer::operator Container &()
 	CHECK_POINTER;
 	return *container_;
 }
-
-DBXML_EXPORT const char *DbXml::dbxml_version(int *majorp, int *minorp, 
-					      int *patchp)
-{
-	if (majorp != NULL)
-		*majorp = DBXML_VERSION_MAJOR;
-	if (minorp != NULL)
-		*minorp = DBXML_VERSION_MINOR;
-	if (patchp != NULL)
-		*patchp = DBXML_VERSION_PATCH;
-	return DBXML_VERSION_STRING;
-}
-
-DBXML_EXPORT void DbXml::setLogLevel(LogLevel level, bool enabled)
-{
-	Log::setLogLevel((Log::ImplLogLevel)level, enabled);
-}
-
-DBXML_EXPORT void DbXml::setLogCategory(LogCategory category, bool enabled)
-{
-	Log::setLogCategory((Log::ImplLogCategory)category, enabled);
-}
