@@ -1,16 +1,14 @@
 //
 // See the file LICENSE for redistribution information.
 //
-// Copyright (c) 2002-2005
-//	Sleepycat Software.  All rights reserved.
+// Copyright (c) 2002,2006 Oracle.  All rights reserved.
 //
-// $Id: XmlMetaDataIterator.cpp,v 1.3 2005/04/05 16:44:04 bostic Exp $
+// $Id: XmlMetaDataIterator.cpp,v 1.7 2006/10/30 17:45:54 bostic Exp $
 //
 
-#include "dbxml_config.h"
+#include "DbXmlInternal.hpp"
 #include <dbxml/XmlMetaDataIterator.hpp>
 #include "Document.hpp"
-#include "Util.hpp"
 
 using namespace DbXml;
 
