@@ -1,14 +1,12 @@
 //
 // See the file LICENSE for redistribution information.
 //
-// Copyright (c) 2002-2005
-//	Sleepycat Software.  All rights reserved.
+// Copyright (c) 2002,2006 Oracle.  All rights reserved.
 //
-// $Id: XmlNamespace.cpp,v 1.21 2005/04/05 16:44:04 bostic Exp $
+// $Id: XmlNamespace.cpp,v 1.24 2006/10/30 17:45:54 bostic Exp $
 //
 
-#include "dbxml_config.h"
-#include "dbxml/XmlPortability.hpp"
+#include "DbXmlInternal.hpp"
 #include "dbxml/XmlNamespace.hpp"
 
 namespace DbXml
