@@ -1,10 +1,9 @@
 //
 // See the file LICENSE for redistribution information.
 //
-// Copyright (c) 2002-2005
-//      Sleepycat Software.  All rights reserved.
+// Copyright (c) 2002,2006 Oracle.  All rights reserved.
 //
-// $Id: NsConstants.hpp,v 1.4 2005/04/05 16:44:05 bostic Exp $
+// $Id: NsConstants.hpp,v 1.8 2006/10/30 17:45:57 bostic Exp $
 //
 
 #ifndef __DBXMLCONSTANTS_HPP
@@ -16,15 +15,6 @@
 
 #include "NsTypes.hpp"
 
-// well-known IDs
-#define NS_ID_OVERHEAD 3  // protocol, id byte, null byte
-#define NS_ID_ZERO 0x01
-#define NS_ID_FIRST 0x02
-#define NS_ID_LAST 0xFF
-#define NS_METADATA_ID (NS_ID_FIRST + 0) // document metadata
-#define NS_NAMESPACE_ID (NS_ID_FIRST + 1) // namespace info
-#define NS_ROOT_ID (NS_ID_FIRST + 2) // the document node
-
 namespace DbXml {
 
 extern const char *_xmlnsUri8;
@@ -50,8 +40,6 @@ extern const xmlch_t _decl_1_1[];
 extern const xmlch_t *_NsDecl[];
 extern const xmlbyte_t *_NsDecl8[];
 extern const char _docRootId[];
-extern const char _docNamespaceId[];
-extern const char _docMetadataId[];
 }
 
 #endif
