@@ -1,21 +1,20 @@
 //
 // See the file LICENSE for redistribution information.
 //
-// Copyright (c) 2002-2005
-//      Sleepycat Software.  All rights reserved.
+// Copyright (c) 2002,2006 Oracle.  All rights reserved.
 //
-// $Id: NsDom.hpp,v 1.27 2005/04/20 18:31:32 bostic Exp $
+// $Id: NsDom.hpp,v 1.50.4.1 2008/04/08 15:06:10 gmf Exp $
 //
 
 #ifndef __DBXMLNSDOM_HPP
 #define __DBXMLNSDOM_HPP
 /*
  * Node storage "DOM" classes.  These are primitive
- * classes that encapsulate nsNode_t structures.
+ * classes that encapsulate NsNode.
  * They provide the underpinnings for higher-level DOM
- * implementations (e.g. Xerces or other).
+ * and navigational implementations (e.g. Xerces or other).
  *
- * The intent is that higher-level implementations
+ * Higher-level implementations must
  * derive from these classes, and provide a
  * node factory implementation to be associated with
  * an NsDocument instance for node creation.
@@ -28,9 +27,9 @@
  *   NsDomNode -- interface.  pure virtual base for nodes.
  *     NsDomNav -- a "navigational" interface, for element and text nodes.
  *         adds parent and sibling pointers
- *       NsDomElement -- element.  Concrete.  Adds nsNode_t * and
+ *       NsDomElement -- element.  Concrete.  Adds NsNode * and
  *           child pointers, as well as plumbing pointers to other
- *           NsDomElement objects which comprise the tree of nsNode_t
+ *           NsDomElement objects which comprise the tree of NsNode
  *           elements (vs DOM nodes).
  *       NsDomText -- concrete text nodes -- includes comments, cdata, PI
  *           points to parent NsDomElem, and caches strings.
@@ -41,35 +40,17 @@
  *   by implementor of the DOM layer on top of these classes.
  *
  * NOTES:
- *  o A number of DOM-like methods are not implemented because they
- *    are not used by Pathan or XQuery.   The implementation can be
- *    extended if necessary.
+ *  o Implementation methods are based on need, and can be expanded
  *  o The Document object is just another NsDomElement
  *  o During a transient parse, an element-only tree is created using
- *    NsDomElement objects (containing nsNode_t pointers).  DOM access
+ *    NsDomElement objects (containing NsNode pointers).  DOM access
  *    to this tree will create the DOM tree on demand, materializing
  *    text and attribute nodes, for example.
- *    This is done for lazy evaluation -- there is no reason to
- *    actively create objects that will never be used.
- *  o It would have been possible to implement higher-level DOM
- *    code by layering on top of a single, NsDomNode, abstraction.
- *    The extra layer serves 2 purposes:
- *    1.  it provides a layer to "cache" transcoded strings, and index offsets
- *    2.  it factors out NsDom-specific code, which simplifies the upper layer.
- *    In addition, the higher layer needs to create objects conforming
- *    to a specific DOM anyway; requiring those objects to derive from
- *    NsDom* classes as well doesn't add significant overhead.
- *  o There are no "creation" interfaces, for now.  They are not required by
- *    Pathan/XQuery, and can be worked around for XmlModify.  These
- *    are a goal for the future, so that applications can perform
- *    persistent DOM modifications directly.
  *
- *  Encoding note:  the underlying nsNode_t structure may have text
+ *  Encoding note:  the underlying NsNode object may have text
  *    in either UTF-8 or UTF-16.  Because the current (only) DOM
  *    implementation is Xerces-based, the text method in these classes
- *    return UTF-16 strings (xmlch_t *).  If a UTF-8 based
- *    implementation is desired, the simplest thing is to add function
- *    overloads that return xmlbyte_t *.  The UTF-16-based implementations
+ *    return UTF-16 strings (xmlch_t *).  The UTF-16-based implementations
  *    will transcode/cache text results on demand.  For this reason,
  *    the methods are not const.
  *
@@ -78,8 +59,9 @@
  *    to avoid collisions with Xerces DOM names.  In general,
  *    they are the same as the DOM names, with the "Ns" inserted.
  */
-#include "NsTypes.hpp"
+#include "NsNode.hpp"
 #include "NsUtil.hpp"
+#include "NsDocument.hpp"
 #include <vector>
 
 namespace DbXml
@@ -92,9 +74,8 @@ class NsDomNav;
 class NsDomText;
 class NsDomAttr;
 class NsDomObj;
-class NsDocument;
 
-// simplified types for NsDom* classes
+// simplified types for NsDom classes
 typedef enum {
 	nsNodeElement,
 	nsNodeText,
@@ -104,27 +85,43 @@ typedef enum {
 	nsNodeEntEnd
 } NsNodeType_t;
 
-/** utility class to cache transcoded (or not) UTF-16 strings
-    for names, attributes, text */
+typedef enum {
+	nsDomInsertBefore,
+	nsDomInsertAfter,
+	nsDomInsertAppend,
+	nsDomInsertUnknown
+} NsDomInsertType;
+	
+// utility class to cache transcoded (or not) UTF-16 strings
+// for names, attributes, text
 class NsDomString {
 public:
-	NsDomString() : _text(0), _owned(false) {}
-	void set(const xmlch_t *text, bool owned);
+	NsDomString() : text_(0), flags_(0) {}
+	void set16(XER_NS MemoryManager *mmgr, const xmlch_t *text, bool owned);
+	void set(XER_NS MemoryManager *mmgr, const xmlbyte_t *text, bool owned);
 	void clear(XER_NS MemoryManager *mmgr);
-	const xmlch_t *get() const;
-	const bool getOwned() const;
+	const xmlch_t *get16() const;
+	const xmlbyte_t *get() const;
+	const bool isOwned() const;
+	const bool isUTF16() const;
 private:
-	const xmlch_t *_text;
-	bool _owned;
+	enum {
+		OWNED = 0x1,
+		UTF16 = 0x2
+	};
+	const void *text_;
+	uint32_t flags_;
 };
 
 class NsDomFactory {
 public:
+	virtual ~NsDomFactory() {}
 	virtual NsDomElement *createNsDomElement(NsDocument *doc,
-						 nsNode_t *node,
+						 NsNode *node,
 						 NsDomElement *parent,
 						 bool standalone) = 0;
 	virtual NsDomText *createNsDomText(NsDomElement *parent,
+					   NsDomElement *owner,
 					   int index, uint32_t type) = 0;
 	virtual NsDomNamedNodeMap *createNsDomNodeMap(NsDomElement *owner) = 0;
 	virtual NsDomNodeList *createNsDomNodeList(NsDomElement *owner) = 0;
@@ -145,36 +142,25 @@ public:
 
 	virtual void destroyDomList() = 0;
 	virtual void destroy(NsDomObj *dnode) = 0;
-
-	/// set document object (instance of document object)
-	void setNsDocumentObj(void *doc) { _domDocumentObj = doc; }
-	/// get document object (instance of document object)
-	void *getNsDocumentObj() { return _domDocumentObj; }
 protected:
-	NsDomFactory() : _domDocumentObj(0), _domFreeList(0) {}
+	NsDomFactory() : _domFreeList(0) {}
 	void addToDomFreeList(NsDomObj *dnode);
 	void removeFromDomFreeList(NsDomObj *dnode);
 protected:
-	void *_domDocumentObj;
 	NsDomObj *_domFreeList;
 };
 
 class NsDomObj {
 public:
+	virtual ~NsDomObj() {}
 	NsDomObj() : _next(0), _prev(0) {}
 	virtual void destroy() = 0;
 	virtual NsDocument *getNsDocument() const = 0;
+	virtual NsDomElement *getNsDocumentObj() { return getNsDocument()->getDocumentNode(); }
 	virtual NsDomFactory *getNsDomFactory() const = 0;
 	/** similar to DOM getInterface -- allows subclasses to
 	    fake dynamic_cast (callers need to check for 0 return) */
 	virtual void *getNsInterface(const char *name) { return 0; }
-	/// handle document object
-	void setNsDocumentObj(void *doc) {
-		getNsDomFactory()->setNsDocumentObj(doc);
-	}
-	void *getNsDocumentObj() {
-		return getNsDomFactory()->getNsDocumentObj();
-	}
 public:
 	NsDomObj *getNextObj() { return _next; }
 	void setNextObj(NsDomObj *next) { _next = next; }
@@ -197,9 +183,10 @@ public:
 	virtual NsDomNav *getNsFirstChild() { return 0; }
 	virtual NsDomNav *getNsLastChild() { return 0; }
 	virtual NsDomNav *getNsNextSibling() { return 0; }
-	virtual NsDomNav *getNsPrevSibling() { return 0; }
+	virtual NsDomNav *getNsPrevSibling(bool fetch = true) { return 0; }
 	virtual bool hasNsAttributes() const { return false; }
 	virtual bool hasNsChildNodes() const { return false; }
+	virtual bool hasNsChildElements() const { return false; }
 	virtual NsDomNodeList *getNsChildNodes();
 	virtual NsNodeType_t getNsNodeType() const = 0;
 	virtual bool getNsIsEntityType() const { return false; }
@@ -210,6 +197,18 @@ public:
 	virtual const xmlch_t *getNsUri() const { return 0; }
 	virtual const xmlch_t *getNsLocalName() const = 0;
 
+	virtual const xmlch_t *getNsBaseUri() const = 0;
+
+	// These functions are only guarenteed to return a
+	// result if isNativeUTF8 returns true
+	virtual bool isNativeUTF8() const = 0;
+	virtual const xmlbyte_t *getNsNodeName8() const = 0;
+	virtual const xmlbyte_t *getNsLocalName8() const = 0;
+	virtual const xmlbyte_t *getNsPrefix8() const { return 0; }
+	virtual const xmlbyte_t *getNsUri8() const { return 0; }
+	virtual const xmlbyte_t *getNsNodeValue8() const = 0;
+
+#if 0
 	// "extensions" beyond DOM spec
 	virtual uint32_t numNsChildNodes() const {
 		return 0;  // include text children
@@ -217,13 +216,30 @@ public:
 	virtual uint32_t numNsChildElements() const {
 		return 0; // only element children
 	}
+#endif
 	virtual uint32_t numNsAttributes() const {
 		return 0;
 	}
-	virtual const nid_t *getNodeId() const = 0;
+	virtual const NsNid *getNodeId() const = 0;
+	virtual const NsNid *getLastDescendantNid() const {
+		return getNsNode()->getLastDescendantNidOrSelf();
+	}
 	virtual int32_t getNsLevel() const = 0; // attrs return level of owner
-	virtual nsNode_t *getNsNode() const = 0;
+	virtual NsNode *getNsNode() const = 0;
 
+	virtual int32_t getIndex() const { return -1; }
+	
+	XER_NS MemoryManager *getMemoryManager() const {
+		return getNsDocument()->getMemoryManager();
+	}
+
+	// Node comparison
+	bool operator==(const NsDomNode &other) const;
+	bool operator!=(const NsDomNode &other) const {
+		return ((*this == other) == false);
+	}
+
+	// 
 	// utilities for use by DOM
 	// see DOM3 for semantics
 	static const bool compareNsNodes(const NsDomNode *node1,
@@ -236,7 +252,6 @@ public:
 					   NsDomElement *origEl);
 	static const xmlch_t *lookupUri(const NsDomNode *node,
 					const xmlch_t *prefix);
-	static const xmlch_t *lookupBaseUri(const NsDomNode *node);
 };
 
 //
@@ -251,85 +266,18 @@ public:
 	void setNsPrevSib(NsDomNav *sib) { _prevSib = sib; }
 	NsDomNav *getPrevSib() const { return _prevSib; }
 	NsDomNav *getNextSib() const { return _nextSib; }
+
+	virtual const xmlch_t *getNsBaseUri() const;
 protected:
 	NsDomElement *_parent;
 	NsDomNav *_prevSib;
 	NsDomNav *_nextSib;
 };
 
-template<class T>
-class Store {
-public:
-	Store(XER_NS MemoryManager *mmgr)
-		: _mmgr(mmgr), _size(0),
-		  _maxsize(0), _store(0) {}
-	~Store() { if(_store) _mmgr->deallocate(_store); }
-	T *&operator[](unsigned int i) {
-		if(i >= _size) resize(i + 1);
-		return _store[i];
-	}
-	const T *&operator[](unsigned int i) const {
-		return _store[i];
-	}
-	void push_back(T *val) {
-		if(_size == _maxsize)
-			reserve((_maxsize + 3) << 1);
-		_store[_size++] = val;
-	}
-	T *&back() {
-		return _store[_size - 1];
-	}
-	const T *&back() const {
-		return _store[_size - 1];
-	}
-	void pop_back() {
-		--_size;
-	}
-	void insert(unsigned int i, T *val) {
-		if(i > _size) resize(i);
-		if(_size == _maxsize)
-			reserve((_maxsize + 3) << 1);
-		T **pos = _store + i;
-		memmove(pos + 1, pos, (_size - i) * sizeof(T*));
-		*pos = val;
-		++_size;
-	}
-	unsigned int size() const { return _size; }
-	void resize(unsigned int i, T *val = 0) {
-		reserve(i);
-		for(unsigned int x = _size; x < i; ++x)
-			_store[x] = val;
-		_size = i;
-	}
-	void reserve(unsigned int i) {
-		if(_maxsize >= i) return;
-
-		T **newstore = (T**)_mmgr->
-			allocate(i * sizeof(T*));
-		if (!newstore)
-			NsUtil::nsThrowException(XmlException::NO_MEMORY_ERROR,
-			  "Unable to allocate memory for Store::reserve",
-			  __FILE__, __LINE__);
-		memcpy(newstore, _store, _size * sizeof(T*));
-
-		T **tmp = _store;
-		_store = newstore;
-		_maxsize = i;
-
-		if(tmp)
-			_mmgr->deallocate(tmp);
-	}
-private:
-	XER_NS MemoryManager *_mmgr;
-	unsigned int _size;
-	unsigned int _maxsize;
-	T **_store;
-};
-
-//
-// NsDomElement -- add nsNode_t and element tree links, as
-// well as NsDomNav child pointers, and an element name cache
-//
+/**
+ * NsDomElement -- add NsNode and element tree links, as
+ * well as NsDomNav child pointers, and an element name cache
+ */
 class NsDomElement : public NsDomNav {
 public:
 	//
@@ -347,24 +295,37 @@ public:
 	virtual void setNsPrefix(const xmlch_t *prefix);
 	virtual const xmlch_t *getNsUri() const;
 	virtual const xmlch_t *getNsNodeValue() const;
+
+	virtual const xmlch_t *getNsBaseUri() const;
+
+	virtual bool isNativeUTF8() const;
+	virtual const xmlbyte_t *getNsNodeName8() const;
+	virtual const xmlbyte_t *getNsLocalName8() const;
+	virtual const xmlbyte_t *getNsPrefix8() const;
+	virtual const xmlbyte_t *getNsUri8() const;
+	virtual const xmlbyte_t *getNsNodeValue8() const;
+
 	virtual NsDomNav *getNsFirstChild();
-	virtual NsDomNav *getNsLastChild();
-	virtual bool hasNsAttributes() const { return nsHasAttrs(_node); }
-	virtual bool hasNsChildNodes() const { return nsHasChildNode(_node); }
+	virtual NsDomNav *getNsLastChild(bool forUpdate = false);
+	virtual bool hasNsAttributes() const { return _node->hasAttributes(); }
+	virtual bool hasNsChildNodes() const { return _node->hasChildNode(); }
+	virtual bool hasNsChildElements() const { return _node->hasChildElem(); }
 	virtual NsDomNodeList *getNsChildNodes();
 	virtual NsNodeType_t getNsNodeType() const;
 	virtual NsDomElement *getNsParentNode();
 	virtual NsDomNav *getNsNextSibling();
-	virtual NsDomNav *getNsPrevSibling();
+	virtual NsDomNav *getNsPrevSibling(bool fetch = true);
 	virtual const xmlch_t *getNsTextContent() const;
 	virtual void setNsTextContent(const xmlch_t *value);
 	// NsDomNode extensions
+#if 0	
 	virtual uint32_t numNsChildNodes() const; // includes entity text nodes
 	virtual uint32_t numNsChildElements() const;
+#endif
 	virtual uint32_t numNsAttributes() const;
-	virtual const nid_t *getNodeId() const;
+	virtual const NsNid *getNodeId() const;
 	virtual int32_t getNsLevel() const;
-	nsNode_t *getNsNode() const { return _node; }
+	NsNode *getNsNode() const { return _node; }
 	void nsMakeTransient();
 	//
 	// methods introduced in NsDomElement
@@ -373,11 +334,22 @@ public:
 	NsDomNamedNodeMap *getNsAttributes();
 	NsDomNav *removeNsChild(NsDomNode *child);
 	// If refChild is null, the insert works like an append
-	NsDomNav *insertNsChild(NsDomNode *child, NsDomNode *refChild);
-
+	NsDomNav *insertNsChild(NsDomNode *child, NsDomNode *refChild,
+				NsDomInsertType itype);
+	bool isDocumentNode() const {
+		return _node->isDoc();
+	}
+	void setLastDescendantNid(const NsNid *nid) {
+		_node->setLastDescendantNid(getMemoryManager(),
+					    nid);
+	}
+	void updateLastDescendants(const NsNid *nid);
+	NsDomText *getNsTextNode(int index);
+	
 public: // these methods are for NsDom* internal use
-	NsDomElement(NsDocument *doc, nsNode_t *node, NsDomElement *parent,
+	NsDomElement(NsDocument *doc, NsNode *node, NsDomElement *parent,
 		     bool standalone);
+	virtual ~NsDomElement() {}
 	virtual void destroy();
 
 	/** Removes the descendants of this element, and schedules them for removal
@@ -386,14 +358,16 @@ public: // these methods are for NsDom* internal use
 
 	// these will fetch persistent node, if necessary
 	NsDomElement *getElemParent(bool fetch=true);
-	NsDomElement *getElemChild(int index, bool fetch=true);
 	NsDomElement *getElemFirstChild(bool fetch=true);
 	NsDomElement *getElemLastChild(bool fetch=true);
 	NsDomElement *getElemPrev(bool fetch=true);
 	NsDomElement *getElemNext(bool fetch=true);
-	int getIndex() const { return _clIndex; }
-	Store<NsDomElement> &getElemChildren() { return _elemChildren; }
-	Store<NsDomText> &getTextChildren() { return _textChildren; }
+	/**
+	 * Returns the descendant with the given NID.
+	 * This method returns null if the node does not exist.
+	 */
+	NsDomElement *lookupDescendantElement(const NsNid *nid, bool fetch=true);
+	NsDomElement *lookupElement(const NsNid *nid, bool fetch=true);
 
 	// set
 	void setElemParent(NsDomElement* node) { setNsParent(node); }
@@ -403,43 +377,55 @@ public: // these methods are for NsDom* internal use
 	void setElemLastChild(NsDomElement* node) {
 		_nsElemLastChild = node;
 	}
-	void setElemPrev(NsDomElement* node) { _nsElemPrev = node; }
-	void setElemNext(NsDomElement* node) { _nsElemNext = node; }
-	void setIndex(int i) { _clIndex = i; }
+	void setElemPrev(NsDomElement* node) {
+		_nsElemPrev = node;
+		if (node) {
+			node->_nsElemNext = this;
+			_node->setFlag(NS_HASPREV);
+			node->getNsNode()->setFlag(NS_HASNEXT);
+		}
+	}
+	void setElemNext(NsDomElement* node) {
+		_nsElemNext = node;
+		if (node) {
+			node->_nsElemPrev = this;
+			_node->setFlag(NS_HASNEXT);
+			node->getNsNode()->setFlag(NS_HASPREV);
+		}
+	}
 
-	// utility -- available to other classes
-	uint32_t getTextIndex();
-	NsDomNav* makeTextNodes(int startIndex, int endIndex,
-				int retIndex, NsDomNav *prev,
-				NsDomNav *next);
+	NsDomNav *makeChildTextNodes(NsDomNav *previous, bool returnLast);
+	NsDomNav *makeLeadingTextNodes(NsDomNav *previous, bool returnLast);
 protected:
-	NsDomElement *_insertNsElement(NsDomElement *child, NsDomNav *refChild);
+	void _moveTextNodes(NsDomText *start, NsDomElement *target);
+	void _coalesceTextNodes(NsDomElement *source,
+				NsDomElement *target, bool toChild);
+	NsDomElement *_insertNsElement(NsDomElement *child, NsDomNav *refChild,
+				       NsDomInsertType itype);
 	NsDomElement *_removeNsElement(NsDomElement *child);
 	NsDomText *_insertNsText(NsDomText *child, NsDomNav *refChild);
 	NsDomText *_removeNsText(NsDomText *child);
 	void _makeStandalone();
-	NsDomElement *_makeNonStandalone(NsDomElement *parent,
-					 int elemIndex,
-					 NsDomElement *previous,
-					 NsDomElement *next,
-					 NsDomElement *preceding,
-					 NsDomElement *following);
+	NsDomElement *_attachToTree(NsDomElement *parent,
+				    NsDomElement *previous,
+				    NsDomElement *next,
+				    const NsNid *preceding,
+				    const NsNid *following,
+				    NsDomInsertType itype);
 
+	void _addTreeToModifications(bool first);
+	
 	/// utility -- internal
 	void _getName() const;
 protected:
-	// NsDocument and nsNode_t
+	// NsDocument and NsNode
 	NsDocument *_document;
-	nsNode_t  *_node;
-	int _clIndex;
+	NsNode  *_node;
 
 	// name, first/lastchild.  Children are  nav nodes
 	NsDomNav *_firstChild;
 	NsDomNav *_lastChild;
 
-	Store<NsDomElement> _elemChildren;
-	Store<NsDomText> _textChildren;
-
 	NsDomNodeList *_childNodeList;
 	NsDomNamedNodeMap *_attributes;
 
@@ -447,6 +433,9 @@ protected:
 	mutable NsDomString _qname;
 	mutable NsDomString _textContent; // getTextContent result
 	mutable const xmlch_t *_lname;  // points into, or at _qname string
+	mutable NsDomString _baseUri;
+	mutable NsDomString _uri;
+	mutable NsDomString _prefix;
 protected:
 	// these links are populated during a transient DOM parse.
 	// they are also initialized during node materialization.
@@ -463,7 +452,8 @@ protected:
 //
 class NsDomText : public NsDomNav {
 public:
-	NsDomText(NsDomElement *parent, int index, uint32_t type);
+	NsDomText(NsDomElement *parent, NsDomElement *owner,
+		  int32_t index, uint32_t type);
 	/// Creates a standalone text type node
 	NsDomText(NsDocument *doc, const xmlch_t *value, uint32_t type);
 	/// Creates a standalone processing instuction type node
@@ -481,17 +471,23 @@ public:
 	virtual const xmlch_t *getNsLocalName() const;
 	virtual const xmlch_t *getNsNodeValue() const;
 	virtual void setNsNodeValue(const xmlch_t *value);
+
+	virtual bool isNativeUTF8() const;
+	virtual const xmlbyte_t *getNsNodeName8() const;
+	virtual const xmlbyte_t *getNsLocalName8() const;
+	virtual const xmlbyte_t *getNsNodeValue8() const;
+
 	virtual NsNodeType_t getNsNodeType() const;
 	virtual bool getNsIsEntityType() const;
 	virtual NsDomElement *getNsParentNode();
 	virtual NsDomNav *getNsNextSibling();
-	virtual NsDomNav *getNsPrevSibling();
+	virtual NsDomNav *getNsPrevSibling(bool fetch = true);
 	virtual int32_t getNsLevel() const;
-	virtual const nid_t *getNodeId() const;
-	virtual nsNode_t *getNsNode() const;
-
-	int getIndex() const { return _index; }
-	void setIndex(int index) { _index = index; }
+	virtual const NsNid *getNodeId() const;
+	virtual NsNode *getNsNode() const;
+	virtual int32_t getIndex() const { return _index; }
+	void setIndex(int32_t index) { _index = index; }
+	
 	uint32_t getNsTextType() const { return _type; }
 	bool nsIgnorableWhitespace() const;
 
@@ -499,16 +495,20 @@ public:
 	// Introduced in NsDomText
 	//
 	virtual void makeStandalone();
-	virtual void makeNonStandalone(NsDomElement *parent, int index,
+	virtual void makeNonStandalone(NsDomElement *parent,
+				       NsDomElement *owner, int index,
 				       NsDomNav *prevSib, NsDomNav *nextSib);
+	virtual NsDomElement *getOwner() const { return _owner; }
+	virtual void setOwner(NsDomElement *owner) { _owner = owner; }
 private:
 	const xmlch_t *_getText() const;
 	void _textSetNodeValue(const xmlch_t *value);
 	void _piSetNodeValue(const xmlch_t *value);
 protected:
 	NsDocument *_doc;
-	int _index;
+	NsDomElement *_owner; // owning node: may be parent or (later) sibling
 	uint32_t _type;
+	int32_t _index;
 	mutable NsDomString _text;
 	mutable NsDomString _value; ///< For processing instructions
 };
@@ -538,11 +538,22 @@ public:
 	virtual const xmlch_t *getNsUri() const;
 	virtual const xmlch_t *getNsNodeValue() const;
 	virtual void setNsNodeValue(const xmlch_t *value);
+
+	virtual const xmlch_t *getNsBaseUri() const;
+
+	virtual bool isNativeUTF8() const;
+	virtual const xmlbyte_t *getNsNodeName8() const;
+	virtual const xmlbyte_t *getNsLocalName8() const;
+	virtual const xmlbyte_t *getNsPrefix8() const;
+	virtual const xmlbyte_t *getNsUri8() const;
+	virtual const xmlbyte_t *getNsNodeValue8() const;
+
 	virtual NsDomElement *getNsParentNode() { return 0; }
 	virtual NsNodeType_t getNsNodeType() const { return nsNodeAttr; }
 	virtual int32_t getNsLevel() const { return _owner ? _owner->getNsLevel() : -1; }
-	virtual const nid_t *getNodeId() const;
-	virtual nsNode_t *getNsNode() const;
+	virtual const NsNid *getNodeId() const;
+	virtual NsNode *getNsNode() const;
+	virtual int32_t getIndex() const { return _index; }
 	//
 	// Introduced in NsDomAttr
 	//
@@ -551,16 +562,15 @@ public:
 	virtual bool isStandalone() const { return _owner == 0; }
 	virtual void makeStandalone();
 	virtual void makeNonStandalone(NsDomElement *owner, int index);
-	virtual int getIndex() const { return _index; }
-	virtual void setIndex(int index) { _index = index; }
 	void nsRename(const xmlch_t *uri, const xmlch_t *qname);
+	void setIndex(int32_t index) { _index = index; }
 private:
 	const void _getName() const;
 protected:
 	/// If _owner is null, this is a standalone attribute
 	NsDomElement *_owner;
 	NsDocument *_doc;
-	int _index;
+	int32_t _index;
 	mutable NsDomString _name;
 	mutable const xmlch_t *_lname;
 	/// when not standalone, _value points into _name string at value part
@@ -621,6 +631,7 @@ public:
 	NsDomNode *getNsItem(int index) const;
 protected:
 	NsDomElement *_owner;
+	mutable int _num;
 };
 
 //
@@ -661,20 +672,50 @@ inline NsDomFactory *NsDomNodeList::getNsDomFactory() const
 	return _owner->getNsDomFactory();
 }
 
-inline void NsDomString::set(const xmlch_t *text, bool owned)
+inline void NsDomString::set16(XER_NS MemoryManager *mmgr,
+			       const xmlch_t *text, bool owned)
 {
-	_text = text;
-	_owned = owned;
+	if (text_)
+		mmgr->deallocate((void *)text_);
+	text_ = (const void *)text;
+	flags_ = UTF16;
+	if (owned)
+		flags_ |= OWNED;
 }
 
-inline const xmlch_t *NsDomString::get() const
+inline void NsDomString::set(XER_NS MemoryManager *mmgr,
+			     const xmlbyte_t *text, bool owned)
 {
-	return _text;
+	if (text_)
+		mmgr->deallocate((void *)text_);
+	text_ = (const void *)text;
+	flags_ = 0;
+	if (owned)
+		flags_ |= OWNED;
 }
 
-inline const bool NsDomString::getOwned() const
+inline const xmlch_t *NsDomString::get16() const
 {
-	return _owned;
+	if (isUTF16())
+		return (const xmlch_t *) text_;
+	return NULL;
+}
+
+inline const xmlbyte_t *NsDomString::get() const
+{
+	if (!isUTF16())
+		return (const xmlbyte_t *) text_;
+	return NULL;
+}
+
+inline const bool NsDomString::isOwned() const
+{
+	return ((flags_ & OWNED) != 0);
+}
+
+inline const bool NsDomString::isUTF16() const
+{
+	return ((flags_ & UTF16) != 0);
 }
 
 }
