@@ -1,10 +1,9 @@
 //
 // See the file LICENSE for redistribution information.
 //
-// Copyright (c) 2002-2005
-//      Sleepycat Software.  All rights reserved.
+// Copyright (c) 2002,2006 Oracle.  All rights reserved.
 //
-// $Id: NsSAX2Reader.cpp,v 1.22 2005/04/20 18:31:32 bostic Exp $
+// $Id: NsSAX2Reader.cpp,v 1.33.4.1 2008/04/08 15:06:11 gmf Exp $
 //
 // The code in this file is derived from Apache's NsSAX2Reader code
 /*
@@ -37,9 +36,10 @@
 #include <xercesc/framework/XMLSchemaDescription.hpp>
 #include <xercesc/util/OutOfMemoryException.hpp>
 #include <xercesc/util/XMLEntityResolver.hpp>
+#include <xercesc/dom/DOMException.hpp>
 #include <string.h>
 
-#include <pathan/XPath2Utils.hpp>
+#include <xqilla/utils/XPath2Utils.hpp>
 
 #include "NsSAX2Reader.hpp"
 #include "NsUtil.hpp"
@@ -62,7 +62,7 @@ const XMLCh gEndExtSubset[] = {  chCloseAngle, chNull };
 
 using namespace DbXml;
 
-class NsEventAttrListXimpl : public NsEventAttrList {
+class NsEventAttrListXimpl : public NsEventAttrList16 {
 public:
 	NsEventAttrListXimpl(const RefVectorOf<XMLAttr> &list,
 			     const SAX2XMLReader &reader,
@@ -77,9 +77,6 @@ public:
 	virtual const xmlch_t *localName(int index) const {
 		return (_list.elementAt(index))->getName();
 	}
-	virtual const xmlch_t *qname(int index) {
-		return (_list.elementAt(index))->getQName();
-	}
 	virtual const xmlch_t *value(int index) const {
 		return (_list.elementAt(index))->getValue();
 	}
@@ -93,6 +90,7 @@ public:
 			return (_reader.getURIText(uriId));
 		return NULL;
 	}
+#ifdef NS_USE_SCHEMATYPES	
 	virtual bool hasTypeInfo(int index) const {
 		// TBD
 		return false;
@@ -105,15 +103,7 @@ public:
 		// TBD:  see AbstractDOMParser
 		return 0;
 	}
-	// don't call this function unless uri() is non-null
-	virtual const int uriId(int index) const {
-		const XMLAttr *attr = _list.elementAt(index);
-		int uriId = attr->getURIId();
-		if (XPath2Utils::equals(attr->getQName(),
-					XMLUni::fgXMLNSString))
-			uriId = 4; // hard-coded XXX
-		return uriId - DBXML_MIN_URI;
-	}
+#endif
 	virtual const bool needsEscape(int index) const {
 		return true;
 	}
@@ -216,9 +206,14 @@ void NsSAX2Reader::initialize(unsigned int flags)
 	// life, at the expense of performance.
 	// TBD: think about more flexible API to control
 	// this behavior
-	fScanner = XMLScannerResolver::resolveScanner(
-		XMLUni::fgIGXMLScanner, 0, fGrammarResolver,
-		fMemoryManager);
+	if (flags & NsParserEventSource::NS_PARSER_WF_ONLY)
+		fScanner = XMLScannerResolver::resolveScanner(
+			XMLUni::fgWFXMLScanner, 0, fGrammarResolver,
+			fMemoryManager);
+	else
+		fScanner = XMLScannerResolver::resolveScanner(
+			XMLUni::fgIGXMLScanner, 0, fGrammarResolver,
+			fMemoryManager);
 #else
 	if (validate) {
 		fScanner = XMLScannerResolver::resolveScanner(
@@ -352,7 +347,7 @@ void NsSAX2Reader::parse (const InputSource& source)
 	}
 	catch(const DbException &e) {
 		fParseInProgress = false;
-		throw XmlException(e.get_errno());
+		throw XmlException(e);
 	}
 	catch(const XmlException &) {
 		fParseInProgress = false;
@@ -508,9 +503,11 @@ void NsSAX2Reader::elementTypeInfo(const XMLCh* const typeName,
 	// this method is called after startElement, but
 	// before endElement, so it's necessary to rely on
 	// the fact that the "current" element is correct.
+#ifdef NS_USE_SCHEMATYPES	
 	if (typeName && *typeName) {
 		fHandler->setTypeInfo(typeName, typeURI);
 	}
+#endif
 }
 
 void NsSAX2Reader::XMLDecl(const XMLCh* const versionStr,
@@ -554,7 +551,7 @@ void NsSAX2Reader::resetDocument()
 
 void NsSAX2Reader::startDocument()
 {
-	fHandler->startDocument();
+	fHandler->startDocument(fScanner->getReaderMgr()->getCurrentEncodingStr());
 }
 
 void NsSAX2Reader::startElement(
@@ -569,14 +566,12 @@ void NsSAX2Reader::startElement(
 	NsEventAttrListXimpl alist(attrList, *this, attrCount);
 	const XMLCh *uri = 0;
 	unsigned int uriId = elemURLId;
-	if (uriId >= DBXML_MIN_URI && getDoNamespaces()) {
+	if (uriId >= DBXML_MIN_URI && getDoNamespaces())
 		uri = getURIText(elemURLId);
-		uriId -= DBXML_MIN_URI;
-	}
 	fHandler->startElement(elemDecl.getBaseName(),
 			       (elemPrefix && *elemPrefix) ? elemPrefix : 0,
-			       uri, uriId, &alist, attrCount,
-			       isEmpty, isRoot);
+			       uri, &alist, attrCount,
+			       isEmpty);
 
 	// Bump the element depth counter if not empty
 	if (!isEmpty)
@@ -589,12 +584,11 @@ void NsSAX2Reader::endElement(const XMLElementDecl& elemDecl,
 			       const XMLCh* const elemPrefix)
 {
 	const XMLCh *uri = 0;
-	if (uriId >= DBXML_MIN_URI && getDoNamespaces()) {
+	if (uriId >= DBXML_MIN_URI && getDoNamespaces())
 		uri = getURIText(uriId);
-	}
 	fHandler->endElement(elemDecl.getBaseName(),
 			     (elemPrefix && *elemPrefix) ? elemPrefix : 0,
-			     uri, isRoot);
+			     uri);
 	//
 	//  Dump the element depth down again. Don't let it underflow in case
 	//  of malformed XML.
@@ -741,17 +735,22 @@ void NsSAX2Reader::doctypeDecl(const DTDElementDecl& elemDecl,
 	fSubset.append(elemDecl.getFullName());
 	fSubset.append(chSpace);
 
-	if (systemId && *systemId) {
-		fSubset.append(XMLUni::fgSysIDString);
+	if (publicId && *publicId) {
+		fSubset.append(XMLUni::fgPubIDString);
 		fSubset.append(chSpace);
 		fSubset.append(chDoubleQuote);
-		fSubset.append(systemId);
+		fSubset.append(publicId);
 		fSubset.append(chDoubleQuote);
 		fSubset.append(chSpace);
 	}
-	if (publicId && *publicId) {
-		fSubset.append(publicId);
-		fSubset.append(chSpace);
+	if (systemId && *systemId) {
+		if (!(publicId && *publicId)) {
+			fSubset.append(XMLUni::fgSysIDString);
+			fSubset.append(chSpace);
+		}
+		fSubset.append(chDoubleQuote);
+		fSubset.append(systemId);
+		fSubset.append(chDoubleQuote);
 	}
 	if (hasIntSubset) {
 		fSubset.append(gStartIntSubset);
@@ -1009,10 +1008,14 @@ void NsSAX2Reader::error(const unsigned int code,
 			  const XMLSSize_t colNum)
 {
 	int len = NsUtil::nsStringLen(errorText);
-	xmlbyte_t buf[500];
-	xmlbyte_t *bufp = buf;
-	len = NsUtil::nsToUTF8((MemoryManager*)0, &bufp,
-			       errorText, len+1, 500);
+	xmlbyte_t *bufp = (xmlbyte_t *) fMemoryManager->allocate(500);
+	try {
+		len = NsUtil::nsToUTF8(fMemoryManager, &bufp,
+				       errorText, len+1, 500);
+	} catch (...) {
+		fMemoryManager->deallocate(bufp);
+		throw;
+	}
 
 	std::ostringstream s;
 	s << "XML Indexer: ";
@@ -1023,7 +1026,8 @@ void NsSAX2Reader::error(const unsigned int code,
 	s << " Parse error in document ";
 	s << "at line, " << lineNum;
 	s << ", char " << colNum;
-	s << ". Parser message: " << buf;
+	s << ". Parser message: " << bufp;
+	fMemoryManager->deallocate(bufp);
 	// log warnings as info, and errors as warning.
 	// Neither is fatal to the program, and may be
 	// what is expected.
