@@ -1,10 +1,9 @@
 //
 // See the file LICENSE for redistribution information.
 //
-// Copyright (c) 2002-2005
-//      Sleepycat Software.  All rights reserved.
+// Copyright (c) 2002,2006 Oracle.  All rights reserved.
 //
-// $Id: NsXercesDom.cpp,v 1.37 2005/05/10 22:35:22 gmf Exp $
+// $Id: NsXercesDom.cpp,v 1.53.2.1 2007/01/18 19:36:13 gmf Exp $
 //
 #include "NsUtil.hpp"
 #include "NsXercesDom.hpp"
@@ -17,8 +16,6 @@
 XERCES_CPP_NAMESPACE_USE
 using namespace DbXml;
 
-// forwards
-class DOMNodeIterator;
 
 void DbXml::NsXDOMNotImplemented(const char *name)
 {
@@ -64,7 +61,7 @@ static inline DOMElement *fakeDynamicCastDOMElement(NsDomNode *node)
 	if (!node)
 		return 0;
 
-	NS_ASSERT(node->getNsNodeType() == nsNodeElement)
+	DBXML_ASSERT(node->getNsNodeType() == nsNodeElement);
 	NsXDOMElement *el =
 		(NsXDOMElement*)node->getNsInterface("elem");
 	return el;
@@ -75,7 +72,7 @@ static inline DOMAttr *fakeDynamicCastDOMAttr(NsDomNode *node)
 	if (!node)
 		return 0;
 
-	NS_ASSERT(node->getNsNodeType() == nsNodeAttr);
+	DBXML_ASSERT(node->getNsNodeType() == nsNodeAttr);
 	NsXDOMAttr *at =
 		(NsXDOMAttr*)node->getNsInterface("attr");
 	return at;
@@ -86,8 +83,8 @@ static inline DOMCDATASection *fakeDynamicCastDOMText(NsDomNode *node)
 	if (!node)
 		return 0;
 
-	NS_ASSERT(node->getNsNodeType() == nsNodeText);
-	NS_ASSERT(nsTextType(((NsDomText*)node)->getNsTextType()) == NS_ENTSTART ||
+	DBXML_ASSERT(node->getNsNodeType() == nsNodeText);
+	DBXML_ASSERT(nsTextType(((NsDomText*)node)->getNsTextType()) == NS_ENTSTART ||
 		  nsTextType(((NsDomText*)node)->getNsTextType()) == NS_ENTEND ||
 		  nsTextType(((NsDomText*)node)->getNsTextType()) == NS_SUBSET ||
 		  nsTextType(((NsDomText*)node)->getNsTextType()) == NS_CDATA ||
@@ -102,8 +99,8 @@ static inline DOMComment *fakeDynamicCastDOMComment(NsDomNode *node)
 	if (!node)
 		return 0;
 
-	NS_ASSERT(node->getNsNodeType() == nsNodeText);
-	NS_ASSERT(nsTextType(((NsDomText*)node)->getNsTextType()) == NS_COMMENT);
+	DBXML_ASSERT(node->getNsNodeType() == nsNodeText);
+	DBXML_ASSERT(nsTextType(((NsDomText*)node)->getNsTextType()) == NS_COMMENT);
 	NsXDOMComment *at =
 		(NsXDOMComment*)node->getNsInterface("comment");
 	return at;
@@ -114,14 +111,14 @@ static inline DOMProcessingInstruction *fakeDynamicCastDOMProcessingInstruction(
 	if (!node)
 		return 0;
 
-	NS_ASSERT(node->getNsNodeType() == nsNodeText);
-	NS_ASSERT(nsTextType(((NsDomText*)node)->getNsTextType()) == NS_PINST);
+	DBXML_ASSERT(node->getNsNodeType() == nsNodeText);
+	DBXML_ASSERT(nsTextType(((NsDomText*)node)->getNsTextType()) == NS_PINST);
 	NsXDOMProcessingInstruction *at =
 		(NsXDOMProcessingInstruction*)node->getNsInterface("pi");
 	return at;
 }
 
-static DOMNode *fakeDynamicCastDOMNode(NsDomNode *node)
+DOMNode *DbXml::fakeDynamicCastDOMNode(NsDomNode *node)
 {
 	if (!node)
 		return 0;
@@ -156,7 +153,7 @@ static DOMNode *fakeDynamicCastDOMNode(NsDomNode *node)
 		case NS_PINST:
 			return (NsXDOMProcessingInstruction*)node->getNsInterface("pi");
 		default:
-			assert(false);
+			DBXML_ASSERT(false);
 		}
 	}
 	case nsNodeDocument:
@@ -189,21 +186,8 @@ static inline DOMNodeList *fakeDynamicCastDOMNodeList(NsDomNodeList *list)
 	return dlist;
 }
 
-static inline NsDomNode *fakeDynamicCastNsDomNode(DOMNode *node)
-{
-	if (!node) return 0;
-	return (NsDomNode*)node->getInterface(_nsDomString);
-}
-
-static inline const NsDomNode *fakeDynamicCastNsDomNode(const DOMNode *node)
-{
-	if (!node) return 0;
-	return (NsDomNode*)
-		const_cast<DOMNode*>(node)->getInterface(_nsDomString);
-}
-
 //
-// DOMTypeInfo class to make Pathan happy
+// DOMTypeInfo class to make XQilla happy
 //
 class NsXDOMTypeInfo : public DOMTypeInfo
 {
@@ -227,21 +211,30 @@ NsXDOMFactory::CastToDOMElement(NsDomElement *node)
 	return fakeDynamicCastDOMElement(node);
 }
 
+DOMDocument *
+NsXDOMFactory::getDocumentNode(NsDocument *nsdoc)
+{
+	if(documentNode_ == 0) {
+		nsdoc->getDocumentNode();
+	}
+	return documentNode_;
+}
+
 NsDomElement *
 NsXDOMFactory::createNsDomElement(NsDocument *doc,
-				  nsNode_t *node,
+				  NsNode *node,
 				  NsDomElement *parent,
 				  bool standalone)
 {
 	// handle both document elements and "normal" elements
-	if (nsIsDoc(node)) {
+	if (node->isDoc()) {
 		NsXDOMDocument *de = (NsXDOMDocument *) _memManager->allocate(
 			sizeof(NsXDOMDocument));
 		if (!de)
 			NsXDOMNoMemory("createNsDomDocument");
 
 		(void) new (de) NsXDOMDocument(doc, node);
-		setNsDocumentObj(de);
+		documentNode_ = de;
 		addToDomFreeList(de);
 		return de;
 	} else {
@@ -258,6 +251,7 @@ NsXDOMFactory::createNsDomElement(NsDocument *doc,
 
 NsDomText *
 NsXDOMFactory::createNsDomText(NsDomElement *parent,
+			       NsDomElement *owner,
 			       int index, uint32_t type)
 {
 	switch (nsTextType(type)) {
@@ -269,7 +263,7 @@ NsXDOMFactory::createNsDomText(NsDomElement *parent,
 		NsXDOMText *dt = (NsXDOMText *)_memManager->
 			allocate(sizeof(NsXDOMText));
 		if (!dt) NsXDOMNoMemory("createNsDomText");
-		(void) new (dt) NsXDOMText(parent, index, type);
+		(void) new (dt) NsXDOMText(parent, owner, index, type);
 		addToDomFreeList(dt);
 		return dt;
 	}
@@ -277,7 +271,7 @@ NsXDOMFactory::createNsDomText(NsDomElement *parent,
 		NsXDOMComment *dt = (NsXDOMComment *)_memManager->
 			allocate(sizeof(NsXDOMComment));
 		if (!dt) NsXDOMNoMemory("createNsDomText");
-		(void) new (dt) NsXDOMComment(parent, index);
+		(void) new (dt) NsXDOMComment(parent, owner, index);
 		addToDomFreeList(dt);
 		return dt;
 	}
@@ -286,12 +280,12 @@ NsXDOMFactory::createNsDomText(NsDomElement *parent,
 			(NsXDOMProcessingInstruction *)_memManager->
 			allocate(sizeof(NsXDOMProcessingInstruction));
 		if (!dt) NsXDOMNoMemory("createNsDomText");
-		(void) new (dt) NsXDOMProcessingInstruction(parent, index);
+		(void) new (dt) NsXDOMProcessingInstruction(parent, owner, index);
 		addToDomFreeList(dt);
 		return dt;
 	}
 	default:
-		NS_ASSERT(0);
+		DBXML_ASSERT(0);
 	}
 	return 0;
 }
@@ -323,7 +317,7 @@ NsXDOMFactory::createNsDomText(NsDocument *doc, const xmlch_t *value,
 	}
 	case NS_PINST:
 	default:
-		NS_ASSERT(0);
+		DBXML_ASSERT(0);
 	}
 	return 0;
 }
@@ -412,16 +406,14 @@ NsXDOMFactory::createNsDomFakeText(NsDocument *doc, const xmlch_t *value,
 void
 NsXDOMFactory::destroyDomList()
 {
-	if (_dealloc) {
-		NsDomObj *list = _domFreeList;
-		while (list) {
-			NsDomObj *node = list->getNextObj();
+	NsDomObj *list = _domFreeList;
+	while (list) {
+		NsDomObj *node = list->getNextObj();
 
-			list->destroy(); // acts as destructor
-			_memManager->deallocate(
-				list->getNsInterface("destroy"));
-			list = node;
-		}
+		list->destroy(); // acts as destructor
+		_memManager->deallocate(
+			list->getNsInterface("destroy"));
+		list = node;
 	}
 	_domFreeList = 0;
 }
@@ -461,35 +453,41 @@ NsXDOMDocument::getDocumentElement() const
 	// This is the root element -- first, and only child of
 	// this (Document) node -- it must exist.
 	NsDomElement *elem = const_cast<NsXDOMDocument*>(this)->getElemFirstChild();
-	NS_ASSERT(elem)
+	DBXML_ASSERT(elem);
 	return fakeDynamicCastDOMElement(elem);
 }
 
 const XMLCh *
 NsXDOMDocument::getVersion() const
 {
-	if (getNsDocument()->getXmlDecl())
-		return getNsDocument()->getXmlDecl();
+	if (getNsDocument()->getXmlDecl16())
+		return getNsDocument()->getXmlDecl16();
 	return _decl_1_0;
 }
 
 const XMLCh *
 NsXDOMDocument::getDocumentURI() const
 {
-	return 0;
+	return _document->getDocumentUri();
+}
+
+void
+NsXDOMDocument::setDocumentURI(const XMLCh* documentURI)
+{
+	_document->setDocumentUri(documentURI);
 }
 
 const XMLCh *
 NsXDOMDocument::getEncoding() const
 {
-	return getNsDocument()->getEncodingStr();
+	return getNsDocument()->getEncodingStr16();
 }
 
 const XMLCh *
 NsXDOMDocument::getActualEncoding() const
 {
 	// actual is same as encoding in NsDocument (TBD: fix?)
-	return getNsDocument()->getEncodingStr();
+	return getNsDocument()->getEncodingStr16();
 }
 
 bool
@@ -767,7 +765,7 @@ NsXDOMDocument::getNodeValue() const
 short
 NsXDOMDocument::getNodeType() const
 {
-	NS_ASSERT(getNsNodeType() == nsNodeDocument)
+	DBXML_ASSERT(getNsNodeType() == nsNodeDocument);
 	return DOMNode::DOCUMENT_NODE;
 }
 
@@ -832,7 +830,7 @@ NsXDOMDocument::cloneNode(bool deep) const
 DOMNode *
 NsXDOMDocument::removeChild(DOMNode *oldChild)
 {
-	if(oldChild->getParentNode() != this) {
+	if (!isSameNode(oldChild->getParentNode())) {
 		throw XNSQ DOMException(XNSQ DOMException::NOT_FOUND_ERR, 0, getNsDocument()->getMemoryManager());
 	}
 	NsDomNode *child = fakeDynamicCastNsDomNode(oldChild);
@@ -855,7 +853,7 @@ NsXDOMDocument::appendChild(DOMNode *newChild)
 		newChild = newChild->getParentNode()->removeChild(newChild);
 		child = fakeDynamicCastNsDomNode(newChild);
 	}
-	return fakeDynamicCastDOMNode(insertNsChild(child, 0));
+	return fakeDynamicCastDOMNode(insertNsChild(child, 0, nsDomInsertAppend));
 }
 
 DOMNode *
@@ -867,7 +865,7 @@ NsXDOMDocument::insertBefore(DOMNode *newChild, DOMNode *refChild)
 		throw XNSQ DOMException(XNSQ DOMException::WRONG_DOCUMENT_ERR, 0, getNsDocument()->getMemoryManager());
 	}
 
-	if(refChild->getParentNode() != this) {
+	if(!isSameNode(refChild->getParentNode())) {
 		throw XNSQ DOMException(XNSQ DOMException::NOT_FOUND_ERR, 0, getNsDocument()->getMemoryManager());
 	}
 
@@ -884,7 +882,7 @@ NsXDOMDocument::insertBefore(DOMNode *newChild, DOMNode *refChild)
 		child = fakeDynamicCastNsDomNode(newChild);
 	}
 	NsDomNode *rchild = fakeDynamicCastNsDomNode(refChild);
-	return fakeDynamicCastDOMNode(insertNsChild(child, rchild));
+	return fakeDynamicCastDOMNode(insertNsChild(child, rchild, nsDomInsertBefore));
 }
 
 DOMNode *
@@ -927,13 +925,16 @@ NsXDOMDocument::hasAttributes() const
 bool
 NsXDOMDocument::isSameNode(const DOMNode* other) const
 {
-	return ((const DOMNode *)this == other);
+	if ((const DOMNode *)this == other)
+		return true;
+	else
+		return operator==(*fakeDynamicCastNsDomNodeC(other));
 }
 
 bool
 NsXDOMDocument::isEqualNode(const DOMNode* arg) const
 {
-	const NsDomNode *thisNode = fakeDynamicCastNsDomNode(this);
+	const NsDomNode *thisNode = fakeDynamicCastNsDomNodeC(this);
 	return compareNsNodes(thisNode, (const NsDomNode *)
 			      (((DOMNode*)arg)->getInterface(_nsDomString)));
 }
@@ -941,22 +942,21 @@ NsXDOMDocument::isEqualNode(const DOMNode* arg) const
 const XMLCh *
 NsXDOMDocument::getBaseURI() const
 {
-	const NsDomNode *thisNode = fakeDynamicCastNsDomNode(this);
-	return lookupBaseUri(thisNode);
+	return _document->getDocumentUri();
 }
 
 static short
 _compareNsPositions(const DOMNode *node1, const DOMNode *node2)
 {
 	if(node1 == 0 || node2 == 0 ||
-	   (node1->getNodeType() == DOMNode::DOCUMENT_NODE ? node1 : node1->getOwnerDocument()) !=
-	   (node2->getNodeType() == DOMNode::DOCUMENT_NODE ? node2 : node2->getOwnerDocument())) {
+	   !(node1->getNodeType() == DOMNode::DOCUMENT_NODE ? node1 : node1->getOwnerDocument())->
+		isSameNode(node2->getNodeType() == DOMNode::DOCUMENT_NODE ? node2 : node2->getOwnerDocument())) {
 		return DOMNode::TREE_POSITION_DISCONNECTED;
 	}
 
 	short nsret = NsDomNode::compareNsPositions(
-		fakeDynamicCastNsDomNode(node1),
-		fakeDynamicCastNsDomNode(node2));
+		fakeDynamicCastNsDomNodeC(node1),
+		fakeDynamicCastNsDomNodeC(node2));
 	return nsret;
 }
 
@@ -1003,7 +1003,7 @@ DOMText *
 NsXDOMDocument::createDocumentTypeTextNode(DOMNode *docTypeNode)
 {
 #if 1
-	// TBD
+	// TBD (not used/needed at this time)
 	return 0;
 #else
 	// serialize the information
@@ -1207,7 +1207,7 @@ NsXDOMElement::getAttributes() const
 DOMDocument *
 NsXDOMElement::getOwnerDocument() const
 {
-	return (DOMDocument *) ((NsXDOMElement*)this)->getNsDocumentObj();
+	return (NsXDOMDocument *) ((NsXDOMElement*)this)->getNsDocumentObj();
 }
 
 DOMNode *
@@ -1250,7 +1250,7 @@ NsXDOMElement::appendChild(DOMNode *newChild)
 		newChild = newChild->getParentNode()->removeChild(newChild);
 		child = fakeDynamicCastNsDomNode(newChild);
 	}
-	return fakeDynamicCastDOMNode(insertNsChild(child, 0));
+	return fakeDynamicCastDOMNode(insertNsChild(child, 0, nsDomInsertAppend));
 }
 
 DOMNode *
@@ -1285,7 +1285,43 @@ NsXDOMElement::insertBefore(DOMNode *newChild, DOMNode *refChild)
 		child = fakeDynamicCastNsDomNode(newChild);
 	}
 	NsDomNode *rchild = fakeDynamicCastNsDomNode(refChild);
-	return fakeDynamicCastDOMNode(insertNsChild(child, rchild));
+	return fakeDynamicCastDOMNode(insertNsChild(child, rchild, nsDomInsertBefore));
+}
+
+DOMNode *
+NsXDOMElement::insertBefore(DOMNode *newChild, DOMNode *refChild,
+			    NsDomInsertType itype)
+{
+	if(refChild == 0) return appendChild(newChild);
+
+	if(newChild->getOwnerDocument() != getOwnerDocument()) {
+		throw XNSQ DOMException(XNSQ DOMException::WRONG_DOCUMENT_ERR, 0, getNsDocument()->getMemoryManager());
+	}
+
+	if (!isSameNode(refChild->getParentNode())) {
+		throw XNSQ DOMException(XNSQ DOMException::NOT_FOUND_ERR, 0, getNsDocument()->getMemoryManager());
+	}
+
+	NsDomNode *child = fakeDynamicCastNsDomNode(newChild);
+	if(child->getNsNodeType() != nsNodeElement &&
+	   child->getNsNodeType() != nsNodeText) {
+		throw XNSQ DOMException(XNSQ DOMException::HIERARCHY_REQUEST_ERR, 0, getNsDocument()->getMemoryManager());
+	}
+	DOMNode *parent = this;
+	while(parent != 0) {
+		if(parent == newChild)
+			throw XNSQ DOMException(XNSQ DOMException::HIERARCHY_REQUEST_ERR, 0, getNsDocument()->getMemoryManager());
+		parent = parent->getParentNode();
+	}
+
+	if(newChild == refChild) return newChild;
+
+	if(newChild->getParentNode() != 0) {
+		newChild = newChild->getParentNode()->removeChild(newChild);
+		child = fakeDynamicCastNsDomNode(newChild);
+	}
+	NsDomNode *rchild = fakeDynamicCastNsDomNode(refChild);
+	return fakeDynamicCastDOMNode(insertNsChild(child, rchild, itype));
 }
 
 DOMNode *
@@ -1336,13 +1372,16 @@ NsXDOMElement::hasAttributes() const
 bool
 NsXDOMElement::isSameNode(const DOMNode* other) const
 {
-	return ((const DOMNode *)this == other);
+	if ((const DOMNode *)this == other)
+		return true;
+	else
+		return operator==(*fakeDynamicCastNsDomNodeC(other));
 }
 
 bool
 NsXDOMElement::isEqualNode(const DOMNode* arg) const
 {
-	const NsDomNode *thisNode = fakeDynamicCastNsDomNode(this);
+	const NsDomNode *thisNode = fakeDynamicCastNsDomNodeC(this);
 	return compareNsNodes(thisNode, (const NsDomNode *)
 			      (((DOMNode*)arg)->getInterface(_nsDomString)));
 }
@@ -1350,8 +1389,7 @@ NsXDOMElement::isEqualNode(const DOMNode* arg) const
 const XMLCh *
 NsXDOMElement::getBaseURI() const
 {
-	const NsDomNode *thisNode = fakeDynamicCastNsDomNode(this);
-	return lookupBaseUri(thisNode);
+	return getNsBaseUri();
 }
 
 short
@@ -1377,14 +1415,14 @@ NsXDOMElement::lookupNamespacePrefix(const XMLCh* namespaceURI,
 				     bool useDefault) const
 {
 
-	NsDomElement *thisElem = (NsDomElement*)fakeDynamicCastNsDomNode(this);
+	NsDomElement *thisElem = (NsDomElement*)fakeDynamicCastNsDomNodeC(this);
 	return lookupPrefix(thisElem, namespaceURI, useDefault, thisElem);
 }
 
 const XMLCh *
 NsXDOMElement::lookupNamespaceURI(const XMLCh* prefix) const
 {
-	const NsDomNode *thisNode = fakeDynamicCastNsDomNode(this);
+	const NsDomNode *thisNode = fakeDynamicCastNsDomNodeC(this);
 	return lookupUri(thisNode, prefix);
 }
 
@@ -1636,7 +1674,7 @@ NsXDOMAttr::getAttributes() const
 DOMDocument *
 NsXDOMAttr::getOwnerDocument() const
 {
-	return (DOMDocument *) (const_cast<NsXDOMAttr*>(this))->
+	return (NsXDOMDocument *) (const_cast<NsXDOMAttr*>(this))->
 		getNsDocumentObj();
 }
 
@@ -1718,14 +1756,17 @@ NsXDOMAttr::hasAttributes() const
 bool
 NsXDOMAttr::isSameNode(const DOMNode* other) const
 {
-	return ((const DOMNode *)this == other);
+	if ((const DOMNode *)this == other)
+		return true;
+	else
+		return operator==(*fakeDynamicCastNsDomNodeC(other));
 }
 
 bool
 NsXDOMAttr::isEqualNode(const DOMNode* arg) const
 {
 	NsDomAttr *thisAttr = (NsDomAttr*)
-		fakeDynamicCastNsDomNode(this);
+		fakeDynamicCastNsDomNodeC(this);
 	return compareNsNodes(thisAttr, (const NsDomNode *)
 			      (((DOMNode*)arg)->getInterface(_nsDomString)));
 }
@@ -1733,9 +1774,7 @@ NsXDOMAttr::isEqualNode(const DOMNode* arg) const
 const XMLCh *
 NsXDOMAttr::getBaseURI() const
 {
-	NsDomAttr *thisAttr = (NsDomAttr*)
-		fakeDynamicCastNsDomNode(this);
-	return lookupBaseUri(thisAttr);
+	return getNsBaseUri();
 }
 
 short
@@ -1760,7 +1799,7 @@ const XMLCh *
 NsXDOMAttr::lookupNamespacePrefix(const XMLCh* namespaceURI,
 				  bool useDefault) const
 {
-	NsDomAttr *thisAttr = (NsDomAttr*)fakeDynamicCastNsDomNode(this);
+	NsDomAttr *thisAttr = (NsDomAttr*)fakeDynamicCastNsDomNodeC(this);
 	NsDomElement *owner = thisAttr->getNsOwner();
 	return lookupPrefix(owner, namespaceURI, useDefault, owner);
 }
@@ -1815,7 +1854,7 @@ NsXDOMText::getNodeType() const
 		// not appear in NsXDOM?  Probably not.
 		return DOMNode::TEXT_NODE;
 	default:
-		NS_ASSERT(0);
+		DBXML_ASSERT(0);
 	}
 	return 0;
 }
@@ -1868,7 +1907,7 @@ NsXDOMText::getAttributes() const
 DOMDocument *
 NsXDOMText::getOwnerDocument() const
 {
-	return (DOMDocument *) const_cast<NsXDOMText*>(this)->
+	return (NsXDOMDocument *) const_cast<NsXDOMText*>(this)->
 		getNsDocumentObj();
 }
 
@@ -1942,14 +1981,17 @@ NsXDOMText::hasAttributes() const
 bool
 NsXDOMText::isSameNode(const DOMNode* other) const
 {
-	return ((const DOMNode *)this == other);
+	if ((const DOMNode *)this == other)
+		return true;
+	else
+		return operator==(*fakeDynamicCastNsDomNodeC(other));
 }
 
 bool
 NsXDOMText::isEqualNode(const DOMNode* arg) const
 {
 	NsDomText *thisText = (NsDomText*)
-		fakeDynamicCastNsDomNode(this);
+		fakeDynamicCastNsDomNodeC(this);
 	return compareNsNodes(thisText, (const NsDomNode *)
 			      (((DOMNode*)arg)->getInterface(_nsDomString)));
 }
@@ -1957,9 +1999,7 @@ NsXDOMText::isEqualNode(const DOMNode* arg) const
 const XMLCh *
 NsXDOMText::getBaseURI() const
 {
-	NsDomText *thisText = (NsDomText*)
-		fakeDynamicCastNsDomNode(this);
-	return lookupBaseUri(thisText);
+	return getNsBaseUri();
 }
 
 short
@@ -2099,7 +2139,7 @@ NsXDOMComment::getAttributes() const
 DOMDocument *
 NsXDOMComment::getOwnerDocument() const
 {
-	return (DOMDocument *) const_cast<NsXDOMComment*>(this)->
+	return (NsXDOMDocument *) const_cast<NsXDOMComment*>(this)->
 		getNsDocumentObj();
 }
 
@@ -2173,14 +2213,17 @@ NsXDOMComment::hasAttributes() const
 bool
 NsXDOMComment::isSameNode(const DOMNode* other) const
 {
-	return ((const DOMNode *)this == other);
+	if ((const DOMNode *)this == other)
+		return true;
+	else
+		return operator==(*fakeDynamicCastNsDomNodeC(other));
 }
 
 bool
 NsXDOMComment::isEqualNode(const DOMNode* arg) const
 {
 	NsDomText *thisText = (NsDomText*)
-		fakeDynamicCastNsDomNode(this);
+		fakeDynamicCastNsDomNodeC(this);
 	return compareNsNodes(thisText, (const NsDomNode *)
 			      (((DOMNode*)arg)->getInterface(_nsDomString)));
 }
@@ -2188,9 +2231,7 @@ NsXDOMComment::isEqualNode(const DOMNode* arg) const
 const XMLCh *
 NsXDOMComment::getBaseURI() const
 {
-	NsDomText *thisText = (NsDomText*)
-		fakeDynamicCastNsDomNode(this);
-	return lookupBaseUri(thisText);
+	return getNsBaseUri();
 }
 
 short
@@ -2321,7 +2362,7 @@ NsXDOMProcessingInstruction::getAttributes() const
 DOMDocument *
 NsXDOMProcessingInstruction::getOwnerDocument() const
 {
-	return (DOMDocument *) const_cast<NsXDOMProcessingInstruction*>(this)->
+	return (NsXDOMDocument *) const_cast<NsXDOMProcessingInstruction*>(this)->
 		getNsDocumentObj();
 }
 
@@ -2396,13 +2437,16 @@ NsXDOMProcessingInstruction::hasAttributes() const
 bool
 NsXDOMProcessingInstruction::isSameNode(const DOMNode* other) const
 {
-	return ((const DOMNode *)this == other);
+	if ((const DOMNode *)this == other)
+		return true;
+	else
+		return operator==(*fakeDynamicCastNsDomNodeC(other));
 }
 
 bool
 NsXDOMProcessingInstruction::isEqualNode(const DOMNode* arg) const
 {
-	NsDomText *thisText = (NsDomText *)fakeDynamicCastNsDomNode(this);
+	NsDomText *thisText = (NsDomText *)fakeDynamicCastNsDomNodeC(this);
 	return compareNsNodes(thisText, (const NsDomNode *)
 			      (((DOMNode*)arg)->getInterface(_nsDomString)));
 }
@@ -2410,8 +2454,7 @@ NsXDOMProcessingInstruction::isEqualNode(const DOMNode* arg) const
 const XMLCh *
 NsXDOMProcessingInstruction::getBaseURI() const
 {
-	NsDomText *thisText = (NsDomText *)fakeDynamicCastNsDomNode(this);
-	return lookupBaseUri(thisText);
+	return getNsBaseUri();
 }
 
 short
