@@ -1,10 +1,9 @@
 //
 // See the file LICENSE for redistribution information.
 //
-// Copyright (c) 2002-2005
-//      Sleepycat Software.  All rights reserved.
+// Copyright (c) 2002,2006 Oracle.  All rights reserved.
 //
-// $Id: NsXercesDom.hpp,v 1.28 2005/05/08 22:33:55 gmf Exp $
+// $Id: NsXercesDom.hpp,v 1.42 2006/10/30 17:45:57 bostic Exp $
 //
 #ifndef __NSXERCESDOM_HPP
 #define __NSXERCESDOM_HPP
@@ -24,7 +23,7 @@
  *   not shareable, this makes sense.
  *   o there are no implementations for comment, cdata, or processing
  *   instruction nodes.  These are all implemented inside NsXDOMText,
- *   which is typed.  Pathan and XQuery never ask for instances of
+ *   which is typed.  XQilla and XQuery never ask for instances of
  *   these types, and instead rely on Node methods.
  *
  * Implementation note:  methods are not implemented/supported are
@@ -55,7 +54,8 @@ namespace DbXml
 
 class NsXDOMDocument;
 
-static const XMLCh _nsDomString[] = { 'N', 's', 'D', 'o', 'm', 0 };
+static const xmlch_t _nsDomString[] = { 'N', 's', 'D', 'o', 'm', 0 };
+static const xmlch_t _nsXDomString[] = { 'N', 's', 'X', 'D', 'o', 'm', 0 };
 
 //
 // Error functions -- will throw
@@ -70,7 +70,7 @@ class NsXDOMDocument: public XNSQ DOMDocument,
 		      public NsDomElement {
 	friend class NsXDOMFactory;
 public:
-	NsXDOMDocument(NsDocument *doc, nsNode_t *node) :
+	NsXDOMDocument(NsDocument *doc, NsNode *node) :
 		NsDomElement(doc, node, 0 /* parent */, false) {};
 
 public:
@@ -85,6 +85,7 @@ public:
 	virtual XNSQ DOMElement *getDocumentElement() const;
 	virtual const XMLCh *getVersion() const;
 	virtual const XMLCh *getDocumentURI() const;
+	virtual void setDocumentURI(const XMLCh* documentURI);
 	virtual bool getStandalone() const;
 
 	//
@@ -159,11 +160,13 @@ public:
 		// This interface is hijacked to cast from DOMNode *
 		// to NsDomNode *.  The feature string must be
 		// "NsDom"
-		if (NsUtil::nsStringEqual(feature, _nsDomString))
-			return (XNSQ DOMNode *)((NsDomNode*)this);
+		if (feature) {
+			if (NsUtil::nsStringEqual((const xmlch_t *)feature,
+						  _nsDomString))
+				return (XNSQ DOMNode *)((NsDomNode*)this);
+		}
 		return 0;
 	}
-
 	// For NsDom extensions, use the
 	// NsDomNode methods, such as numNsAttributes(), nodeId(), etc.
 	// directly
@@ -238,9 +241,6 @@ public:
 	virtual void setVersion(const XMLCh* version) {
 		NsXDOMNotImplemented("setVersion");
 	}
-	virtual void setDocumentURI(const XMLCh* documentURI) {
-		NsXDOMNotImplemented("setDocumentURI");
-	}
 	virtual bool getStrictErrorChecking() const {
 		NsXDOMNotImplemented("getStrictErrorChecking"); return false;
 	}
@@ -316,7 +316,7 @@ class NsXDOMElement: public XNSQ DOMElement,
 		     public NsDomElement {
 	friend class NsXDOMFactory;
 public:
-	NsXDOMElement(NsDocument *doc, nsNode_t *node,
+	NsXDOMElement(NsDocument *doc, NsNode *node,
 		      NsDomElement *parent, bool standalone) :
 		NsDomElement(doc, node, parent, standalone) {};
 
@@ -400,8 +400,14 @@ public:
 		// This interface is hijacked to cast from DOMNode *
 		// to NsDomNode *.  The feature string must be
 		// "NsDom"
-		if (NsUtil::nsStringEqual(feature, _nsDomString))
-			return (XNSQ DOMNode *)((NsDomNode*)this);
+		if (feature) {
+			if (NsUtil::nsStringEqual((const xmlch_t *)feature,
+						  _nsDomString))
+				return (XNSQ DOMNode *)((NsDomNode*)this);
+			if (NsUtil::nsStringEqual((const xmlch_t *)feature,
+						  _nsXDomString))
+				return this;
+		}
 		return 0;
 	}
 
@@ -410,6 +416,11 @@ public:
 	// directly
 	virtual void *getNsInterface(const char *name) { return this; }
 
+	// NsXDOM extension (not virtual)
+	XNSQ DOMNode *insertBefore(XNSQ DOMNode *newChild,
+				   XNSQ DOMNode *refChild,
+				   NsDomInsertType itype);
+
 	//
 	// unimplemented DOMNode methods
 	//
@@ -536,7 +547,8 @@ public:
 		// This interface is hijacked to cast from DOMNode *
 		// to NsDomNode *.  The feature string must be
 		// "NsDom"
-		if (NsUtil::nsStringEqual(feature, _nsDomString))
+		if (NsUtil::nsStringEqual((const xmlch_t *)feature,
+					  _nsDomString))
 			return (XNSQ DOMNode *)((NsDomNode*)this);
 		return 0;
 	}
@@ -576,8 +588,9 @@ public:
 class NsXDOMText: public XNSQ DOMCDATASection, public NsDomText {
 	friend class NsXDOMFactory;
 public:
-	NsXDOMText(NsDomElement *parent, int index, uint32_t type)
-		: NsDomText(parent, index, type) {}
+	NsXDOMText(NsDomElement *parent, NsDomElement *owner,
+		   int index, uint32_t type)
+		: NsDomText(parent, owner, index, type) {}
 	NsXDOMText(NsDocument *doc, const xmlch_t *value, uint32_t type)
 		: NsDomText(doc, value, type) {}
 public:
@@ -633,7 +646,8 @@ public:
 		// This interface is hijacked to cast from DOMNode *
 		// to NsDomNode *.  The feature string must be
 		// "NsDom"
-		if (NsUtil::nsStringEqual(feature, _nsDomString))
+		if (NsUtil::nsStringEqual((const xmlch_t *)feature,
+					  _nsDomString))
 			return (XNSQ DOMNode *)((NsDomNode*)this);
 		return 0;
 	}
@@ -713,8 +727,8 @@ public:
 class NsXDOMComment: public XNSQ DOMComment, public NsDomText {
 	friend class NsXDOMFactory;
 public:
-	NsXDOMComment(NsDomElement *parent, int index)
-		: NsDomText(parent, index, NS_COMMENT) {}
+	NsXDOMComment(NsDomElement *parent, NsDomElement *owner, int index)
+		: NsDomText(parent, owner, index, NS_COMMENT) {}
 	NsXDOMComment(NsDocument *doc, const xmlch_t *value)
 		: NsDomText(doc, value, NS_COMMENT) {}
 public:
@@ -768,7 +782,8 @@ public:
 		// This interface is hijacked to cast from DOMNode *
 		// to NsDomNode *.  The feature string must be
 		// "NsDom"
-		if (NsUtil::nsStringEqual(feature, _nsDomString))
+		if (NsUtil::nsStringEqual((const xmlch_t *)feature,
+					  _nsDomString))
 			return (XNSQ DOMNode *)((NsDomNode*)this);
 		return 0;
 	}
@@ -836,8 +851,8 @@ public:
 class NsXDOMProcessingInstruction: public XNSQ DOMProcessingInstruction, public NsDomText {
 	friend class NsXDOMFactory;
 public:
-	NsXDOMProcessingInstruction(NsDomElement *parent, int index)
-		: NsDomText(parent, index, NS_PINST) {}
+	NsXDOMProcessingInstruction(NsDomElement *parent, NsDomElement *owner, int index)
+		: NsDomText(parent, owner, index, NS_PINST) {}
 	NsXDOMProcessingInstruction(NsDocument *doc, const xmlch_t *target, const xmlch_t *data)
 		: NsDomText(doc, target, data) {}
 public:
@@ -891,7 +906,8 @@ public:
 		// This interface is hijacked to cast from DOMNode *
 		// to NsDomNode *.  The feature string must be
 		// "NsDom"
-		if (NsUtil::nsStringEqual(feature, _nsDomString))
+		if (NsUtil::nsStringEqual((const xmlch_t *)feature,
+					  _nsDomString))
 			return (XNSQ DOMNode *)((NsDomNode*)this);
 		return 0;
 	}
@@ -989,7 +1005,7 @@ public:
 		: NsXDOMText(document, (const xmlch_t*)0, NS_TEXT),
 		  _parent(parent)
 	{
-		_text.set(value, false);
+		_text.set16(document->getMemoryManager(), value, false);
 	}
 	virtual XNSQ DOMNode *getParentNode() const { return _parent; }
 private:
@@ -1001,14 +1017,15 @@ private:
 //
 class NsXDOMFactory: public NsDomFactory {
 public:
-	NsXDOMFactory(XNSQ MemoryManager *memManager, bool dealloc)
-		: _emptyNodeList(0), _memManager(memManager), _dealloc(dealloc) {}
+	NsXDOMFactory(XNSQ MemoryManager *memManager)
+		: _memManager(memManager), _emptyNodeList(0), documentNode_(0) {}
 public:
 	virtual NsDomElement *createNsDomElement(NsDocument *doc,
-						 nsNode_t *node,
+						 NsNode *node,
 						 NsDomElement *parent,
 						 bool standalone);
 	virtual NsDomText *createNsDomText(NsDomElement *parent,
+					   NsDomElement *owner,
 					   int index, uint32_t type);
 	virtual NsDomNamedNodeMap *createNsDomNodeMap(NsDomElement *owner);
 	virtual NsDomNodeList *createNsDomNodeList(NsDomElement *owner);
@@ -1027,11 +1044,7 @@ public:
 	virtual void destroyDomList();
 	virtual void destroy(NsDomObj *dnode);
 
-	bool getDealloc() const { return _dealloc; }
-	void setDealloc(bool dealloc) { _dealloc = dealloc; }
-	XNSQ DOMDocument *getDocumentNode() const {
-		return (NsXDOMDocument*)_domDocumentObj;
-	}
+	XNSQ DOMDocument *getDocumentNode(NsDocument *nsdoc);
 
 	// Hack: Xerces' importNode assumes that attribute
 	// value is a child text node... So create a fake one.
@@ -1041,10 +1054,21 @@ public:
 	static XNSQ DOMElement *CastToDOMElement(NsDomElement *node);
 private:
 	XNSQ MemoryManager *_memManager;
-	bool _dealloc;
 	NsDomNodeList *_emptyNodeList;
+	NsXDOMDocument *documentNode_;
 };
 
+XNSQ DOMNode *fakeDynamicCastDOMNode(NsDomNode *node);
+inline NsDomNode *fakeDynamicCastNsDomNode(XNSQ DOMNode *node)
+{
+	if (!node) return 0;
+	return (NsDomNode*)node->getInterface(_nsDomString);
+}
+inline const XNSQ DOMNode *fakeDynamicCastDOMNodeC(const NsDomNode *node)
+{ return fakeDynamicCastDOMNode(const_cast<NsDomNode*>(node)); }
+inline const NsDomNode *fakeDynamicCastNsDomNodeC(const XNSQ DOMNode *node)
+{ return fakeDynamicCastNsDomNode(const_cast<XNSQ DOMNode*>(node)); }
+
 } // namespace DbXml
 
 #endif
