@@ -1,16 +1,15 @@
 //
 // See the file LICENSE for redistribution information.
 //
-// Copyright (c) 2002-2005
-//	Sleepycat Software.  All rights reserved.
+// Copyright (c) 2002,2006 Oracle.  All rights reserved.
 //
-// $Id: Optimizer.hpp,v 1.4 2005/04/05 16:44:14 bostic Exp $
+// $Id: Optimizer.hpp,v 1.8 2006/10/30 17:45:59 bostic Exp $
 //
 
 #ifndef __OPTIMIZER_HPP
 #define	__OPTIMIZER_HPP
 
-class DataItem;
+class ASTNode;
 class DynamicContext;
 
 class XQQuery;
@@ -25,12 +24,12 @@ public:
   virtual ~Optimizer();
 
   void startOptimize(XQQuery *query);
-  DataItem *startOptimize(DataItem *item);
+  ASTNode *startOptimize(ASTNode *item);
   void reset();
 
 protected:
   virtual void optimize(XQQuery *query) = 0;
-  virtual DataItem *optimize(DataItem *item) = 0;
+  virtual ASTNode *optimize(ASTNode *item) = 0;
   virtual void resetInternal() = 0;
 
 private:
@@ -49,7 +48,23 @@ public:
 
 protected:
   virtual void optimize(XQQuery *query);
-  virtual DataItem *optimize(DataItem *item);
+  virtual ASTNode *optimize(ASTNode *item);
+  virtual void resetInternal() {}
+
+private:
+  DynamicContext *xpc_;
+};
+
+class StaticTyper : public Optimizer
+{
+public:
+  StaticTyper(DynamicContext *xpc, Optimizer *parent = 0)
+    : Optimizer(parent), xpc_(xpc) {}
+  virtual ~StaticTyper() {}
+
+protected:
+  virtual void optimize(XQQuery *query);
+  virtual ASTNode *optimize(ASTNode *item);
   virtual void resetInternal() {}
 
 private:
