@@ -4,7 +4,7 @@ ACTUAL RESULT:
 <?php
 $book_name = 'book1';
 $book_content = '<book><title>Knowledge Discovery in Databases.</title></book>';
-$mgr = new XmlManager(null);
+$mgr = new XmlManager();
 $con = $mgr->createContainer("test.dbxml");
 $con->putDocument($book_name, $book_content);
 $doc = $con->getDocument($book_name);
