@@ -10,7 +10,7 @@ $book_name = 'book1';
 $book_content = '<book><title>Knowledge Discovery in Databases.</title></book>';
 $container_name = 'test.dbxml';
 
-$mgr = new XmlManager(null);
+$mgr = new XmlManager();
 if(file_exists("test.dbxml")) {
     $mgr->removeContainer("test.dbxml");
 }
