@@ -1,11 +1,10 @@
 /*-
- *  Copyright (c) 2004
- *  Sleepycat Software.  All rights reserved.
+ * Copyright (c) 2004,2006 Oracle.  All rights reserved.
  *
- *  http://www.apache.org/licenses/LICENSE-2.0.txt
+ * http://www.apache.org/licenses/LICENSE-2.0.txt
  * 
- *  authors: Wez Furlong <wez@omniti.com>  
- *           George Schlossnagle <george@omniti.com>
+ * authors: Wez Furlong <wez@omniti.com>  
+ *          George Schlossnagle <george@omniti.com>
  */
 
 #include "php_dbxml_int.hpp"
@@ -23,7 +22,6 @@ PHP_DBXML_METHOD_BEGIN(XmlContainer, XmlContainer)
 }
 PHP_DBXML_METHOD_END()
 
-#if DBBXML_VERSION_MAJOR == 2 && DBXML_VERSION_MINOR == 0 && DBXML_VERSION_PATCH > 2
 PHP_DBXML_METHOD_BEGIN(XmlContainer, sync)
 {
   PHP_DBXML_STUFFED_THIS(XmlContainer);
@@ -33,15 +31,12 @@ PHP_DBXML_METHOD_BEGIN(XmlContainer, sync)
   This.sync();
 }
 PHP_DBXML_METHOD_END()
-#endif
 
 PHP_DBXML_METHOD_BEGIN(XmlContainer, getName)
 {
   PHP_DBXML_STUFFED_THIS(XmlContainer);
-  if (FAILURE == zend_parse_parameters(ZEND_NUM_ARGS() TSRMLS_CC, "")) {
-    return;
-  }
-  /* FIXME */
+  std::string buffer = This.getName();
+  RETURN_STRINGL((char *)buffer.data(), buffer.length(), 1);
 }
 PHP_DBXML_METHOD_END()
 
@@ -49,6 +44,15 @@ PHP_DBXML_METHOD_END()
  */
 PHP_DBXML_METHOD_BEGIN(XmlContainer, getContainerType)
 {
+  PHP_DBXML_STUFFED_THIS(XmlContainer);
+  RETURN_LONG(This.getContainerType());
+}
+PHP_DBXML_METHOD_END()
+
+PHP_DBXML_METHOD_BEGIN(XmlContainer, getIndexNodes)
+{
+  PHP_DBXML_STUFFED_THIS(XmlContainer);
+  RETURN_BOOL(This.getIndexNodes());
 }
 PHP_DBXML_METHOD_END()
 
@@ -77,10 +81,10 @@ PHP_DBXML_METHOD_BEGIN(XmlContainer, setIndexSpecification)
   else {
     return;
   }
-  XmlIndexSpecification idx = php_dbxml_get_XmlIndexSpecification_object_pointer(zidx);
-  XmlUpdateContext ctx = zctx?php_dbxml_get_XmlUpdateContext_object_pointer(zctx):This.getManager().createUpdateContext();
+  XmlIndexSpecification idx = php_dbxml_get_XmlIndexSpecification_object_pointer(zidx TSRMLS_CC);
+  XmlUpdateContext ctx = zctx?php_dbxml_get_XmlUpdateContext_object_pointer(zctx TSRMLS_CC):This.getManager().createUpdateContext();
   if(ztxn) {
-    XmlTransaction txn = php_dbxml_get_XmlTransaction_object_pointer(ztxn);
+    XmlTransaction txn = php_dbxml_get_XmlTransaction_object_pointer(ztxn TSRMLS_CC);
     This.setIndexSpecification(txn, idx, ctx);
   } else {
     This.setIndexSpecification(idx, ctx);
@@ -106,9 +110,9 @@ PHP_DBXML_METHOD_BEGIN(XmlContainer, addIndex)
   else {
     return;
   }
-  XmlUpdateContext ctx = zctx?php_dbxml_get_XmlUpdateContext_object_pointer(zctx):This.getManager().createUpdateContext();
+  XmlUpdateContext ctx = zctx?php_dbxml_get_XmlUpdateContext_object_pointer(zctx TSRMLS_CC):This.getManager().createUpdateContext();
   if(ztxn) {
-    XmlTransaction txn = php_dbxml_get_XmlTransaction_object_pointer(ztxn);
+    XmlTransaction txn = php_dbxml_get_XmlTransaction_object_pointer(ztxn TSRMLS_CC);
     This.addIndex(txn, std::string(uri, urilen), std::string(name, namelen), std::string(index, indexlen), ctx);
   } else {
     This.addIndex(std::string(uri, urilen), std::string(name, namelen), std::string(index, indexlen), ctx);
@@ -134,9 +138,9 @@ PHP_DBXML_METHOD_BEGIN(XmlContainer, deleteIndex)
   else {
     return;
   }
-  XmlUpdateContext ctx = zctx?php_dbxml_get_XmlUpdateContext_object_pointer(zctx):This.getManager().createUpdateContext();
+  XmlUpdateContext ctx = zctx?php_dbxml_get_XmlUpdateContext_object_pointer(zctx TSRMLS_CC):This.getManager().createUpdateContext();
   if(ztxn) {
-    XmlTransaction txn = php_dbxml_get_XmlTransaction_object_pointer(ztxn);
+    XmlTransaction txn = php_dbxml_get_XmlTransaction_object_pointer(ztxn TSRMLS_CC);
     This.deleteIndex(txn, std::string(uri, urilen), std::string(name, namelen), std::string(index, indexlen), ctx);
   } else {
     This.deleteIndex(std::string(uri, urilen), std::string(name, namelen), std::string(index, indexlen), ctx);
@@ -162,9 +166,9 @@ PHP_DBXML_METHOD_BEGIN(XmlContainer, replaceIndex)
   else {
     return;
   }
-  XmlUpdateContext ctx = zctx?php_dbxml_get_XmlUpdateContext_object_pointer(zctx):This.getManager().createUpdateContext();
+  XmlUpdateContext ctx = zctx?php_dbxml_get_XmlUpdateContext_object_pointer(zctx TSRMLS_CC):This.getManager().createUpdateContext();
   if(ztxn) {
-    XmlTransaction txn = php_dbxml_get_XmlTransaction_object_pointer(ztxn);
+    XmlTransaction txn = php_dbxml_get_XmlTransaction_object_pointer(ztxn TSRMLS_CC);
     This.replaceIndex(txn, std::string(uri, urilen), std::string(name, namelen), std::string(index, indexlen), ctx);
   } else {
     This.replaceIndex(std::string(uri, urilen), std::string(name, namelen), std::string(index, indexlen), ctx);
@@ -190,9 +194,9 @@ PHP_DBXML_METHOD_BEGIN(XmlContainer, addDefaultIndex)
   else {
     return;
   }
-  XmlUpdateContext ctx = zctx?php_dbxml_get_XmlUpdateContext_object_pointer(zctx):This.getManager().createUpdateContext();
+  XmlUpdateContext ctx = zctx?php_dbxml_get_XmlUpdateContext_object_pointer(zctx TSRMLS_CC):This.getManager().createUpdateContext();
   if(ztxn) {
-    XmlTransaction txn = php_dbxml_get_XmlTransaction_object_pointer(ztxn);
+    XmlTransaction txn = php_dbxml_get_XmlTransaction_object_pointer(ztxn TSRMLS_CC);
     This.addDefaultIndex(txn, std::string(index, indexlen), ctx);
   } else {
     This.addDefaultIndex(std::string(index, indexlen), ctx);
@@ -218,9 +222,9 @@ PHP_DBXML_METHOD_BEGIN(XmlContainer, deleteDefaultIndex)
   else {
     return;
   }
-  XmlUpdateContext ctx = zctx?php_dbxml_get_XmlUpdateContext_object_pointer(zctx):This.getManager().createUpdateContext();
+  XmlUpdateContext ctx = zctx?php_dbxml_get_XmlUpdateContext_object_pointer(zctx TSRMLS_CC):This.getManager().createUpdateContext();
   if(ztxn) {
-    XmlTransaction txn = php_dbxml_get_XmlTransaction_object_pointer(ztxn);
+    XmlTransaction txn = php_dbxml_get_XmlTransaction_object_pointer(ztxn TSRMLS_CC);
     This.deleteDefaultIndex(txn, std::string(index, indexlen), ctx);
   } else {
     This.deleteDefaultIndex(std::string(index, indexlen), ctx);
@@ -246,9 +250,9 @@ PHP_DBXML_METHOD_BEGIN(XmlContainer, replaceDefaultIndex)
   else {
     return;
   }
-  XmlUpdateContext ctx = zctx?php_dbxml_get_XmlUpdateContext_object_pointer(zctx):This.getManager().createUpdateContext();
+  XmlUpdateContext ctx = zctx?php_dbxml_get_XmlUpdateContext_object_pointer(zctx TSRMLS_CC):This.getManager().createUpdateContext();
   if(ztxn) {
-    XmlTransaction txn = php_dbxml_get_XmlTransaction_object_pointer(ztxn);
+    XmlTransaction txn = php_dbxml_get_XmlTransaction_object_pointer(ztxn TSRMLS_CC);
     This.replaceDefaultIndex(txn, std::string(index, indexlen), ctx);
   } else {
     This.replaceDefaultIndex(std::string(index, indexlen), ctx);
@@ -270,25 +274,52 @@ PHP_DBXML_METHOD_BEGIN(XmlContainer, getDocument)
   long flags = 0;
   if(SUCCESS == zend_parse_parameters_ex(ZEND_PARSE_PARAMS_QUIET, ZEND_NUM_ARGS() TSRMLS_CC, "s|l", 
       &name, &namelen, &flags)) {
-  }
-  else if(SUCCESS == zend_parse_parameters(ZEND_NUM_ARGS() TSRMLS_CC, "Os|l", 
+  } else if(SUCCESS == zend_parse_parameters(ZEND_NUM_ARGS() TSRMLS_CC, "Os|l", 
       &ztxn, php_dbxml_XmlTransaction_ce,
       &name, &namelen, &flags)) {
-  }
-  else {
+  } else {
     return;
   }
   if(ztxn) {
-    XmlTransaction txn = php_dbxml_get_XmlTransaction_object_pointer(ztxn);
+    XmlTransaction txn = php_dbxml_get_XmlTransaction_object_pointer(ztxn TSRMLS_CC);
     XmlDocument doc = This.getDocument(txn, std::string(name, namelen), flags);
-    php_dbxml_set_XmlDocument_object_factory(return_value, doc);
+    php_dbxml_set_XmlDocument_object_factory(return_value, doc TSRMLS_CC);
   } else {
     XmlDocument doc = This.getDocument(std::string(name, namelen), flags);
-    php_dbxml_set_XmlDocument_object_factory(return_value, doc);
+    php_dbxml_set_XmlDocument_object_factory(return_value, doc TSRMLS_CC);
   }
 }
 PHP_DBXML_METHOD_END()
 
+/* {{{ Node Retrieval
+ */
+
+PHP_DBXML_METHOD_BEGIN(XmlContainer, getNode)
+{
+	PHP_DBXML_STUFFED_THIS(XmlContainer);
+	zval *ztxn = NULL;
+	char *handle;
+	long handleLen;
+	long flags = 0;
+	if(SUCCESS == zend_parse_parameters_ex(ZEND_PARSE_PARAMS_QUIET, ZEND_NUM_ARGS() TSRMLS_CC, "s|l", 
+					       &handle, &handleLen, &flags)) {
+	} else if(SUCCESS == zend_parse_parameters(ZEND_NUM_ARGS() TSRMLS_CC, "Os|l", 
+						 &ztxn, php_dbxml_XmlTransaction_ce,
+						 &handle, &handleLen, &flags)) {
+	} else {
+		return;
+	}
+	if(ztxn) {
+		XmlTransaction txn = php_dbxml_get_XmlTransaction_object_pointer(ztxn TSRMLS_CC);
+		XmlValue doc = This.getNode(txn, std::string(handle, handleLen), flags);
+		php_dbxml_set_XmlValue_object_factory(return_value, doc TSRMLS_CC);
+	} else {
+		XmlValue doc = This.getNode(std::string(handle, handleLen), flags);
+		php_dbxml_set_XmlValue_object_factory(return_value, doc TSRMLS_CC);
+	}
+}
+PHP_DBXML_METHOD_END()
+
 PHP_DBXML_METHOD_BEGIN(XmlContainer, putDocument)
 {
   PHP_DBXML_STUFFED_THIS(XmlContainer);
@@ -299,17 +330,17 @@ PHP_DBXML_METHOD_BEGIN(XmlContainer, putDocument)
   if(SUCCESS == zend_parse_parameters_ex(ZEND_PARSE_PARAMS_QUIET, ZEND_NUM_ARGS() TSRMLS_CC, "O|Ol",
     &zdoc, php_dbxml_XmlDocument_ce,
     &zctx, php_dbxml_XmlUpdateContext_ce, &flags)) {
-    XmlDocument doc = php_dbxml_get_XmlDocument_object_pointer(zdoc);
-    XmlUpdateContext ctx = zctx?php_dbxml_get_XmlUpdateContext_object_pointer(zctx):This.getManager().createUpdateContext();
+    XmlDocument doc = php_dbxml_get_XmlDocument_object_pointer(zdoc TSRMLS_CC);
+    XmlUpdateContext ctx = zctx?php_dbxml_get_XmlUpdateContext_object_pointer(zctx TSRMLS_CC):This.getManager().createUpdateContext();
     This.putDocument(doc, ctx, flags);
   }
   else if(SUCCESS == zend_parse_parameters_ex(ZEND_PARSE_PARAMS_QUIET, ZEND_NUM_ARGS() TSRMLS_CC, "OO|Ol",
     &ztxn, php_dbxml_XmlTransaction_ce,
     &zdoc, php_dbxml_XmlDocument_ce,
     &zctx, php_dbxml_XmlUpdateContext_ce, &flags)) {
-    XmlTransaction txn = php_dbxml_get_XmlTransaction_object_pointer(ztxn);
-    XmlDocument doc = php_dbxml_get_XmlDocument_object_pointer(zdoc);
-    XmlUpdateContext ctx = zctx?php_dbxml_get_XmlUpdateContext_object_pointer(zctx):This.getManager().createUpdateContext();
+    XmlTransaction txn = php_dbxml_get_XmlTransaction_object_pointer(ztxn TSRMLS_CC);
+    XmlDocument doc = php_dbxml_get_XmlDocument_object_pointer(zdoc TSRMLS_CC);
+    XmlUpdateContext ctx = zctx?php_dbxml_get_XmlUpdateContext_object_pointer(zctx TSRMLS_CC):This.getManager().createUpdateContext();
     This.putDocument(txn, doc, ctx, flags);
   }
   else if(SUCCESS == zend_parse_parameters_ex(ZEND_PARSE_PARAMS_QUIET, ZEND_NUM_ARGS() TSRMLS_CC, "sO|Ol",
@@ -317,7 +348,7 @@ PHP_DBXML_METHOD_BEGIN(XmlContainer, putDocument)
     &zstream, php_dbxml_XmlInputStream_ce,
     &zctx, php_dbxml_XmlUpdateContext_ce, &flags)) {
     XmlInputStream *stm = PHP_DBXML_GET(zstream, XmlInputStream);
-    XmlUpdateContext ctx = zctx?php_dbxml_get_XmlUpdateContext_object_pointer(zctx):This.getManager().createUpdateContext();
+    XmlUpdateContext ctx = zctx?php_dbxml_get_XmlUpdateContext_object_pointer(zctx TSRMLS_CC):This.getManager().createUpdateContext();
     This.putDocument(std::string(name, namelen), stm, ctx, flags);
   }
   else if(SUCCESS == zend_parse_parameters_ex(ZEND_PARSE_PARAMS_QUIET, ZEND_NUM_ARGS() TSRMLS_CC, "OsO|Ol",
@@ -325,16 +356,16 @@ PHP_DBXML_METHOD_BEGIN(XmlContainer, putDocument)
     &name, &namelen,
     &zstream, php_dbxml_XmlInputStream_ce,
     &zctx, php_dbxml_XmlUpdateContext_ce, &flags)) {
-    XmlTransaction txn = php_dbxml_get_XmlTransaction_object_pointer(ztxn);
+    XmlTransaction txn = php_dbxml_get_XmlTransaction_object_pointer(ztxn TSRMLS_CC);
     XmlInputStream *stm = PHP_DBXML_GET(zstream, XmlInputStream);
-    XmlUpdateContext ctx = zctx?php_dbxml_get_XmlUpdateContext_object_pointer(zctx):This.getManager().createUpdateContext();
+    XmlUpdateContext ctx = zctx?php_dbxml_get_XmlUpdateContext_object_pointer(zctx TSRMLS_CC):This.getManager().createUpdateContext();
     This.putDocument(txn, std::string(name, namelen), stm, ctx, flags);
   }
   else if(SUCCESS == zend_parse_parameters_ex(ZEND_PARSE_PARAMS_QUIET, ZEND_NUM_ARGS() TSRMLS_CC, "ss|Ol",
     &name, &namelen,
     &contents, &contentslen,
     &zctx, php_dbxml_XmlUpdateContext_ce, &flags)) {
-    XmlUpdateContext ctx = zctx?php_dbxml_get_XmlUpdateContext_object_pointer(zctx):This.getManager().createUpdateContext();
+    XmlUpdateContext ctx = zctx?php_dbxml_get_XmlUpdateContext_object_pointer(zctx TSRMLS_CC):This.getManager().createUpdateContext();
     This.putDocument(std::string(name, namelen), std::string(contents, contentslen), ctx, flags);
   }
   else if(SUCCESS == zend_parse_parameters(ZEND_NUM_ARGS() TSRMLS_CC, "Oss|Ol",
@@ -342,8 +373,8 @@ PHP_DBXML_METHOD_BEGIN(XmlContainer, putDocument)
     &name, &namelen,
     &contents, &contentslen,
     &zctx, php_dbxml_XmlUpdateContext_ce, &flags)) {
-    XmlTransaction txn = php_dbxml_get_XmlTransaction_object_pointer(ztxn);
-    XmlUpdateContext ctx = zctx?php_dbxml_get_XmlUpdateContext_object_pointer(zctx):This.getManager().createUpdateContext();
+    XmlTransaction txn = php_dbxml_get_XmlTransaction_object_pointer(ztxn TSRMLS_CC);
+    XmlUpdateContext ctx = zctx?php_dbxml_get_XmlUpdateContext_object_pointer(zctx TSRMLS_CC):This.getManager().createUpdateContext();
     This.putDocument(txn, std::string(name, namelen), std::string(contents, contentslen), ctx, flags);
   }
   else {
@@ -352,6 +383,39 @@ PHP_DBXML_METHOD_BEGIN(XmlContainer, putDocument)
 }
 PHP_DBXML_METHOD_END()
 
+// This shares much of its code with putDocument, but
+// returns XmlEventWriter
+PHP_DBXML_METHOD_BEGIN(XmlContainer, putDocumentAsEventWriter)
+{
+  PHP_DBXML_STUFFED_THIS(XmlContainer);
+  zval *ztxn = NULL, *zdoc = NULL, *zstream = NULL, *zctx = NULL;
+  char *name, *contents;
+  long namelen, contentslen, flags = 0;
+  /* XmlDocument */
+  if(SUCCESS == zend_parse_parameters_ex(ZEND_PARSE_PARAMS_QUIET, ZEND_NUM_ARGS() TSRMLS_CC, "O|Ol",
+    &zdoc, php_dbxml_XmlDocument_ce,
+    &zctx, php_dbxml_XmlUpdateContext_ce, &flags)) {
+    XmlDocument doc = php_dbxml_get_XmlDocument_object_pointer(zdoc TSRMLS_CC);
+    XmlUpdateContext ctx = zctx?php_dbxml_get_XmlUpdateContext_object_pointer(zctx TSRMLS_CC):This.getManager().createUpdateContext();
+    XmlEventWriter &writer = This.putDocumentAsEventWriter(doc, ctx, flags);
+    php_dbxml_set_XmlEventWriter_object_factory(return_value, writer TSRMLS_CC);
+  }
+  else if(SUCCESS == zend_parse_parameters_ex(ZEND_PARSE_PARAMS_QUIET, ZEND_NUM_ARGS() TSRMLS_CC, "OO|Ol",
+    &ztxn, php_dbxml_XmlTransaction_ce,
+    &zdoc, php_dbxml_XmlDocument_ce,
+    &zctx, php_dbxml_XmlUpdateContext_ce, &flags)) {
+    XmlTransaction txn = php_dbxml_get_XmlTransaction_object_pointer(ztxn TSRMLS_CC);
+    XmlDocument doc = php_dbxml_get_XmlDocument_object_pointer(zdoc TSRMLS_CC);
+    XmlUpdateContext ctx = zctx?php_dbxml_get_XmlUpdateContext_object_pointer(zctx TSRMLS_CC):This.getManager().createUpdateContext();
+    XmlEventWriter &writer = This.putDocumentAsEventWriter(txn, doc, ctx, flags);
+    php_dbxml_set_XmlEventWriter_object_factory(return_value, writer TSRMLS_CC);
+  }
+  else {
+	  php_error(E_WARNING, "usage: putDocumentAsEventWriter [txn] doc [context] [flags]");
+  } 
+}
+PHP_DBXML_METHOD_END()
+
 PHP_DBXML_METHOD_BEGIN(XmlContainer, deleteDocument)
 {
   PHP_DBXML_STUFFED_THIS(XmlContainer);
@@ -361,31 +425,31 @@ PHP_DBXML_METHOD_BEGIN(XmlContainer, deleteDocument)
   if(SUCCESS == zend_parse_parameters_ex(ZEND_PARSE_PARAMS_QUIET, ZEND_NUM_ARGS() TSRMLS_CC, "O|O",
     &zdoc, php_dbxml_XmlDocument_ce,
     &zctx, php_dbxml_XmlUpdateContext_ce)) {
-    XmlDocument doc = php_dbxml_get_XmlDocument_object_pointer(zdoc);
-    XmlUpdateContext ctx = zctx?php_dbxml_get_XmlUpdateContext_object_pointer(zctx):This.getManager().createUpdateContext();
+    XmlDocument doc = php_dbxml_get_XmlDocument_object_pointer(zdoc TSRMLS_CC);
+    XmlUpdateContext ctx = zctx?php_dbxml_get_XmlUpdateContext_object_pointer(zctx TSRMLS_CC):This.getManager().createUpdateContext();
     This.deleteDocument(doc, ctx);
   }
   else if(SUCCESS == zend_parse_parameters_ex(ZEND_PARSE_PARAMS_QUIET, ZEND_NUM_ARGS() TSRMLS_CC, "OO|O",
     &ztxn, php_dbxml_XmlTransaction_ce,
     &zdoc, php_dbxml_XmlDocument_ce,
     &zctx, php_dbxml_XmlUpdateContext_ce)) {
-    XmlTransaction txn = php_dbxml_get_XmlTransaction_object_pointer(ztxn);
-    XmlDocument doc = php_dbxml_get_XmlDocument_object_pointer(zdoc);
-    XmlUpdateContext ctx = zctx?php_dbxml_get_XmlUpdateContext_object_pointer(zctx):This.getManager().createUpdateContext();
+    XmlTransaction txn = php_dbxml_get_XmlTransaction_object_pointer(ztxn TSRMLS_CC);
+    XmlDocument doc = php_dbxml_get_XmlDocument_object_pointer(zdoc TSRMLS_CC);
+    XmlUpdateContext ctx = zctx?php_dbxml_get_XmlUpdateContext_object_pointer(zctx TSRMLS_CC):This.getManager().createUpdateContext();
     This.deleteDocument(txn, doc, ctx);
   }
   else if(SUCCESS == zend_parse_parameters_ex(ZEND_PARSE_PARAMS_QUIET, ZEND_NUM_ARGS() TSRMLS_CC, "s|O",
     &name, &namelen,
     &zctx, php_dbxml_XmlUpdateContext_ce)) {
-    XmlUpdateContext ctx = zctx?php_dbxml_get_XmlUpdateContext_object_pointer(zctx):This.getManager().createUpdateContext();
+    XmlUpdateContext ctx = zctx?php_dbxml_get_XmlUpdateContext_object_pointer(zctx TSRMLS_CC):This.getManager().createUpdateContext();
     This.deleteDocument(std::string(name, namelen), ctx);
   }
   else if(SUCCESS == zend_parse_parameters(ZEND_NUM_ARGS() TSRMLS_CC, "Os|O",
     &ztxn, php_dbxml_XmlTransaction_ce,
     &name, &namelen,
     &zctx, php_dbxml_XmlUpdateContext_ce)) {
-    XmlTransaction txn = php_dbxml_get_XmlTransaction_object_pointer(ztxn);
-    XmlUpdateContext ctx = zctx?php_dbxml_get_XmlUpdateContext_object_pointer(zctx):This.getManager().createUpdateContext();
+    XmlTransaction txn = php_dbxml_get_XmlTransaction_object_pointer(ztxn TSRMLS_CC);
+    XmlUpdateContext ctx = zctx?php_dbxml_get_XmlUpdateContext_object_pointer(zctx TSRMLS_CC):This.getManager().createUpdateContext();
     This.deleteDocument(txn, std::string(name, namelen), ctx);
   }
   else {
@@ -403,17 +467,17 @@ PHP_DBXML_METHOD_BEGIN(XmlContainer, updateDocument)
   if(SUCCESS == zend_parse_parameters_ex(ZEND_PARSE_PARAMS_QUIET, ZEND_NUM_ARGS() TSRMLS_CC, "O|O",
     &zdoc, php_dbxml_XmlDocument_ce,
     &zctx, php_dbxml_XmlUpdateContext_ce)) {
-    XmlDocument doc = php_dbxml_get_XmlDocument_object_pointer(zdoc);
-    XmlUpdateContext ctx = zctx?php_dbxml_get_XmlUpdateContext_object_pointer(zctx):This.getManager().createUpdateContext();
+    XmlDocument doc = php_dbxml_get_XmlDocument_object_pointer(zdoc TSRMLS_CC);
+    XmlUpdateContext ctx = zctx?php_dbxml_get_XmlUpdateContext_object_pointer(zctx TSRMLS_CC):This.getManager().createUpdateContext();
     This.updateDocument(doc, ctx);
   }
   else if(SUCCESS == zend_parse_parameters(ZEND_NUM_ARGS() TSRMLS_CC, "OO|O",
     &ztxn, php_dbxml_XmlTransaction_ce,
     &zdoc, php_dbxml_XmlDocument_ce,
     &zctx, php_dbxml_XmlUpdateContext_ce)) {
-    XmlTransaction txn = php_dbxml_get_XmlTransaction_object_pointer(ztxn);
-    XmlDocument doc = php_dbxml_get_XmlDocument_object_pointer(zdoc);
-    XmlUpdateContext ctx = zctx?php_dbxml_get_XmlUpdateContext_object_pointer(zctx):This.getManager().createUpdateContext();
+    XmlTransaction txn = php_dbxml_get_XmlTransaction_object_pointer(ztxn TSRMLS_CC);
+    XmlDocument doc = php_dbxml_get_XmlDocument_object_pointer(zdoc TSRMLS_CC);
+    XmlUpdateContext ctx = zctx?php_dbxml_get_XmlUpdateContext_object_pointer(zctx TSRMLS_CC):This.getManager().createUpdateContext();
     This.updateDocument(txn, doc, ctx);
   }
   else { return; }
@@ -434,9 +498,9 @@ PHP_DBXML_METHOD_BEGIN(XmlContainer, lookupIndex)
       &uri, &urilen, &name, &namelen, &index, &indexlen, 
       &zvl, php_dbxml_XmlValue_ce, &flags))
   {
-    XmlQueryContext ctx = php_dbxml_get_XmlQueryContext_object_pointer(zctx);
+    XmlQueryContext ctx = php_dbxml_get_XmlQueryContext_object_pointer(zctx TSRMLS_CC);
     if(zvl) {
-      XmlValue vl = php_dbxml_get_XmlValue_object_pointer(zvl);
+      XmlValue vl = php_dbxml_get_XmlValue_object_pointer(zvl TSRMLS_CC);
       XmlResults re = This.lookupIndex(ctx, std::string(uri, urilen), std::string(name, namelen), std::string(index, indexlen), vl, flags);
       php_dbxml_set_XmlResults_object_factory(return_value, re TSRMLS_CC);
     } else {
@@ -450,10 +514,10 @@ PHP_DBXML_METHOD_BEGIN(XmlContainer, lookupIndex)
       &uri, &urilen, &name, &namelen, &index, &indexlen, 
       &zvl, php_dbxml_XmlValue_ce, &flags))
   {
-    XmlTransaction txn = php_dbxml_get_XmlTransaction_object_pointer(ztxn);
-    XmlQueryContext ctx = php_dbxml_get_XmlQueryContext_object_pointer(zctx);
+    XmlTransaction txn = php_dbxml_get_XmlTransaction_object_pointer(ztxn TSRMLS_CC);
+    XmlQueryContext ctx = php_dbxml_get_XmlQueryContext_object_pointer(zctx TSRMLS_CC);
     if(zvl) {
-      XmlValue vl = php_dbxml_get_XmlValue_object_pointer(zvl);
+      XmlValue vl = php_dbxml_get_XmlValue_object_pointer(zvl TSRMLS_CC);
       XmlResults re = This.lookupIndex(txn, ctx, std::string(uri, urilen), std::string(name, namelen), std::string(index, indexlen), vl, flags);
       php_dbxml_set_XmlResults_object_factory(return_value, re TSRMLS_CC);
     } else {
@@ -467,9 +531,9 @@ PHP_DBXML_METHOD_BEGIN(XmlContainer, lookupIndex)
       &puri, &purilen, &pname, &pnamelen, &index, &indexlen, 
       &zvl, php_dbxml_XmlValue_ce, &flags))
   {
-    XmlQueryContext ctx = php_dbxml_get_XmlQueryContext_object_pointer(zctx);
+    XmlQueryContext ctx = php_dbxml_get_XmlQueryContext_object_pointer(zctx TSRMLS_CC);
     if(zvl) {
-      XmlValue vl = php_dbxml_get_XmlValue_object_pointer(zvl);
+      XmlValue vl = php_dbxml_get_XmlValue_object_pointer(zvl TSRMLS_CC);
       XmlResults re = This.lookupIndex(ctx, 
           std::string(uri, urilen), std::string(name, namelen), 
           std::string(puri, purilen), std::string(pname, pnamelen), std::string(index, indexlen),
@@ -490,10 +554,10 @@ PHP_DBXML_METHOD_BEGIN(XmlContainer, lookupIndex)
       &puri, &purilen, &pname, &pnamelen, &index, &indexlen, 
       &zvl, php_dbxml_XmlValue_ce, &flags))
   {
-    XmlTransaction txn = php_dbxml_get_XmlTransaction_object_pointer(ztxn);
-    XmlQueryContext ctx = php_dbxml_get_XmlQueryContext_object_pointer(zctx);
+    XmlTransaction txn = php_dbxml_get_XmlTransaction_object_pointer(ztxn TSRMLS_CC);
+    XmlQueryContext ctx = php_dbxml_get_XmlQueryContext_object_pointer(zctx TSRMLS_CC);
     if(zvl) {
-      XmlValue vl = php_dbxml_get_XmlValue_object_pointer(zvl);
+      XmlValue vl = php_dbxml_get_XmlValue_object_pointer(zvl TSRMLS_CC);
       XmlResults re = This.lookupIndex(txn, ctx, 
           std::string(uri, urilen), std::string(name, namelen), 
           std::string(puri, purilen), std::string(pname, pnamelen), std::string(index, indexlen),
@@ -526,7 +590,7 @@ PHP_DBXML_METHOD_BEGIN(XmlContainer, lookupStatistics)
       &zvl, php_dbxml_XmlValue_ce))
   {
     if(zvl) {
-      XmlValue vl = php_dbxml_get_XmlValue_object_pointer(zvl);
+      XmlValue vl = php_dbxml_get_XmlValue_object_pointer(zvl TSRMLS_CC);
       XmlStatistics re = This.lookupStatistics(std::string(uri, urilen), std::string(name, namelen), std::string(index, indexlen), vl);
       php_dbxml_set_XmlStatistics_object_factory(return_value, re TSRMLS_CC);
     } else {
@@ -539,9 +603,9 @@ PHP_DBXML_METHOD_BEGIN(XmlContainer, lookupStatistics)
       &uri, &urilen, &name, &namelen, &index, &indexlen, 
       &zvl, php_dbxml_XmlValue_ce))
   {
-    XmlTransaction txn = php_dbxml_get_XmlTransaction_object_pointer(ztxn);
+    XmlTransaction txn = php_dbxml_get_XmlTransaction_object_pointer(ztxn TSRMLS_CC);
     if(zvl) {
-      XmlValue vl = php_dbxml_get_XmlValue_object_pointer(zvl);
+      XmlValue vl = php_dbxml_get_XmlValue_object_pointer(zvl TSRMLS_CC);
       XmlStatistics re = This.lookupStatistics(txn, std::string(uri, urilen), std::string(name, namelen), std::string(index, indexlen), vl);
       php_dbxml_set_XmlStatistics_object_factory(return_value, re TSRMLS_CC);
     } else {
@@ -555,7 +619,7 @@ PHP_DBXML_METHOD_BEGIN(XmlContainer, lookupStatistics)
       &zvl, php_dbxml_XmlValue_ce))
   {
     if(zvl) {
-      XmlValue vl = php_dbxml_get_XmlValue_object_pointer(zvl);
+      XmlValue vl = php_dbxml_get_XmlValue_object_pointer(zvl TSRMLS_CC);
       XmlStatistics re = This.lookupStatistics(std::string(uri, urilen), std::string(name, namelen), 
           std::string(puri, purilen), std::string(pname, pnamelen), std::string(index, indexlen),
           vl);
@@ -573,9 +637,9 @@ PHP_DBXML_METHOD_BEGIN(XmlContainer, lookupStatistics)
       &puri, &purilen, &pname, &pnamelen, &index, &indexlen, 
       &zvl, php_dbxml_XmlValue_ce))
   {
-    XmlTransaction txn = php_dbxml_get_XmlTransaction_object_pointer(ztxn);
+    XmlTransaction txn = php_dbxml_get_XmlTransaction_object_pointer(ztxn TSRMLS_CC);
     if(zvl) {
-      XmlValue vl = php_dbxml_get_XmlValue_object_pointer(zvl);
+      XmlValue vl = php_dbxml_get_XmlValue_object_pointer(zvl TSRMLS_CC);
       XmlStatistics re = This.lookupStatistics(txn,
           std::string(uri, urilen), std::string(name, namelen), 
           std::string(puri, purilen), std::string(pname, pnamelen), std::string(index, indexlen),
@@ -627,7 +691,7 @@ PHP_DBXML_METHOD_BEGIN(XmlContainer, getAllDocuments)
     return;
   } 
   if(ztxn) {
-    XmlTransaction txn = php_dbxml_get_XmlTransaction_object_pointer(ztxn);
+    XmlTransaction txn = php_dbxml_get_XmlTransaction_object_pointer(ztxn TSRMLS_CC);
     XmlResults re = This.getAllDocuments(txn, flags);
     php_dbxml_set_XmlResults_object_factory(return_value, re TSRMLS_CC);
   } else {
@@ -647,7 +711,7 @@ PHP_DBXML_METHOD_BEGIN(XmlContainer, getNumDocuments)
     return;
   } 
   if(ztxn) {
-    XmlTransaction txn = php_dbxml_get_XmlTransaction_object_pointer(ztxn);
+    XmlTransaction txn = php_dbxml_get_XmlTransaction_object_pointer(ztxn TSRMLS_CC);
     RETURN_LONG(This.getNumDocuments(txn));
   } else {
     RETURN_LONG(This.getNumDocuments());
@@ -665,11 +729,10 @@ PHP_DBXML_METHOD_END()
 
 function_entry php_dbxml_XmlContainer_methods[] = {
   PHP_ME(XmlContainer, XmlContainer, NULL, 0)
-#if DBBXML_VERSION_MAJOR == 2 && DBXML_VERSION_MINOR == 0 && DBXML_VERSION_PATCH > 2
   PHP_ME(XmlContainer, sync, NULL, 0)
-#endif
   PHP_ME(XmlContainer, getName, NULL, 0)
   PHP_ME(XmlContainer, getContainerType, NULL, 0)
+  PHP_ME(XmlContainer, getIndexNodes, NULL, 0)
   PHP_ME(XmlContainer, getIndexSpecification, NULL, 0)
   PHP_ME(XmlContainer, setIndexSpecification, NULL, 0)
   PHP_ME(XmlContainer, addIndex, NULL, 0)
@@ -679,7 +742,9 @@ function_entry php_dbxml_XmlContainer_methods[] = {
   PHP_ME(XmlContainer, deleteDefaultIndex, NULL, 0)
   PHP_ME(XmlContainer, replaceDefaultIndex, NULL, 0)
   PHP_ME(XmlContainer, getDocument, NULL, 0)
+  PHP_ME(XmlContainer, getNode, NULL, 0)
   PHP_ME(XmlContainer, putDocument, NULL, 0)
+  PHP_ME(XmlContainer, putDocumentAsEventWriter, NULL, 0)
   PHP_ME(XmlContainer, deleteDocument, NULL, 0)
   PHP_ME(XmlContainer, updateDocument, NULL, 0)
   PHP_ME(XmlContainer, lookupIndex, NULL, 0)
