@@ -1,11 +1,10 @@
 /*-
- *  Copyright (c) 2004
- *  Sleepycat Software.  All rights reserved.
+ * Copyright (c) 2004,2006 Oracle.  All rights reserved.
  *
- *  http://www.apache.org/licenses/LICENSE-2.0.txt
+ * http://www.apache.org/licenses/LICENSE-2.0.txt
  * 
- *  authors: Wez Furlong <wez@omniti.com>  
- *           George Schlossnagle <george@omniti.com>
+ * authors: Wez Furlong <wez@omniti.com>  
+ *          George Schlossnagle <george@omniti.com>
  */
 
 #include "php_dbxml_int.hpp"
@@ -16,7 +15,7 @@ ZEND_RSRC_DTOR_FUNC(php_dbxml_XmlStatistics_dtor)
 }
 
 /* {{{ proto XmlStatistics::XmlStatistics(object dbenv [, string name])
-   Create a new container using an optional dbenv (can be null) and name */
+   Create a new XmlStatistics object -- INVALID */
 PHP_METHOD(XmlStatistics, XmlStatistics)
 {
   zend_error(E_ERROR, "XmlStatistics can not be instantiated from PHP");
