@@ -1,10 +1,9 @@
 //
 // See the file LICENSE for redistribution information.
 //
-// Copyright (c) 2002-2005
-//	Sleepycat Software.  All rights reserved.
+// Copyright (c) 2002,2006 Oracle.  All rights reserved.
 //
-// $Id: QueryRunner.cpp,v 1.16 2005/04/20 18:31:37 bostic Exp $
+// $Id: QueryRunner.cpp,v 1.19 2006/10/30 17:46:05 bostic Exp $
 //
 
 #include "dbxml/DbXml.hpp"
@@ -154,7 +153,7 @@ namespace {
   }
 }
 
-QueryRunner::QueryRunner(const std::string &configFile) : writeResults(false), debug(false), outputFileName(0) {
+QueryRunner::QueryRunner(const std::string &configFile) : outputFileName(0), writeResults(false), debug(false)  {
 
     initialise();
     parseConfigFile(configFile);
@@ -294,7 +293,7 @@ void QueryRunner::runTests() const {
           inputString = previousContainerName;
           replacement = currentContainerName;
         }
-        int inputIndex = queryString.find(inputString, 0);
+        unsigned int inputIndex = queryString.find(inputString, 0);
         while(inputIndex != std::string::npos) {
           queryString.replace(inputIndex, inputString.length(), replacement);
           inputIndex = queryString.find(inputString, 0);
