@@ -1,10 +1,9 @@
 //
 // See the file LICENSE for redistribution information.
 //
-// Copyright (c) 2002-2005
-//	Sleepycat Software.  All rights reserved.
+// Copyright (c) 2002,2006 Oracle.  All rights reserved.
 //
-// $Id: InputParser.hpp,v 1.4 2005/04/05 16:44:28 bostic Exp $
+// $Id: InputParser.hpp,v 1.7 2006/10/30 17:46:08 bostic Exp $
 //
 
 #ifndef __INPUTPARSER_HPP
@@ -32,11 +31,11 @@ private:
   void openCurly();
   void closeCurly();
   void endOfArg(Args &args);
-  void addArgument(Args &args, int start, int end);
+  void addArgument(Args &args, std::string::size_type start, std::string::size_type end);
   void nextLine(std::istream &input, Args &args);
 
   std::string line_;
-  int start_, pos_;
+  std::string::size_type start_, pos_;
   char quoteChar_;
   int curlyBraceDepth_;
   bool seenNonWS_;
