@@ -1,10 +1,9 @@
 //
 // See the file LICENSE for redistribution information.
 //
-// Copyright (c) 2002-2005
-//	Sleepycat Software.  All rights reserved.
+// Copyright (c) 2002,2006 Oracle.  All rights reserved.
 //
-// $Id: OpenContainerCommand.cpp,v 1.8 2005/04/05 16:44:28 bostic Exp $
+// $Id: OpenContainerCommand.cpp,v 1.15.2.1 2006/12/19 15:15:56 gmf Exp $
 //
 
 #include "OpenContainerCommand.hpp"
@@ -31,7 +30,9 @@ string OpenContainerCommand::getMoreHelp() const
 {
 	return
 		string("Usage: openContainer <container> [[no]validate]\n")+
-		string("This command uses the XmlManager::openContainer() method.");
+		string("This command uses the XmlManager::openContainer() method. It also sets\n") +
+		string("the default collection, using the XmlQueryContext::setDefaultCollection()\n") +
+		string("method.");
 }
 
 void OpenContainerCommand::execute(Args &args, Environment &env)
@@ -46,8 +47,11 @@ void OpenContainerCommand::execute(Args &args, Environment &env)
 	}
 	if (env.transactions())
 		flags |= DBXML_TRANSACTIONAL;
-	env.deleteContainer();
-	env.container() = new XmlContainer(
+	XmlContainer *newContainer = new XmlContainer(
 		env.db().openContainer(args[1], flags));
+	env.deleteContainer();
+	env.container() = newContainer;
+
+	env.context().setDefaultCollection("dbxml:///" + env.container()->getName() + "/");
 }
 
