@@ -1,10 +1,9 @@
 //
 // See the file LICENSE for redistribution information.
 //
-// Copyright (c) 2002-2005
-//	Sleepycat Software.  All rights reserved.
+// Copyright (c) 2002,2006 Oracle.  All rights reserved.
 //
-// $Id: SetVariableCommand.cpp,v 1.4 2005/04/05 16:44:29 bostic Exp $
+// $Id: SetVariableCommand.cpp,v 1.7 2006/10/30 17:46:08 bostic Exp $
 //
 
 #include "SetVariableCommand.hpp"
@@ -30,7 +29,7 @@ string SetVariableCommand::getBriefHelp() const
 string SetVariableCommand::getMoreHelp() const
 {
 	return
-		string("Usage: setVariable <varName> <value>\n") +
+		string("Usage: setVariable <varName> <value> (<value>...)\n") +
 		string("This command uses XmlQueryContext::setVariableValue()\n") +
 		string("to set a variable value (untyped).  Use setTypedVariable\n") +
 		string("to set a typed variable.");
@@ -38,12 +37,21 @@ string SetVariableCommand::getMoreHelp() const
 
 void SetVariableCommand::execute(Args &args, Environment &env)
 {
-	if(args.size() != 3) {
+	if(args.size() < 3) {
 		throw CommandException("Wrong number of arguments");
 	}
 	
-	if(env.verbose()) cout << "Setting $" << args[1] << " = " <<
-		args[2] << endl;
-	env.context().setVariableValue(args[1], args[2]);
+	if(env.verbose()) cout << "Setting $" << args[1] << " =";
+
+	XmlResults results = env.db().createResults();
+
+	for(unsigned int arg = 2; arg < args.size(); ++arg) {
+		if(env.verbose()) cout << " " << args[arg];
+		results.add(args[arg]);
+	}
+
+	if(env.verbose()) cout << endl;
+
+	env.context().setVariableValue(args[1], results);
 }
 
