@@ -1,10 +1,9 @@
 //
 // See the file LICENSE for redistribution information.
 //
-// Copyright (c) 2002-2005
-//	Sleepycat Software.  All rights reserved.
+// Copyright (c) 2002,2006 Oracle.  All rights reserved.
 //
-// $Id: inputStreams.cpp,v 1.13 2005/04/20 18:31:45 bostic Exp $
+// $Id: inputStreams.cpp,v 1.21 2006/10/30 17:46:11 bostic Exp $
 //
 
 // Test program for XmlInputStream (unit tests for the class, plus some functional tests
@@ -15,8 +14,8 @@
 #include <sstream>
 #include "string.h"
 
-// Pathan includes
-#include <pathan/PathanException.hpp>
+// XQilla includes
+#include <xqilla/exceptions/XQException.hpp>
 
 // Xerces includes
 #include <xercesc/dom/DOMException.hpp>
@@ -66,14 +65,18 @@ using namespace DbXmlTest;
 using namespace std;
 using namespace DbXmlInputStreamTest;
 
-void doTest(TestLogger &log, const std::string &testid, bool transacted, bool nodeLevelStorage, const std::string &path2DbEnv, StreamType type, const std::string &localFile, const std::string &baseId);
+void doTest(TestLogger &log, const std::string &testid, bool cdb, bool transacted,
+	    bool nodeLevelStorage, bool indexNodes, const std::string &path2DbEnv,
+	    StreamType type, const std::string &localFile, const std::string &baseId);
 void usage(const std::string &progname, int exitCode);
 
 int main(int argc, char **argv)
 {
   string testid;                 // arbitrary id for test (used for file names)
   bool transacted(false);        // true iff a transacted environment is used
+  bool cdb(false);               // true iff a CDS environment is used
   bool nodeLevelStorage(false);  // node level storage (whole docs otherwise)
+  bool indexNodes(false);        // use the DBXML_INDEX_NODES flag
   bool verifyLog(false);         // true iff log file is to be verified
   string path2DbEnv;             // path to DB XML env (must exist)
   string logDir("./");           // path to dir to contain log file
@@ -83,6 +86,8 @@ int main(int argc, char **argv)
 
   int err(1);
 
+  cout << "inputstreams" << endl;
+  cerr << "inputstreams" << endl;
   for(int i=1; i<argc; ++i) {
     if(argv[i][0] == '-') {
       // help
@@ -102,12 +107,20 @@ int main(int argc, char **argv)
       else if(!strncmp(argv[i], "-t", 2) || !strncmp(argv[i], "--transacted", 12)) {
         transacted = true;
       }
+      // CDB environment
+      else if(!strncmp(argv[i], "-c", 2) || !strncmp(argv[i], "--cdb", 5)) {
+        cdb = true;
+      }
       // NLS
       else if(!strncmp(argv[i], "-n", 2) || !strncmp(argv[i], "--nls", 5)) {
         nodeLevelStorage = true;
       }
+      // node indexes
+      else if(!strncmp(argv[i], "--indexnodes", 12)) {
+        indexNodes = true;
+      }
       // test id
-      else if(!strncmp(argv[i], "-i", 2) || !strncmp(argv[i], "--id", 4)) {
+      else if(!strncmp(argv[i], "--id", 4)) {
         const char* const arg = argv[++i];
         if(!arg || !strlen(arg)) {
           cerr << "No test ID specified!" << endl;
@@ -124,9 +137,9 @@ int main(int argc, char **argv)
         }
         // ensure we have a trailing slash
         string tmp(arg);
-        int pos = (int) tmp.rfind("/");
+        unsigned int pos = tmp.rfind("/");
         if(pos != tmp.length()-1) { //maybe on windows?
-          pos = (int) tmp.rfind("\\");
+          pos = tmp.rfind("\\");
         }
         if (pos != tmp.length()-1) { //just add a slash
           tmp.append("/");
@@ -211,16 +224,21 @@ int main(int argc, char **argv)
   INFO_MSG(log, "Unit test driver running for test id %1") << testid;
   INFO_MSG(log, "Using DB environment '%1'") << path2DbEnv;
   INFO_MSG(log, "Using a %1 transacted environment") << (transacted ? "" : "non");
+  if (cdb)
+	  INFO_MSG(log, "Using a CDS environment");
   if(nodeLevelStorage) {
     INFO_MSG(log, "Using a node level storage model");
   } else {
     INFO_MSG(log, "Using a document level storage model");
   }
+  if(indexNodes) {
+    INFO_MSG(log, "Indexing nodes");
+  }
 
   TEST_MSG(log, "Testing an input stream that uses ") << typeToString(streamType);
 
   try {
-    doTest(log, testid, transacted, nodeLevelStorage, path2DbEnv, streamType, localFile, baseId);
+    doTest(log, testid, cdb, transacted, nodeLevelStorage, indexNodes, path2DbEnv, streamType, localFile, baseId);
     ENV_MSG(log, "Successful completion of test");
     err = 0;
   }
@@ -230,8 +248,11 @@ int main(int argc, char **argv)
   catch(XERCES_CPP_NAMESPACE_QUALIFIER XMLException &e) {
     INFO_MSG(log, "Caught Xerces Exception: %1") << XMLChToUTF8(e.getMessage()).str();
   }
-  catch(PathanException &e) {
-    INFO_MSG(log, "Caught Pathan Exception: %1") << XMLChToUTF8(e.getString()).str();
+  catch(XQException &e) {
+    INFO_MSG(log, "Caught XQilla Exception: %1") << XMLChToUTF8(e.getError()).str();
+  }
+  catch(DbException &de) {
+    INFO_MSG(log, "Caught DB Exception: %1: %2") << de.get_errno() << (char*)de.what();
   }
   catch(XmlException &e) {
     INFO_MSG(log, "Caught DB XML Exception: %1: %2") << e.getExceptionCode() << (char*)e.what();
@@ -257,9 +278,9 @@ int main(int argc, char **argv)
 
 void usage(const std::string &progname, int exitCode)
 {
-  int pos = (int) progname.rfind('/');
+  unsigned int pos = progname.rfind('/');
   if(pos == string::npos) {
-    pos = (int) progname.rfind('\\');
+    pos = progname.rfind('\\');
   }
   if(pos == string::npos) {
     pos = 0;
@@ -270,9 +291,11 @@ void usage(const std::string &progname, int exitCode)
 
   cerr << "Usage: " << progname.substr(pos) << " [OPTIONS] --env1 path_to_dbenv --env2 path_to_dbenv " << endl;
   cerr << "Options:" << endl;
+  cerr << "\t-c|--CDB\t\tuse a CDS environment" << endl;
   cerr << "\t-t|--transacted\t\tuse a transacted environment" << endl;
   cerr << "\t-n|--nls\t\tuse node level storage" << endl;
-  cerr << "\t-i|--id id\t\tassign a (test) ID to this invocation" << endl;
+  cerr << "\t--indexnodes\t\tuse DBXML_INDEX_NODES flag" << endl;
+  cerr << "\t--id id\t\tassign a (test) ID to this invocation" << endl;
   cerr << "\t-h|--help|-?\t\tprint this help" << endl;
   cerr << "\t-s|--stream\t\tstream type [file|url|membuf|stdin]" << endl;
   cerr << "\t-b|--base\t\tbaseId for URL type" << endl;
@@ -293,19 +316,38 @@ void loadDocument(TestLogger &log, XmlInputStream *xis, XmlContainer *container,
 void streamDocument(TestLogger &log, XmlInputStream *xis, XmlManager &db);
 
 // the actual test
-void doTest(TestLogger &log, const std::string &testid, bool transacted, bool nodeLevelStorage, const std::string &path2DbEnv, StreamType type, const std::string &localFile, const std::string &baseId)
+void doTest(TestLogger &log, const std::string &testid, bool cdb, bool transacted,
+	    bool nodeLevelStorage, bool indexNodes, const std::string &path2DbEnv,
+	    StreamType type, const std::string &localFile, const std::string &baseId)
 {
   TEST_MSG(log, "Starting test");
 
   ///////////////////////////////////////////////////////
   // prepare - database, container
-  u_int32_t flags = (DB_CREATE|DB_INIT_LOCK|DB_INIT_LOG|DB_INIT_MPOOL);
-  if(transacted) flags |= DB_INIT_TXN;
+  u_int32_t flags = (DB_CREATE|DB_INIT_MPOOL);
+  if (cdb) {
+	  flags |= DB_INIT_CDB;
+	  cout << "Using CDB" << endl;
+	  cerr << "Using CDB" << endl;
+  } else if (transacted) {
+	  flags |=( DB_INIT_TXN|DB_INIT_LOG|DB_INIT_LOCK);
+	  cout << "Using txns" << endl;
+	  cerr << "Using txns" << endl;
+  }
+  try {
+	  // remove original env first
+	  DbEnv tenv(0);
+	  tenv.remove(path2DbEnv.c_str(), 0);
+  } catch (...) {}
+  
   DbEnv env(0);
+  if (flags & DB_INIT_CDB)
+	  env.set_flags(DB_CDB_ALLDB, 1);
   env.open(path2DbEnv.c_str(), flags, 0);
   XmlManager db(&env);
 
   db.setDefaultContainerType(nodeLevelStorage ? XmlContainer::NodeContainer : XmlContainer::WholedocContainer);
+  db.setDefaultContainerFlags(indexNodes ? DBXML_INDEX_NODES : 0);
 
   XmlTransaction *txn = 0;
   if(transacted) {
