@@ -1,10 +1,9 @@
 //
 // See the file LICENSE for redistribution information.
 //
-// Copyright (c) 2002-2005
-//	Sleepycat Software.  All rights reserved.
+// Copyright (c) 2002,2006 Oracle.  All rights reserved.
 //
-// $Id: TestEnvironment.cpp,v 1.18 2005/04/20 18:31:46 bostic Exp $
+// $Id: TestEnvironment.cpp,v 1.31 2006/10/30 17:46:13 bostic Exp $
 //
 
 // DB XML includes (external)
@@ -21,14 +20,12 @@
 #include <xercesc/util/XMLString.hpp>
 #include <xercesc/dom/DOM.hpp>
 
-// Pathan includes
-#include <pathan/Sequence.hpp>
-#include <pathan/internal/XPath2MemoryManagerImpl.hpp>
-
-// XQuery includes
-#include <xquery/XQEvaluator.hpp>
-#include <xquery/context/XQContextImpl.hpp>
-#include <xquery/XQQuery.hpp>
+// XQilla includes
+#include <xqilla/runtime/Sequence.hpp>
+#include <xqilla/framework/XPath2MemoryManagerImpl.hpp>
+#include <xqilla/items/Node.hpp>
+#include <xqilla/xqilla-simple.hpp>
+#include <xqilla/context/impl/XQContextImpl.hpp>
 
 // test code includes
 #include "../util/TestLogging.hpp"
@@ -141,9 +138,8 @@ namespace {
 }
 
 TestEnvironment::TestEnvironment(TestLogger *log, const std::string &db_home,
-				 bool debug, bool transacted,
-				 bool wholeDocumentStorage, bool secure,
-				 bool validate)
+	bool debug, bool transacted, bool wholeDocumentStorage, bool indexNodes,
+	bool secure, bool validate)
 	: log_(log),
 	  db_(0),
 	  container_(0),
@@ -154,6 +150,7 @@ TestEnvironment::TestEnvironment(TestLogger *log, const std::string &db_home,
 	  transacted_(transacted),
 	  storageModel_(wholeDocumentStorage ? XmlContainer::WholedocContainer
 			: XmlContainer::NodeContainer),
+	  flags_(indexNodes ? DBXML_INDEX_NODES : 0),
 	  secure_(secure),
 	  validate_(validate)
 {
@@ -216,8 +213,8 @@ void TestEnvironment::run(const std::string &id, const std::string &dir,
 	if(log_) ENV_MSG(*log_, "Registered test resolver for local directory '%1'") << dataDirectory_;
 
 	// set up the database for this group of tests
-	addGroupDocuments();
 	addGroupIndexes();
+	addGroupDocuments();
 	addGroupContext();
 
 	if(log_) ENV_MSG(*log_, "Context has a return type of '%1'") << context_->getReturnType();
@@ -267,15 +264,17 @@ Sequence TestEnvironment::query(
 {
 	assert(testDOM_);
 	DOMNode *n = contextNode ? contextNode : testDOM_->getDocumentElement();
-	XQQuery *parsedQuery = XQEvaluator::parse(UTF8ToXMLCh(query).str(),
-						  xqContext_);
+
+	XQilla xqilla;
+	XQQuery *parsedQuery = xqilla.parse(UTF8ToXMLCh(query).str(), XQilla::XQUERY,
+		xqContext_, 0, XQilla::NO_ADOPT_CONTEXT);
 
 	// set node context and evaluate query
 	xqContext_->setExternalContextNode(n);
 	xqContext_->setContextPosition(1);
 	xqContext_->setContextSize(1);
 	Sequence result = parsedQuery->
-		evaluate(xqContext_).toSequence(xqContext_);
+		execute(xqContext_)->toSequence(xqContext_);
 	return result;
 }
 
@@ -288,23 +287,24 @@ void TestEnvironment::addIndexSpecifications(Sequence &result)
 		return;
 	for (Sequence::iterator it = result.begin(); it != result.end(); it++) {
 		if((*it)->isNode()) {
+			const DOMNode *domnode = (const DOMNode*)(*it)->getInterface(Node::gXerces);
+			assert(domnode != 0);
+
 			string uri, node, index;
-			getIndexSpecification(((const Node::Ptr)(*it))->
-					      getDOMNode(),
-					      uri, node, index);
+			getIndexSpecification(domnode, uri, node, index);
 			indexSpec.addIndex(uri, node, index);
 
 			if(log_) ENV_MSG(*log_, "Added the index specification '%1' / '%2' / '%3'") << uri << node << index;
 		}
 	}
-
-	if(transacted_) {
-		XmlTransaction myTxn = db_->createTransaction();
-		container_->setIndexSpecification(myTxn, indexSpec, *uc_);
-		myTxn.commit();
-	} else {
-		container_->setIndexSpecification(indexSpec, *uc_);
-	}
+// don't transact this operation -- it reindexes the world
+//	if(transacted_) {
+//		XmlTransaction myTxn = db_->createTransaction();
+//		container_->setIndexSpecification(myTxn, indexSpec, *uc_);
+//		myTxn.commit();
+//	} else {
+	container_->setIndexSpecification(indexSpec, *uc_);
+//	}
 }
 
 // adds variables from a query result
@@ -314,9 +314,11 @@ void TestEnvironment::addVariables(Sequence &result)
 		return;
 	for (Sequence::iterator it = result.begin(); it != result.end(); it++) {
 		if((*it)->isNode()) {
+			const DOMNode *domnode = (const DOMNode*)(*it)->getInterface(Node::gXerces);
+			assert(domnode != 0);
+
 			string name, value;
-			getVariable(((const Node::Ptr)(*it))->
-				    getDOMNode(), name, value);
+			getVariable(domnode, name, value);
 
 			XmlValue v(value);
 			context_->setVariableValue(name, v);
@@ -384,7 +386,9 @@ void TestEnvironment::getTextFromChildNode(DOMNode *node,
 		throw std::exception();
 	}
 
-	const DOMNode *n = ((const Node::Ptr)(theNode))->getDOMNode();
+	const DOMNode *n = (const DOMNode*)theNode->getInterface(Node::gXerces);
+	assert(n != 0);
+
 	if(n->getFirstChild()) {
 		const XMLCh* xstr = n->getFirstChild()->getNodeValue();
 		text.assign(XMLChToUTF8(xstr).str());
@@ -514,9 +518,9 @@ void TestEnvironment::putFileInDocument(const std::string &file, DbXml::XmlDocum
 
 	// set name of document (attempt to strip off path information)
 	string name(file);
-	int pos = (int) name.rfind("/");
+	unsigned int pos = name.rfind("/");
 	if(pos == string::npos)  // windows
-		pos = (int) name.rfind("\\");
+		pos = name.rfind("\\");
 	if(pos != string::npos)
 		name.assign(file, pos+1, file.length());
 
@@ -573,6 +577,11 @@ XmlContainer::ContainerType &TestEnvironment::storageModel()
 	return storageModel_;
 }
 
+u_int32_t TestEnvironment::flags()
+{
+	return flags_;
+}
+
 std::string &TestEnvironment::id()
 {
 	return testId_;
@@ -617,18 +626,34 @@ void TestEnvironment::reloadContainer(XmlTransaction *xtxn)
 
 void TestEnvironment::createDBXMLEnvironment(const std::string &db_home)
 {
-	// create the database
-	u_int32_t dbFlags = (DB_CREATE|DB_INIT_LOCK|DB_INIT_LOG|DB_INIT_MPOOL);
+	// create the environment and manager
+	DbEnv *env = new DbEnv(0);
 	u_int32_t dbxmlFlags = DBXML_ADOPT_DBENV;
+	int dberr = 0;
+	u_int32_t dbFlags = (DB_CREATE|DB_INIT_LOCK|DB_INIT_LOG|DB_INIT_MPOOL);
 	if(transacted_) dbFlags |= DB_INIT_TXN;
 	if(!secure_) dbxmlFlags |= DBXML_ALLOW_EXTERNAL_ACCESS;
-	DbEnv *env = new DbEnv(0);
 	env->set_cachesize(0, 64 * 1024 * 1024, 1);
 	env->set_error_stream(&cerr);
-	env->open(db_home.c_str(), dbFlags, 0);
+	env->set_lk_max_lockers(10000);
+	env->set_lk_max_locks(10000);
+	env->set_lk_max_objects(10000);
+
+	try {
+		dberr = env->open(db_home.c_str(), DB_JOINENV, 0);
+	} catch (DbException &de) {
+		dberr = de.get_errno();
+	}
+	if (dberr != 0) {
+		if(log_) ENV_MSG(*log_, "Creating new environment");
+		dberr = env->open(db_home.c_str(), dbFlags, 0);
+	} else {
+		if(log_) ENV_MSG(*log_, "Joined existing environment");
+	}
 
 	db_ = new XmlManager(env, dbxmlFlags);
 	db_->setDefaultContainerType(storageModel_);
+	db_->setDefaultContainerFlags(flags_);
 
 	// create other bits and pieces for the environment
 	context_ = new XmlQueryContext(db_->createQueryContext());
@@ -644,9 +669,11 @@ Sequence TestEnvironment::parseTestInstructions()
 	if(log_) ENV_MSG(*log_, "Reading index file '%1'") << indexFile;
 
 	UTF8ToXMLCh ifile(indexFile.c_str());
-	Sequence dseq = xqContext_->resolveDocument(ifile.str());
+	Sequence dseq = xqContext_->resolveDocument(ifile.str(), 0);
+
+	const DOMNode *dnode = (const DOMNode*)dseq.first()->getInterface(Node::gXerces);
+	assert(dnode != 0);
 
-	DOMNode *dnode = (DOMNode*)((const Node::Ptr)dseq.first())->getDOMNode();
 	XERCES_CPP_NAMESPACE_QUALIFIER DOMDocument *document =
 		(XERCES_CPP_NAMESPACE_QUALIFIER DOMDocument *)dnode;
 
@@ -688,7 +715,7 @@ void TestEnvironment::openContainer(const std::string &name)
 	string containerName(name);
 	if(name.empty()) containerName = testId_ + ".dbxml";
 
-	u_int32_t oflags = 0;
+	u_int32_t oflags = flags_;
 	if(name.empty())
 		oflags |= DB_CREATE;
 	if (validate_)
@@ -701,6 +728,7 @@ void TestEnvironment::openContainer(const std::string &name)
 	} else {
 		container_ = new XmlContainer(db_->openContainer(containerName, oflags));
 	}
+	context_->setDefaultCollection(containerName);
 
 	ostringstream os;
 	os << "Opened the container '" << container_->getName() << "'";
@@ -730,8 +758,8 @@ void TestEnvironment::addGroupContext()
 	Sequence result = query("//test/context");
 	if(result.getLength() == 0) return;
 
-	const DOMNode *context =
-		((const Node::Ptr)result.first())->getDOMNode();
+	const DOMNode *context = (const DOMNode*)result.first()->getInterface(Node::gXerces);
+	assert(context != 0);
 
 	// return type (optional)
 	DOMNode *n = context->getFirstChild();
@@ -770,8 +798,10 @@ void TestEnvironment::addGroupDocuments()
 
 	for (Sequence::iterator it = result.begin(); it != result.end(); it++) {
 		if((*it)->isNode()) {
-			const XMLCh *xstr = ((const Node::Ptr)(*it))->
-				getDOMNode()->getFirstChild()->getNodeValue();
+			const DOMNode *domnode = (const DOMNode*)(*it)->getInterface(Node::gXerces);
+			assert(domnode != 0);
+
+			const XMLCh *xstr = domnode->getFirstChild()->getNodeValue();
 
 			// create a document from the contents and
 			// add to container
@@ -807,7 +837,9 @@ TestEnvironment::TestType TestEnvironment::getTestType()
 	TestType type(NONE);
 
 	Sequence result = query("//test/category");
-	const DOMNode* n = ((const Node::Ptr)result.first())->getDOMNode();
+	const DOMNode *n = (const DOMNode*)result.first()->getInterface(Node::gXerces);
+	assert(n != 0);
+
 	if(n->getFirstChild()) {
 			const XMLCh* xstr = n->getFirstChild()->getNodeValue();
 			type = convertTestType(XMLChToUTF8(xstr).str());
