@@ -1,10 +1,9 @@
 //
 // See the file LICENSE for redistribution information.
 //
-// Copyright (c) 2002-2005
-//	Sleepycat Software.  All rights reserved.
+// Copyright (c) 2002,2006 Oracle.  All rights reserved.
 //
-// $Id: UpdateDocumentTest.cpp,v 1.12 2005/04/20 18:31:46 bostic Exp $
+// $Id: UpdateDocumentTest.cpp,v 1.19 2006/10/30 17:46:14 bostic Exp $
 //
 
 // tests the updating of documents (Xerces-C DOM methods)
@@ -18,14 +17,15 @@
 #include <iostream>
 #include <sstream>
 #include <map>
+#include <cassert>
 
 // Xerces includes
 #include <xercesc/util/XMLUniDefs.hpp>
 #include <xercesc/util/XMLString.hpp>
 #include <xercesc/dom/DOM.hpp>
 
-// Pathan includes
-#include <pathan/XPath2Result.hpp>
+// XQilla includes
+#include <xqilla/items/Node.hpp>
 
 // test code includes
 #include "../util/TestLogging.hpp"
@@ -209,7 +209,7 @@ public:
 			DOMElement *ele = static_cast<DOMElement*>(node);
 			DOMAttr *attr = ele->getAttributeNode(depth);
 			if(!attr) {
-				if(log) ERROR_MSG(*log, "Attribute does not exist") << XMLChToUTF8(node->getTextContent()).str();
+				if(log) ERROR_MSG(*log, "Attribute does not exist");
 				return;
 			}
 			else if(!XMLString::equals(attr->getNodeValue(), ten)) {
@@ -223,7 +223,69 @@ public:
 			}
 		}
 
-		if(log) TEST_MSG(*log, "Text content OK");
+		if(log) TEST_MSG(*log, "Attribute OK");
+		footer(log, "check", getName(), node);
+	}
+};
+
+////////////////////////////////////////////////////////////////////////
+// SetAttributeNS
+
+static XMLCh random_uri[] = { 'r', 'a', 'n', ':', 'd', 'o', 'm', 0 };
+
+class SetAttributeNSFunctor : public Functor
+{
+public:
+	SetAttributeNSFunctor() : Functor("setAttributeNS") {}
+
+	//////
+	// modify
+	virtual void modify(TestLogger *log, XERCES_CPP_NAMESPACE_QUALIFIER DOMNode *node) const {
+		header(log, "modify", getName(), node);
+
+		if(node->getNodeType() == XERCES_CPP_NAMESPACE_QUALIFIER DOMNode::ELEMENT_NODE) {
+			// modify - create a "depth" attribute
+			DOMElement *ele = static_cast<DOMElement*>(node);
+			ele->setAttributeNS(random_uri, depth, ten);
+		}
+
+		footer(log, "modify", getName(), node);
+	}
+
+	//////
+	// check
+	virtual void check(TestLogger *log, XERCES_CPP_NAMESPACE_QUALIFIER DOMNode *node) const {
+		header(log, "check", getName(), node);
+
+		if(node->getNodeType() == XERCES_CPP_NAMESPACE_QUALIFIER DOMNode::ELEMENT_NODE) {
+			DOMElement *ele = static_cast<DOMElement*>(node);
+
+			DOMAttr *attr = ele->getAttributeNodeNS(random_uri, depth);
+			if(!attr) {
+				if(log) ERROR_MSG(*log, "Attribute does not exist");
+				return;
+			}
+			else if(!XMLString::equals(attr->getNodeValue(), ten)) {
+				if(log) ERROR_MSG(*log, "Attribute equals '%1'") << XMLChToUTF8(attr->getNodeValue()).str();
+				return;
+			}
+
+			if(string(XMLChToUTF8(node->getLocalName()).str()) == "TestDoc") {
+				DOMAttr *attr2 = ele->getAttributeNodeNS(UTF8ToXMLCh("http://www.w3.org/2000/xmlns/").str(),
+					UTF8ToXMLCh("t").str());
+				if(!attr2) {
+					if(log) ERROR_MSG(*log, "Namespace attribute does not exist");
+					return;
+				}
+			}
+		}
+		else {
+			if(node->getAttributes() != 0) {
+				if(log) ERROR_MSG(*log, "Node type '%1' has attributes") << node->getNodeType();
+			}
+		}
+
+		if(log) TEST_MSG(*log, "Attribute OK");
 		footer(log, "check", getName(), node);
 	}
 };
@@ -283,6 +345,8 @@ void UpdateDocumentTest::doTest(const std::string &method, const std::string &xp
     xqc->setEvaluationType(XmlQueryContext::Lazy);
   }
 
+  xqc->setNamespace("t", "test:namespace");
+
   // start a transaction
   XmlTransaction *xtxn = 0;
   bool transacted = env_.transacted();
@@ -498,7 +562,8 @@ void UpdateDocumentTest::prepare()
   // build associations between update method IDs and methods
   updateMethods_["setTextContent"] = new SetTextContentFunctor;
   updateMethods_["setTextContent2"] = new SetTextContent2Functor;
-  updateMethods_["setAttribute"]   = new SetAttributeFunctor;
+  updateMethods_["setAttribute"] = new SetAttributeFunctor;
+  updateMethods_["setAttributeNS"] = new SetAttributeNSFunctor;
 
   //////////////
   // get the name of the document (needed for getting orginal document)
@@ -548,15 +613,14 @@ void UpdateDocumentTest::prepare()
 // for each test, reads an update method and an xpath query
 void UpdateDocumentTest::execute()
 {
-	TestLogger *log = env_.log();
-
 	// iterate over individual tests (<base> elements in the index file)
 	Sequence result = env_.query("/test/base");
 	for (Sequence::iterator it = result.begin();
 	     it != result.end(); it++) {
 		if((*it)->isNode()) {
-			const DOMNode *node = ((Node::Ptr)(*it))->
-				getDOMNode();
+			const DOMNode *node = (const DOMNode*)(*it)->getInterface(Node::gXerces);
+			assert(node != 0);
+
 			// for each test, get the (update) method, query, ...
 			string method;
 			env_.getTextFromChildNode((DOMNode*)node,
