@@ -1,9 +1,8 @@
 # See the file LICENSE for redistribution information
 #
-# Copyright (c) 2002-2005
-# 	Sleepycat Software.  All rights reserved.
+# Copyright (c) 2002,2006 Oracle.  All rights reserved.
 #
-# $Id: test.tcl,v 1.55 2005/04/20 18:31:40 bostic Exp $
+# $Id: test.tcl,v 1.68 2006/10/30 17:46:09 bostic Exp $
 #
 
 source ./include.tcl
@@ -41,6 +40,10 @@ if { [string first "exec format error" $ret] != -1 } {
 
 source $test_path/xmlparams.tcl
 
+# Set up any OS-specific values
+global tcl_platform
+set is_windows_test [is_substr $tcl_platform(os) "Win"]
+
 # Set variables required to use standard testutils, even if they
 # never get exercised by XML.
 set dict $db_test_path/wordlist
@@ -82,59 +85,87 @@ proc run_xml {} {
     lappend container_types $XmlContainer_WholedocContainer
     lappend container_types $XmlContainer_NodeContainer
 
+    list set indexnodes_types {}
+    lappend indexnodes_types 1
+    lappend indexnodes_types 0
+
     foreach type $container_types {
-        if { $type == $XmlContainer_WholedocContainer } {
-            puts "Using whole document storage"
-        } elseif { $type == $XmlContainer_NodeContainer } {
-            puts "Using node level storage"
-        }
-
-        # Run with no environment
-        puts "  Running tests with no environment"
-        foreach test $test_names(xml) {
-            if [catch {exec $tclsh_path \
-                           << "source $test_path/test.tcl; run_xmltest $test {} $type" \
-                           >>& ALL.OUT } res] {
-
-                set o [open ALL.OUT a]
-                puts $o "FAIL: $test: $res"
-                close $o
-
-                # test group is abandoned - forcibly clean up test environment
-                fileremove -f $testdir
-                file mkdir $testdir
+        foreach indexnodes $indexnodes_types {
+            if { $type == $XmlContainer_WholedocContainer } {
+                puts -nonewline "Using whole document storage"
+            } elseif { $type == $XmlContainer_NodeContainer } {
+                puts -nonewline "Using node level storage"
             }
-        }
-
-        # Run under run_tdsmethod (transactional environment)
-        puts "  Running tests in a transactional environment"
-        foreach test $test_names(xml) {
-            if [catch {exec $tclsh_path \
-                           << "source $test_path/test.tcl; run_tdsmethod $test {} $type" \
-                           >>& ALL.OUT } res] {
-                set o [open ALL.OUT a]
-                puts $o "FAIL: run_tdsmethod $test: $res"
-                close $o
-
-                # test group is abandoned - forcibly clean up test environment
-                fileremove -f $testdir
-                file mkdir $testdir
+            if { $indexnodes == 1} {
+                puts " with indexed nodes"
+            } else {
+                puts ""
             }
-        }
 
-        # Run under run_tdsmethod with -thread
-        # TODO: test whether berkdb_env supports -thread
-        #puts "Running tests in a threaded environment"
-        #foreach test $test_names(xml) {
-        #	if [catch {exec $tclsh_path \
-            #	    << "source $test_path/test.tcl; \
-            #	    run_tdsmethod $test thread $XmlContainer_WholedocContainer" \
-            #	    >>& ALL.OUT } res] {
-        #		set o [open ALL.OUT a]
-        #		puts $o "FAIL: run_tdsmethod $test thread: $res"
-        #		close $o
-        #	}
-        #}
+            # Run with no environment
+            puts "  Running tests with no environment (DS)"
+            foreach test $test_names(xml) {
+                if [catch {exec $tclsh_path \
+                               << "source $test_path/test.tcl; run_xmltest $test {} $type $indexnodes" \
+                               >>& ALL.OUT } res] {
+
+                    set o [open ALL.OUT a]
+                    puts $o "FAIL: $test: $res"
+                    close $o
+
+                    # test group is abandoned - forcibly clean up test environment
+                    fileremove -f $testdir
+                    file mkdir $testdir
+                }
+            }
+
+            # Run under run_tdsmethod (transactional environment)
+            puts "  Running tests in a transactional environment (TDS)"
+            foreach test $test_names(xml) {
+                if [catch {exec $tclsh_path \
+                               << "source $test_path/test.tcl; run_tdsmethod $test {} $type $indexnodes" \
+                               >>& ALL.OUT } res] {
+                    set o [open ALL.OUT a]
+                    puts $o "FAIL: run_tdsmethod $test: $res"
+                    close $o
+
+                    # test group is abandoned - forcibly clean up test environment
+                    fileremove -f $testdir
+                    file mkdir $testdir
+                }
+            }
+
+#skip CDS for now, until fixed
+            # Run under run_cdsmethod (locking environment)
+            puts "  Running tests in a concurrent environment (CDS)"
+            foreach test $test_names(xml) {
+                if [catch {exec $tclsh_path \
+                               << "source $test_path/test.tcl; run_cdsmethod $test {} $type $indexnodes" \
+                               >>& ALL.OUT } res] {
+                    set o [open ALL.OUT a]
+                    puts $o "FAIL: run_cdsmethod $test: $res"
+                    close $o
+
+                    # test group is abandoned - forcibly clean up test environment
+                    fileremove -f $testdir
+                    file mkdir $testdir
+                }
+            }
+
+            # Run under run_tdsmethod with -thread
+            # TODO: test whether berkdb_env supports -thread
+            #puts "Running tests in a threaded environment"
+            #foreach test $test_names(xml) {
+            #	if [catch {exec $tclsh_path \
+                #	    << "source $test_path/test.tcl; \
+                #	    run_tdsmethod $test thread $XmlContainer_WholedocContainer 0" \
+                #	    >>& ALL.OUT } res] {
+            #		set o [open ALL.OUT a]
+            #		puts $o "FAIL: run_tdsmethod $test thread: $res"
+            #		close $o
+            #	}
+            #}
+        }
     }
 
     # end of test run - now to analyse results...
@@ -173,12 +204,90 @@ proc run_xml {} {
     return $err
 }
 
-proc run_all { { id { 0 } } { env { t n } } { use_nls { 1 0 } } { nofork 1 } } {
+proc run_xqts { { path_to_test_suite 0 } { use_nls { 1 0 } } { indexing { node document none } } } {
+    source ./include.tcl
+
+    if { ![string compare $path_to_test_suite ?]} {
+        puts ""
+        puts "Usage:  run_xqts path_to_test_suite \{1|0\} \{node|document|none\}"
+        puts ""
+        puts "1 specifies NLS, 0 specifies DLS."
+        puts ""
+        puts "node, document, and none are the indexing types to use."
+        puts ""
+        return
+    }
+
+    set fail 0
+    foreach i $indexing {
+        foreach u $use_nls {
+            set args ""
+
+            puts ""
+            puts "\#\#\#\#\#\#\#\#\#\#\#\#\#"
+            if { $u == 1 } {
+                puts -nonewline "\# Node Level"
+                set storage "NLS"
+            } else {
+                puts -nonewline "\# Whole Document"
+                set storage "DLS"
+            }
+            puts -nonewline " Storage - "
+
+            if { ! [string compare $i "node"] } {
+                puts -nonewline "Node Indexing"
+                set storage "$storage+"
+                set args "-i"
+            } else {
+                if { ! [string compare $i "document"] } {
+                    puts -nonewline "Document Indexing"
+                    set args "-i"
+                } else {
+                    puts -nonewline "Not Indexed"
+                }
+            }
+            puts ""
+            puts "\#\#\#\#\#\#\#\#\#\#\#\#\#"
+
+            set args [concat $args "-s $storage"]
+            set args [concat $args "-h $testdir"]
+            set args [concat $args "-e"]
+            set args [concat $args [file join $test_path "xqts-errors.xml"]]
+            set args [concat $args $path_to_test_suite]
+
+            set prog [file join "." "dbxml_xqts_runner"]
+
+            # forcibly remove the TESTDIR to avoid inconsistent state
+            fileremove -f $testdir
+            file mkdir $testdir
+
+            # puts "$xmltest_path/$prog $args 2>> xqts_tests.err"
+
+            catch {
+                set output [open "|$xmltest_path/$prog $args 2>> xqts_tests.err" r]
+                while { [eof $output] != 1 } {
+                    puts -nonewline [read $output 1]
+                    flush stdout
+                }
+                close $output
+            } ret;
+
+            if { [string compare $ret ""] } {
+                set fail 1
+                dbxml_error_check_good "run_xqts $args" $ret ""
+            }
+        }
+    }
+
+    summary $fail
+}
+
+proc run_all { { id { 0 } } { env { t n } } { use_nls { 1 0 } } { index_nodes { 1 0 } } { nofork 1 } } {
     source ./include.tcl
 
     if { ![string compare $id ?]} {
         puts ""
-        puts "Usage:  run_all \{ids\} \{t*|n\}"
+        puts "Usage:  run_all \{ids\} \{t|n\} \{1|0\}"
         puts ""
         puts "\"ids\" is a list of test groups (use \"help\" to find out more about the test groups)."
         puts "If \"id\" is omitted (or is 0) then all test groups are run."
@@ -192,25 +301,30 @@ proc run_all { { id { 0 } } { env { t n } } { use_nls { 1 0 } } { nofork 1 } } {
     foreach i $id {
         foreach e $env {
             foreach u $use_nls {
+                foreach n $index_nodes {
+                    puts ""
+                    puts "\#\#\#\#\#\#\#\#\#\#\#\#\#"
+                    if { $u == 1 } {
+                        puts -nonewline "\# Node Level"
+                    } else {
+                        puts -nonewline "\# Whole Document"
+                    }
+                    puts -nonewline " Storage - "
 
-                puts ""
-                puts "\#\#\#\#\#\#\#\#\#\#\#\#\#"
-                if { $u == 1 } {
-                    puts -nonewline "\# Node Level"
-                } else {
-                    puts -nonewline "\# Whole Document"
+                    if { ! [string compare $e "t"] } {
+                        puts -nonewline "Transacted"
+                    } else {
+                        puts -nonewline "Non-transacted"
+                    }
+                    puts -nonewline " mode"
+                    if { $n == 1 } {
+                        puts -nonewline " - Nodes Indexed"
+                    }
+                    puts ""
+                    puts "\#\#\#\#\#\#\#\#\#\#\#\#\#"
+
+                    set fail [expr $fail || [run $i $e $u $n $nofork 0]]
                 }
-                puts -nonewline " Storage - "
-
-                if { ! [string compare $e "t"] } {
-                    puts -nonewline "Transacted"
-                } else {
-                    puts -nonewline "Non-transacted"
-                }
-                puts " mode:"
-                puts "\#\#\#\#\#\#\#\#\#\#\#\#\#"
-
-                set fail [expr $fail || [run $i $e $u $nofork 0]]
             }
         }
     }
@@ -222,13 +336,14 @@ proc run_all { { id { 0 } } { env { t n } } { use_nls { 1 0 } } { nofork 1 } } {
 #   uses node level storage, wraps "run"
 proc run_nls { { id 0 } {env t } { nofork 1 } } {
 	source ./include.tcl
-  run $id $env 1 $nofork
+  run $id $env 1 0 $nofork
+  run $id $env 1 1 $nofork
 }
 
 # execute a specific test group in the XML subsystem (transacted environment)
 #   allows quick testing of a particular feature
 #   summary pass/fail message printed
-proc run { { id 0 } {env t } {use_nls 0} { nofork 1 } { output_summary 1 } } {
+proc run { { id 0 } {env t } {use_nls 0} {index_nodes 0} { nofork 1 } { output_summary 1 } } {
     source ./include.tcl
     global test_names
     global errorCode
@@ -244,16 +359,23 @@ proc run { { id 0 } {env t } {use_nls 0} { nofork 1 } { output_summary 1 } } {
     # help option (this could be automated, i.e. read file headers...)
     if { ![string compare $id ?]} {
         puts ""
-        puts "Usage:  run \[id\] \[t*|n\]"
-        puts "        run_nls \[id\] \[t*|n\]"
+        puts "Usage:  run \[id\] \[t|c|n\]"
+        puts "        run_nls \[id\] \[t|c|n\]"
         puts ""
         puts "\"id\" is a test group (use \"help\" to find out more about the test groups)."
         puts "If \"id\" is omitted (or is 0) then all test groups are run."
         puts ""
-        puts "If \"n\" is specified then a NULL environment is used. Otherwise a transacted environment is used. This switch is reversed for Group 3, which only runs in a non-transacted environment."
+        puts "If \"n\" is specified then a NULL environment is used. If \"c\" is specified, a CDS environment is used.  Otherwise a transacted environment is used. This switch is reversed for Group 3, which only runs in a non-transacted environment."
         puts ""
         puts "The 'dbxml_dump' and 'dbxml_load' programs (plus 'db_dump') are run at the end of each test group, but only for a NULL environment."
         puts ""
+	puts "Also available:"
+	puts "run_xmltest|run_tdsmethod|run_cdsmethod id args storage_type index_nodes"
+	puts ""
+	puts "id is test number (e.g. xml011), args are ??? storage_type (0 or 1)"
+	puts "where 1 is node, index_nodes 0 (false) or 1"
+	puts ""
+	puts "run_envmethod id 'cdb|txn' args storage_type index_nodes"
         return
     }
 
@@ -274,13 +396,22 @@ proc run { { id 0 } {env t } {use_nls 0} { nofork 1 } { output_summary 1 } } {
             # swap environments for group 3 since this group only runs in a
             # non-transacted environment
             if { ! [string compare $env n] } {
+		puts "Running in a DS (non-transacted) environment"
                 # NULL environment
                 if { [string match "*03" $test] || $id == 3} {
                     set run_method run_tdsmethod
                 } else {
                     set run_method run_xmltest
                 }
+            } elseif { ! [string compare $env c] } {
+		puts "Running in a CDS environment"
+                if { [string match "*03" $test] || $id == 3} {
+                    set run_method run_tdsmethod
+                } else {
+                    set run_method run_cdsmethod
+                }
             } else {
+		puts "Running in a TDS (transacted) environment"
                 if { [string match "*03" $test] || $id == 3} {
                     set run_method run_xmltest
                 } else {
@@ -294,7 +425,7 @@ proc run { { id 0 } {env t } {use_nls 0} { nofork 1 } { output_summary 1 } } {
                 set err 1
                 catch {
                     puts [exec $tclsh_path \
-                              << "source $test_path/test.tcl; $run_method $test \"\" $storage_type"]
+                              << "source $test_path/test.tcl; $run_method $test \"\" $storage_type $index_nodes"]
                     set err 0
                 } ret
 
@@ -311,7 +442,7 @@ proc run { { id 0 } {env t } {use_nls 0} { nofork 1 } { output_summary 1 } } {
                 }
             } else {
                 set args {}
-                if {[catch {$run_method $test $args $storage_type} res] != 0 } {
+                if {[catch {$run_method $test $args $storage_type $index_nodes} res] != 0 } {
                     puts $res
                     set fail 1
                 }
@@ -398,79 +529,92 @@ proc check_test_run { file failed_out pending_out warning_out {fail_text "^FAIL"
 }
 
 # Run a test in an environment configured for transactional data store.
-proc run_tdsmethod { test args storage_type } {
-	run_envmethod $test txn $args $storage_type
+proc run_tdsmethod { test args storage_type indexnodes} {
+	run_envmethod $test txn $args $storage_type $indexnodes
 }
 
-proc run_cdsmethod { test { args "" } } {
-	run_envmethod $test cdb $args $storage_type
+proc run_cdsmethod { test args storage_type indexnodes} {
+	run_envmethod $test cdb $args $storage_type $indexnodes
 }
 
 # Run a test in an environment configured for transactional data store.
-proc run_envmethod { test {envtype ""} args storage_type } {
+proc run_envmethod { test {envtype ""} args storage_type indexnodes} {
     source ./include.tcl
+    puts "run_envmethod test=$test envtype=$envtype args=$args storage_type=$storage_type indexnodes=$indexnodes"
     global dbxml_err
     env_cleanup $testdir
-    set oflags " -create"
-
+    set oflags " -create -lock_max_locks 10000 -lock_max_lockers 10000"
+    
     if { [lsearch -exact $args thread] >= 0 } {
 	append oflags " -thread "
     }
-
+    
     if { $envtype == "tds" || $envtype == "txn" } {
 	set envflag "-txn"
     } elseif { $envtype == "cdb" } {
-	set envflag "-cdb"
+	set envflag "-cdb -cdb_alldb"
     } elseif { $envtype == "" } {
 	set envflag ""
     } else {
 	puts "Environment type $envtype not recognized."
     }
-
+    
     set stat [catch {
 	set env [eval {berkdb_env} $oflags $envflag \
 		     -mode 0644 -home $testdir -errfile errors.txt]
 	error_check_good env_open [is_valid_env $env] TRUE
 	append args " -env $env "
 	append args " -storage $storage_type"
+	if { $envtype == "cdb" } {
+	    append args " -cdb -cdb_alldb"
+	}
+	
+	if { $indexnodes == 1 } {
+	    append args " -indexnodes "
+	}
 	puts "[timestamp]"
-
+	
 	eval $test $args
-
+	
 	flush stdout
 	flush stderr
 	error_check_good envclose [$env close] 0
 	error_check_good envremove [berkdb envremove \
 					-home $testdir] 0
     } res]
-
+    
     if { $stat != 0} {
 	global errorInfo;
-
+	
 	set fnl [string first "\n" $errorInfo]
 	set theError [string range $errorInfo 0 [expr $fnl - 1]]
 	if {[string first FAIL $errorInfo] == -1} {
-	    error "FAIL:[timestamp]\
+	    puts "FAIL:[timestamp]\
 			    run_envmethod with $envtype env: $test: $theError"
+	    incr dbxml_err
 	} else {
-	    error $theError;
+	    puts "FAIL:[timestamp] $theError"
+	    incr dbxml_err
 	}
     }
 }
 
 # runs a specific test group with no environment
-proc run_xmltest { test args storage_type} {
-	source ./include.tcl
-  global dbxml_err
- 	check_handles
-
-  append args " -storage $storage_type"
-
-  eval $test $args
-
-  if { $dbxml_err == 0 } {
-    set dbxml_err [xml_verify_dir $testdir "" 1 1]
-  }
+proc run_xmltest { test args storage_type index_nodes} {
+    source ./include.tcl
+    global dbxml_err
+    check_handles
+    
+    append args " -storage $storage_type"
+    if { $index_nodes == 1} {
+	append args " -indexnodes "
+    }
+    
+    eval $test $args
+    
+    if { $dbxml_err == 0 } {
+	set dbxml_err [xml_verify_dir $testdir "" 1 1]
+    }
 }
 
 # run the conversion of the indexer/query processor tests
