@@ -1,9 +1,8 @@
 # See the file LICENSE for redistribution information.
 #
-# Copyright (c) 2000-2005
-#	Sleepycat Software.  All rights reserved.
+# Copyright (c) 2000,2006 Oracle.  All rights reserved.
 #
-# $Id: xml003.tcl,v 1.50 2005/04/20 18:31:40 bostic Exp $
+# $Id: xml003.tcl,v 1.58 2006/10/30 17:46:09 bostic Exp $
 #
 # TEST	xml003
 # TEST	Transaction testing
@@ -41,6 +40,12 @@ proc xml003 { {args ""} } {
 	set global_container_type [lindex $args $tindex]
     }
 
+    if { [lsearch -exact $args "-indexnodes"] >= 0 } {
+	set global_container_flags $DBXML_INDEX_NODES
+    } else {
+	set global_container_flags 0
+    }
+
     set tnum 3
     set basename $tnum
 
@@ -65,6 +70,7 @@ proc xml003 { {args ""} } {
 
     dbxml_error_check_good no_exceptions $ret ""
     dbxml_error_check_good env_close [$env close] 0
+    berkdb envremove -home $testdir
 
     puts "\tXml003.2: auto_commit tests"
     set basename $tnum.2
@@ -96,6 +102,7 @@ proc xml003 { {args ""} } {
 
     dbxml_error_check_good no_exceptions $ret ""
     dbxml_error_check_good env_close [$env close] 0
+    berkdb envremove -home $testdir
 
     puts "\tXml003.3: Transactionally linked DB-XML and BDB records."
     set basename $tnum.3
@@ -107,6 +114,7 @@ proc xml003 { {args ""} } {
     xml003_3 $env $basename.1 $oargs
 
     dbxml_error_check_good env_close [$env close] 0
+    berkdb envremove -home $testdir
     return
 }
 
@@ -164,7 +172,7 @@ proc xml003_1_2 { {env "NULL"} {basename $tnum} oargs } {
 	    dbxml_error_check_bad declare_commit $idecl NULL
 	    wrap XmlIndexDeclaration xid $idecl
 	    set retindex [$xid get_index]
-	    dbxml_error_check_good index $index $retindex
+	    dbxml_error_check_good index $retindex $index
 	    delete xid
 	}
 	delete container
@@ -603,11 +611,10 @@ proc xml003_1_6_1 { {env "NULL"} {basename $tnum} oargs } {
 	wrap XmlTransaction xtxn [$db createTransaction]
 	wrap XmlQueryExpression xe [$db prepare $xtxn $fullQuery $context]
 	wrap XmlResults xr [$xe execute $xtxn $context]
-	$xtxn $op
-	delete xtxn
-	$xr next $xv
 
 	dbxml_error_check_good "size_of_result_set" [$xr size] 1
+
+	$xr next $xv
 	new XmlValue newxv [$xv asString]
 	dbxml_error_check_good "check_contents" [$newxv asString] [$xv asString]
   delete newxv
@@ -617,6 +624,9 @@ proc xml003_1_6_1 { {env "NULL"} {basename $tnum} oargs } {
   dbxml_error_check_good "check parent" [$parent asString] $data_in
   delete parent
 
+	$xtxn $op
+	delete xtxn
+
 	delete xr
 	delete xe
 	delete xv
@@ -669,11 +679,10 @@ proc xml003_1_6_2 { {env "NULL"} {basename $tnum} oargs } {
 	new XmlValue xv
 	wrap XmlTransaction xtxn [$db createTransaction]
 	wrap XmlResults xr [$db query $xtxn $fullQuery $context 0]
-	$xtxn $op
-	delete xtxn
-	$xr next $xv
 
 	dbxml_error_check_good "size_of_result_set" [$xr size] 1
+
+	$xr next $xv
 	new XmlValue newxv [$xv asString]
 	dbxml_error_check_good "check_contents" [$newxv asString] [$xv asString]
 	delete newxv
@@ -683,6 +692,9 @@ proc xml003_1_6_2 { {env "NULL"} {basename $tnum} oargs } {
   dbxml_error_check_good "check parent" [$parent asString] $data_in
   delete parent
 
+	$xtxn $op
+	delete xtxn
+
 	delete xv
 	delete xr
 	delete xd
@@ -822,7 +834,7 @@ proc xml003_2_3 { {env "NULL"} {basename $tnum} oargs } {
     wrap XmlIndexDeclaration xid $idecl
     set retindex [$xid get_index]
     delete xid
-    dbxml_error_check_good index $index $retindex
+    dbxml_error_check_good index $retindex $index
 
     delete container
     delete uc
@@ -1066,12 +1078,10 @@ proc xml003_2_7_1 { {env "NULL"} {basename $tnum} oargs } {
     new XmlValue xv
     wrap XmlTransaction xtxn [$db createTransaction]
     wrap XmlResults xr [$db query $xtxn $fullQuery $context 0]
-    $xtxn commit
-    delete xtxn
-    $xr next $xv
 
     dbxml_error_check_good "size_of_result_set" [$xr size] 1
 
+    $xr next $xv
     new XmlValue newxv [$xv asString]
     dbxml_error_check_good "check_contents" [$newxv asString] [$xv asString]
     delete newxv
@@ -1081,6 +1091,9 @@ proc xml003_2_7_1 { {env "NULL"} {basename $tnum} oargs } {
     dbxml_error_check_good "check parent" [$parent asString] $data_in
     delete parent
 
+    $xtxn commit
+    delete xtxn
+
     delete xr
     delete xv
     delete xd
@@ -1311,7 +1324,9 @@ proc xml003_3 { {env "NULL"} {basename $tnum} oargs } {
 	wrap XmlDocument xd [$container getDocument $txn $doc_name]
 	set data_out [$xd getContentAsString]
 	delete xd
-	$dbtxn commit
+	# commit via BDB XML this time
+	#$dbtxn commit
+	$txn commit
 	delete txn
 	# Now check that both values were updated and that they match.
 	#		error_check_good bdb_get $ret [list [list $key $str]]
@@ -1335,8 +1350,9 @@ proc xml003_3 { {env "NULL"} {basename $tnum} oargs } {
 	# use update for in-place modification
 	$container updateDocument $txn $xd $uc
 	set ret [eval {$db put} -txn $dbtxn $key $bad_data_in]
-	# abort (use DB's txn this time)
-	$dbtxn abort
+	# abort (use XmlTransaction this time)
+	#$dbtxn abort
+	$txn abort
 	delete txn
 	delete xd
 
@@ -1348,7 +1364,13 @@ proc xml003_3 { {env "NULL"} {basename $tnum} oargs } {
 	wrap XmlDocument xd [$container getDocument $txn $doc_name]
 	set data_out [$xd getContentAsString]
 	delete xd
+	# commit via DbTxn this time
 	$dbtxn commit
+
+	# validate error if attempt to commit XmlTransaction
+	catch { $txn commit } ret
+	dbxml_error_check_good check_abort \
+	    [is_substr $ret "already committed"] 1
 	delete txn
 	# Now check that both values were updated and that they match.
 	#		error_check_good bdb_get $ret [list [list $key $str]]
@@ -1361,6 +1383,8 @@ proc xml003_3 { {env "NULL"} {basename $tnum} oargs } {
     }
     close $did
     error_check_good db_close [$db close] 0
+    berkdb dbremove -env $env $testfile
+
     delete container
     delete uc
     delete xmldb
