@@ -1,64 +1,78 @@
 # See the file LICENSE for redistribution information.
 #
-# Copyright (c) 2000-2005
-#	Sleepycat Software.  All rights reserved.
+# Copyright (c) 2000,2006 Oracle.  All rights reserved.
 #
-# $Id: xml008.tcl,v 1.34 2005/04/20 18:31:40 bostic Exp $
+# $Id: xml008.tcl,v 1.44.2.2 2007/01/09 22:52:32 gmf Exp $
 #
 # TEST	xml008
 # TEST	Container Management
 
 proc xml008 { args } {
-	puts "\nXml008: Container and database management ($args)"
-	source ./include.tcl
-
-	# Determine whether procedure has been called within an environment,
-	# and whether it is a transactional environment.
-	# If we are using an env, then the filename should just be the test
-	# number.  Otherwise it is the test directory and the test number.
-	set eindex [lsearch -exact $args "-env"]
-	set txnenv 0
-	set tnum 8
-	set oargs [eval {set_openargs} $args]
-
-	if { $eindex == -1 } {
-		set env NULL
-	} else {
-		incr eindex
-		set env [lindex $args $eindex]
-		set txnenv [is_txnenv $env]
-		if { $txnenv == 1 } {
-		}
-		set testdir [get_home $env]
+    puts "\nXml008: Container and database management ($args)"
+    source ./include.tcl
+    
+    # Determine whether procedure has been called within an environment,
+    # and whether it is a transactional environment.
+    # If we are using an env, then the filename should just be the test
+    # number.  Otherwise it is the test directory and the test number.
+    set eindex [lsearch -exact $args "-env"]
+    set txnenv 0
+    set tnum 8
+    set oargs [eval {set_openargs} $args]
+    
+    if { $eindex == -1 } {
+	set env NULL
+    } else {
+	incr eindex
+	set env [lindex $args $eindex]
+	set txnenv [is_txnenv $env]
+	if { $txnenv == 1 } {
 	}
+	set testdir [get_home $env]
+    }
+    
+    # assign the container type (default to whole document)
+    set tindex [lsearch -exact $args "-storage"]
+    if { $tindex == -1 } {
+	set global_container_type $XmlContainer_WholedocContainer
+    } else {
+	incr tindex
+	set global_container_type [lindex $args $tindex]
+    }
+    
+    if { [lsearch -exact $args "-indexnodes"] >= 0 } {
+	set global_container_flags $DBXML_INDEX_NODES
+    } else {
+	set global_container_flags 0
+    }
 
-  # assign the container type (default to whole document)
-  set tindex [lsearch -exact $args "-storage"]
-  if { $tindex == -1 } {
-    set global_container_type $XmlContainer_WholedocContainer
-  } else {
-    incr tindex
-    set global_container_type [lindex $args $tindex]
-  }
+    set basename $tnum
+    
+    # multiple containers
+    xml008_1 $env $txnenv $basename.1 $oargs
+    
+    # Miscellaneous container API
+    xml008_2 $env $txnenv $basename.2 $oargs
 
-	set basename $tnum
+    # XmlContainer API (regression test suite)
+    xml008_3 $env $txnenv $basename.3 $oargs
+    
+    # multiple databases
+    xml008_5 $env $txnenv $basename.5 $oargs
+    
+    # container type
+    xml008_6 $env $txnenv $basename.6 $oargs
+    
+    # container aliases
+    xml008_7 $env $txnenv $basename.7 $oargs
 
-  # multiple containers
-	xml008_1 $env $txnenv $basename.1 $oargs
+    # query timeout
+    xml008_8 $env $txnenv $basename.8 $oargs
 
-  # XmlContainer API (regression test suite)
-	xml008_3 $env $txnenv $basename.3 $oargs
-
-  # multiple databases
-	xml008_5 $env $txnenv $basename.5 $oargs
-
-  # container type
-	xml008_6 $env $txnenv $basename.6 $oargs
-
-  # container aliases
-	xml008_7 $env $txnenv $basename.6 $oargs
-
-	return
+    # container not found
+    xml008_9 $env $txnenv $basename.9 $oargs
+    
+    return
 }
 
 # multiple containers existing simultaneously
@@ -91,6 +105,40 @@ proc xml008_1 { {env "NULL"} {txnenv 0} {basename $tnum} oargs } {
     delete db
 }
 
+# miscellaneous container tests
+proc xml008_2 { {env "NULL"} {txnenv 0} {basename $tnum} oargs } {
+    puts "\tXml008.2: Miscellaneous container tests"
+    xml008_2_1 $env $txnenv $basename.1 $oargs
+}
+
+proc xml008_2_1 { {env "NULL"} {txnenv 0} {basename $tnum} oargs } {
+    puts "\t\tXml008.2.1: Changing default page size"
+    source ./include.tcl
+    set txn NULL
+    # transactions are not used, as they are not relevant
+    # to this case
+    xml_cleanup $testdir $env
+
+    xml_database db $testdir $env
+    # create a container using the default page size
+    wrap XmlContainer container [$db createContainer "$basename.1.dbxml" $oargs $global_container_type]
+
+    delete container
+    $db setDefaultPageSize 65536
+
+    # The fact that this works is enough to indicate passing
+    # Pass DB_CREATE to openContainer for existing container, and
+    # be sure the pagesizes do not match [#13768]
+    set targs [expr $oargs + $DB_CREATE]
+    wrap XmlContainer container [$db openContainer "$basename.1.dbxml" $targs $global_container_type]
+
+    # test getFlags interface (trivially)
+    set flags [$container getFlags]
+    dbxml_error_check_good "$basename: getFlags" $flags $targs
+    delete container
+    delete db
+}
+
 # XmlContainer API (DB XML 1.x regression test suite)
 proc xml008_3 { {env "NULL"} {txnenv 0} {basename $tnum} oargs } {
     puts "\tXml008.3: Test container methods"
@@ -114,6 +162,10 @@ proc xml008_3_1 { {env "NULL"} {txnenv 0} {basename $tnum} oargs } {
 
     set container_name $basename.dbxml
 
+    # make sure it doesn't exist
+    set exists [$db existsContainer $container_name]
+    dbxml_error_check_good "$basename: existsContainer" $exists 0
+
     # create a container
     if { $txnenv == 1 } {
 	wrap XmlTransaction txn [$db createTransaction]
@@ -126,6 +178,10 @@ proc xml008_3_1 { {env "NULL"} {txnenv 0} {basename $tnum} oargs } {
 
     delete container
 
+    # make sure it's there now
+    set exists [$db existsContainer $container_name]
+    dbxml_error_check_bad "$basename: existsContainer" $exists 0
+
     # open the existing container
     catch {
 	if { $txnenv == 1 } {
@@ -187,6 +243,8 @@ proc xml008_3_2 { {env "NULL"} {txnenv 0} {basename $tnum} oargs } {
 	} else {
 	    wrap XmlContainer container [$db openContainer $new_container_name $oargs]
 	}
+	# test getFlags while we're here
+	dbxml_error_check_good "$basename: getFlags" [$container getFlags] $oargs
 	delete container
     } ret
 
@@ -621,7 +679,7 @@ proc xml008_7 { {env "NULL"} {txnenv 0} {basename $tnum} oargs } {
 
     # try using dbxml:container.  This ignores
     # baseURI (failure would throw)
-    set cname "dbxml:$container_name"
+    set cname "dbxml:/$container_name"
     set queryExp1 "collection('$cname')"
     # query the container using alias
     if { $txnenv == 1 } {
@@ -657,3 +715,83 @@ proc xml008_7 { {env "NULL"} {txnenv 0} {basename $tnum} oargs } {
     delete mgr
 }
 
+# Query timeout test
+proc xml008_8 { {env "NULL"} {txnenv 0} {basename $tnum} oargs } {
+    puts "\t8.8: Query timeout"
+    source ./include.tcl
+    set txn "0"
+
+    xml_cleanup $testdir $env
+    xml_database mgr $testdir $env
+
+    # this is a very simple test that doesn't even create
+    # a container.   It uses a long-running standalong query
+    # to test timeout.
+
+    set queryExp "for \$i in (1 to 100000) return \$i"
+    wrap XmlQueryContext qc [$mgr createQueryContext]
+
+    # first let it work
+    wrap XmlResults res [$mgr query $queryExp $qc]
+    dbxml_error_check_good $basename.8 [$res size] 100000
+    delete res
+
+    $qc setQueryTimeoutSeconds 1
+    catch { 
+	wrap XmlResults nores [$mgr query $queryExp $qc]
+	puts "FAIL: should not have gotten here"
+	dbxml_error_check_bad "$basename.8" 1 1
+    } ret
+
+    dbxml_error_check_good $basename.8 [is_substr $ret "Query timed"] 1
+
+    delete qc
+    delete mgr
+}
+
+
+# Container not found error
+proc xml008_9 { {env "NULL"} {txnenv 0} {basename $tnum} oargs } {
+    puts "\t8.8: Container not found"
+    source ./include.tcl
+    set txn "0"
+
+    xml_cleanup $testdir $env
+    xml_database mgr $testdir $env
+
+    set nocontainer "not_a_container"
+    set is_container_name "is_a_container"
+    set dbnames {
+	"secondary_dictionary"
+	"secondary_document"
+	"primary_dictionary"
+	"secondary_configuration"
+    }
+
+    catch { 
+	wrap XmlContainer cont [$mgr openContainer $nocontainer $oargs ]
+	puts "FAIL: should not have gotten here"
+	dbxml_error_check_bad "$basename.9" 1 1
+    } ret
+
+    dbxml_error_check_good $basename.9 [is_substr $ret "file not found"] 1
+
+    foreach name $dbnames {
+	wrap XmlContainer container [$mgr createContainer $is_container_name $oargs $global_container_type]
+
+	# now close it
+	delete container
+
+	# remove a critical subdb
+	berkdb dbremove -env $env $is_container_name $name
+	catch { 
+	    wrap XmlContainer cont [$mgr openContainer $is_container_name $oargs ]
+	    puts "FAIL: should not have gotten here"
+	    dbxml_error_check_bad "$basename.9.$name" 1 1
+	} ret
+	dbxml_error_check_good $basename.9 [is_substr $ret "file not found"] 1
+	$mgr removeContainer $is_container_name
+    }
+    delete mgr
+}
+
