@@ -1,9 +1,8 @@
 # See the file LICENSE for redistribution information.
 #
-# Copyright (c) 2000-2005
-#	Sleepycat Software.  All rights reserved.
+# Copyright (c) 2000,2006 Oracle.  All rights reserved.
 #
-# $Id: xml009.tcl,v 1.64 2005/05/16 22:57:05 gmf Exp $
+# $Id: xml009.tcl,v 1.69.2.1 2007/01/09 23:22:24 gmf Exp $
 #
 # TEST	xml009
 # TEST	Container Document Storage
@@ -41,6 +40,12 @@ proc xml009 { args } {
 	set global_container_type [lindex $args $tindex]
     }
 
+    if { [lsearch -exact $args "-indexnodes"] >= 0 } {
+	set global_container_flags $DBXML_INDEX_NODES
+    } else {
+	set global_container_flags 0
+    }
+
     set basename $tnum
 
     # put and get documents (no namespace)
@@ -58,6 +63,9 @@ proc xml009 { args } {
     # "put document as" methods
     xml009_5 $env $txnenv $basename.5 $oargs
 
+    # misc document tests
+    xml009_6 $env $txnenv $basename.6 $oargs
+
     return
 }
 
@@ -1091,3 +1099,191 @@ proc compare_to_file { data filename testid } {
     dbxml_error_check_good $testid $data $fdata
 
 }
+
+#
+# Miscellaneous document tests
+#
+proc xml009_6 { {env "NULL"} {txnenv 0} {basename $tnum} oargs } {
+    puts "\n\txml009.6: Miscellanous document tests"
+    xml009_6_1 $env $txnenv $basename.1 $oargs
+    xml009_6_2 $env $txnenv $basename.2 $oargs
+}
+
+proc xml009_6_1 { {env "NULL"} {txnenv 0} {basename $tnum} oargs } {
+    puts "\n\t\t$basename: Document and node identity"
+    source ./include.tcl
+
+    xml_cleanup $testdir $env
+
+    set xtxn NULL
+    set id 0
+
+    # create manager, container
+    xml_database mgr $testdir $env
+
+    wrap XmlUpdateContext uc [$mgr createUpdateContext]
+
+    if {$txnenv == 1} {
+	wrap XmlTransaction xtxn [$mgr createTransaction]
+	wrap XmlContainer container [$mgr createContainer $xtxn $basename.dbxml $oargs $global_container_type]
+	$xtxn commit
+	delete xtxn
+    } else {
+	wrap XmlContainer container [$mgr createContainer $basename.dbxml $oargs $global_container_type]
+    }
+
+    # put document
+    incr id
+    set testid $basename.$id
+    puts "\t\t\t$testid: Document identity"
+
+    set content "<foo att=\"hello\"><bar>foobar</bar></foo>"
+    set name "foobar"
+    if {$txnenv == 1} {
+	wrap XmlTransaction xtxn [$mgr createTransaction]
+	set name_out [$container putDocument $xtxn $name $content $uc]
+	$xtxn commit
+	delete xtxn
+    } else {
+	set name_out [$container putDocument $name $content $uc]
+    }
+
+    # ignore transactions -- they won't affect these results
+
+    # check document equality/inequality
+    wrap XmlDocument doc1 [$container getDocument $name_out]
+    wrap XmlDocument doc2 [$container getDocument $name_out]
+    wrap XmlDocument doc3 [$mgr createDocument]
+    set eq [$doc1 equals $doc2]
+    dbxml_error_check_good $testid $eq 1
+    set eq [$doc1 equals $doc3]
+    dbxml_error_check_good $testid $eq 0
+
+    # check node equality/inequality
+    set queryExp "collection('$basename.dbxml')/foo/bar"
+    wrap XmlQueryContext qc [$mgr createQueryContext]
+    wrap XmlResults res1 [$mgr query $queryExp $qc]
+    wrap XmlResults res2 [$mgr query $queryExp $qc]
+
+    new XmlValue xv1
+    new XmlValue xv2
+    $res1 next $xv1
+    $res2 next $xv2
+
+    incr id
+    set testid $basename.$id
+    puts "\t\t\t$testid: Node identity"
+
+    set eq [$xv1 equals $xv2]
+    dbxml_error_check_good $testid $eq 1
+
+    # unequal nodes
+    set queryExp "collection('$basename.dbxml')/foo"
+    delete res1
+    wrap XmlResults res1 [$mgr query $queryExp $qc]
+    $res1 next $xv1
+    set eq [$xv1 equals $xv2]
+    dbxml_error_check_good $testid $eq 0
+
+    delete xv1
+    delete xv2
+    delete res1
+    delete res2
+    delete qc
+    delete doc1
+    delete doc2
+    delete doc3
+    # clean up
+    delete container
+    delete uc
+    delete mgr
+}
+
+#
+# This test exercises code that checks to be sure that an
+# operation is not incorrectly performed on a container
+# from which a document did not belong.  While this can technically
+# succeed or fail on its own, it's almost certainly an application
+# bug, and one that can lead to perceived data corruption
+#
+proc xml009_6_2 { {env "NULL"} {txnenv 0} {basename $tnum} oargs } {
+    puts "\n\t\t$basename: Document and container affinity"
+    source ./include.tcl
+
+    xml_cleanup $testdir $env
+
+    set xtxn NULL
+    set id 0
+
+    set cname1 $basename.1.dbxml
+    set cname2 $basename.2.dbxml
+
+    # create manager, containers
+    xml_database mgr $testdir $env
+
+    wrap XmlUpdateContext uc [$mgr createUpdateContext]
+
+    if {$txnenv == 1} {
+	wrap XmlTransaction xtxn [$mgr createTransaction]
+	wrap XmlContainer container1 [$mgr createContainer $xtxn $cname1 $oargs $global_container_type]
+	wrap XmlContainer container2 [$mgr createContainer $xtxn $cname2 $oargs $global_container_type]
+	$xtxn commit
+	delete xtxn
+    } else {
+	wrap XmlContainer container1 [$mgr createContainer $cname1 $oargs $global_container_type]
+	wrap XmlContainer container2 [$mgr createContainer $cname2 $oargs $global_container_type]
+    }
+
+    # put the same documents in both
+    set content "<foo att=\"hello\"><bar>foobar</bar></foo>"
+    set name1 "foobar1"
+    set name2 "foobar2"
+
+    if {$txnenv == 1} {
+	wrap XmlTransaction xtxn [$mgr createTransaction]
+	$container1 putDocument $xtxn $name1 $content $uc
+	$container1 putDocument $xtxn $name2 $content $uc
+	$container2 putDocument $xtxn $name1 $content $uc
+	$container2 putDocument $xtxn $name2 $content $uc
+	$xtxn commit
+	delete xtxn
+    } else {
+	$container1 putDocument $name1 $content $uc
+	$container1 putDocument $name2 $content $uc
+	$container2 putDocument $name1 $content $uc
+	$container2 putDocument $name2 $content $uc
+    }
+
+    # get document from container1, and remove it from container2
+    if {$txnenv == 1} {
+	wrap XmlTransaction xtxn [$mgr createTransaction]
+	wrap XmlDocument doc_to_delete [$container1 getDocument $xtxn $name1]
+	catch { $container2 deleteDocument $xtxn $doc_to_delete $uc 
+	    puts "FAIL: should not get here"
+	} ret
+	dbxml_error_check_good $basename [is_substr $ret "different container"] 1
+	catch { $container2 updateDocument $xtxn $doc_to_delete $uc 
+	    puts "FAIL: should not get here"
+	} ret
+	dbxml_error_check_good $basename [is_substr $ret "different container"] 1
+	$xtxn commit
+	delete xtxn
+    } else {
+	wrap XmlDocument doc_to_delete [$container1 getDocument $name1]
+	catch { $container2 deleteDocument $doc_to_delete $uc
+	    puts "FAIL: should not get here"
+	} ret
+	dbxml_error_check_good $basename [is_substr $ret "different container"] 1
+	catch { $container2 updateDocument $doc_to_delete $uc
+	    puts "FAIL: should not get here"
+	} ret
+	dbxml_error_check_good $basename [is_substr $ret "different container"] 1
+    }
+
+    delete doc_to_delete
+    delete container2
+    delete container1
+    delete uc
+    delete mgr
+}
+
