@@ -1,9 +1,8 @@
 # See the file LICENSE for redistribution information.
 #
-# Copyright (c) 2000-2005
-#	Sleepycat Software.  All rights reserved.
+# Copyright (c) 2000,2006 Oracle.  All rights reserved.
 #
-# $Id: xml012.tcl,v 1.37 2005/04/20 18:31:41 bostic Exp $
+# $Id: xml012.tcl,v 1.45 2006/10/30 17:46:10 bostic Exp $
 #
 # TEST	xml012
 # TEST	Delete Document
@@ -39,11 +38,22 @@ proc xml012 { args } {
         set global_container_type [lindex $args $tindex]
     }
 
+    if { [lsearch -exact $args "-indexnodes"] >= 0 } {
+	set global_container_flags $DBXML_INDEX_NODES
+    } else {
+	set global_container_flags 0
+    }
+
     set basename $tnum
 
+    # delete docs
     xml012_1 $env $txnenv $basename.1 $oargs
+    # update docs
     xml012_2 $env $txnenv $basename.2 $oargs
+    # indexing tests, with delete flag
     xml012_3 $env $txnenv $basename.3 $oargs
+    # update with indexes and bug fixes
+    xml012_4 $env $txnenv $basename.4 $oargs
 
     return
 }
@@ -94,7 +104,7 @@ proc xml012_3 { {env "NULL"} {txnenv 0} {basename $tnum} oargs {regress 0} } {
 }
 
 proc xml012_1_1 { {env "NULL"} {txnenv 0} {basename $tnum} oargs } {
-    puts "\t\txml012.1.1: Delete document with document name that does not exist (should fail)."
+    puts "\t\txml012.1.1: Get/Delete document with document name that does not exist (should fail)."
     source ./include.tcl
     xml_cleanup $testdir $env
 
@@ -110,6 +120,21 @@ proc xml012_1_1 { {env "NULL"} {txnenv 0} {basename $tnum} oargs } {
 	wrap XmlContainer container [$db createContainer $basename.dbxml $oargs]
     }
 
+    # get doc
+    if { $txnenv == 1 } {
+	wrap XmlTransaction txn [$db createTransaction]
+	catch {$container getDocument $txn ghost} ret
+	$txn commit
+	delete txn
+    } else {
+	catch {$container getDocument ghost } ret
+    }
+
+    dbxml_error_check_good $basename  \
+	[is_substr $ret "Document not found"] 1
+
+    set ret "yyy"
+    # now try delete
     if { $txnenv == 1 } {
 	wrap XmlTransaction txn [$db createTransaction]
 	catch {$container deleteDocument $txn ghost $uc} ret
@@ -379,7 +404,7 @@ proc xml012_2_2 { env txnenv basename oargs } {
         }
     } ret
     # except an exception
-    dbxml_error_check_good $basename [string match "*Error*" $ret] 1
+    dbxml_error_check_good $basename [string match "*Document not found*" $ret] 1
     delete xd
 
     # clean up
@@ -582,7 +607,7 @@ proc xml012_2_3_2 { env txnenv basename oargs } {
         }
     } ret
     # expect a (uniquess constraint) exception
-    dbxml_error_check_good $basename [string match "*Error*unique*" $ret] 1
+    dbxml_error_check_good $basename [string match "*Uniqueness*violation*" $ret] 1
 
     if { $txnenv == 1 } {
         $txn abort
@@ -798,7 +823,7 @@ proc xml012_2_4 { env txnenv basename oargs } {
 }
 
 # updates involving lazy evaluation, node storage
-# motivated by sr [#11838]
+# motivated by sr [#11838] and [#13933]
 proc xml012_2_5 { env txnenv basename oargs } {
     source ./include.tcl
 
@@ -819,21 +844,29 @@ proc xml012_2_5 { env txnenv basename oargs } {
         wrap XmlContainer container [$db createContainer $basename.dbxml $oargs $global_container_type]
     }
 
-    # add a document
+    # add several documents
     wrap XmlDocument xd [$db createDocument]
     set name "my_doc"
     $xd setName $name
     set content "<a><b/></a>"
-    set newContent "<a><c/></a>"
+    set newContent "<root><a>x</a><b>x</b><c><d>x</d></c></root>"
     $xd setContent $content
 
     if { $txnenv == 1 } {
         wrap XmlTransaction txn [$db createTransaction]
         $container putDocument $txn $xd $uc
+	# 3 more docs
+	$container putDocument $txn $name $content $uc $DBXML_GEN_NAME
+	$container putDocument $txn $name $content $uc $DBXML_GEN_NAME
+	$container putDocument $txn $name $content $uc $DBXML_GEN_NAME
         $txn commit
         delete txn
     } else {
         $container putDocument $xd $uc
+	# 3 more docs
+	$container putDocument $name $content $uc $DBXML_GEN_NAME
+	$container putDocument $name $content $uc $DBXML_GEN_NAME
+	$container putDocument $name $content $uc $DBXML_GEN_NAME
     }
     delete xd
 
@@ -844,26 +877,27 @@ proc xml012_2_5 { env txnenv basename oargs } {
         wrap XmlTransaction txn [$db createTransaction]
         wrap XmlResults xr [$db query $txn $queryString $qc]
         new XmlValue xv
-        $xr next $xv
-	dbxml_error_check_good "results exist" [$xv isNull] 0
-	wrap XmlDocument xd [$xv asDocument]
-	set val [$xv asString]
-	$xd setContent $newContent
-	$container updateDocument $txn $xd $uc
-
+        while { [ $xr next $xv ] } {
+	    dbxml_error_check_good "results exist" [$xv isNull] 0
+	    wrap XmlDocument xd [$xv asDocument]
+	    set val [$xv asString]
+	    $xd setContent $newContent
+	    $container updateDocument $txn $xd $uc
+	    delete xd
+	}
         $txn commit
         delete txn
-        delete xd
     } else {
         wrap XmlResults xr [$db query $queryString $qc]
         new XmlValue xv
-        $xr next $xv
-	dbxml_error_check_good "results exist" [$xv isNull] 0
-	wrap XmlDocument xd [$xv asDocument]
-	set val [$xv asString]
-	$xd setContent $newContent
-	$container updateDocument $xd $uc
-        delete xd
+        while { [ $xr next $xv ] } {
+	    dbxml_error_check_good "results exist" [$xv isNull] 0
+	    wrap XmlDocument xd [$xv asDocument]
+	    set val [$xv asString]
+	    $xd setContent $newContent
+	    $container updateDocument $xd $uc
+	    delete xd
+	}
     }
     delete xv
     delete xr
@@ -891,6 +925,95 @@ proc xml012_2_5 { env txnenv basename oargs } {
     delete db
 }
 
+# 12.4.1 - update where index values are the same [#14276] and [#14173]
+proc xml012_4_1 { env txnenv basename oargs } {
+    source ./include.tcl
+    puts "\t\t$basename: update with duplicate index entries"
+    set testid $basename
+    # prepare - database, contexts, container
+    xml_cleanup $testdir $env
+    xml_database mgr $testdir $env
+    wrap XmlUpdateContext uc [$mgr createUpdateContext]
+    wrap XmlQueryContext qc [$mgr createQueryContext]
+
+    if {$txnenv == 1} {
+        wrap XmlTransaction xtxn [$mgr createTransaction]
+        wrap XmlContainer container [$mgr createContainer $xtxn $basename.dbxml $oargs $global_container_type]
+        $xtxn commit
+        delete xtxn
+    } else {
+        wrap XmlContainer container [$mgr createContainer $basename.dbxml $oargs $global_container_type]
+    }
+
+    # this test case adds identical index entries
+    set content "<index><w n='XXX'><lk/></w><w n='XXX'><lk/></w></index>"
+    set newcontent "<index><w n='XXX'><lk/></w><w n='XXX'><lk/></w><w n='XXX'><lk/></w></index>"
+    set docname "foo"
+    set uri ""
+    set nodename "n"
+    set idx "node-attribute-equality-string" 
+    # add an index and put document and verify index entries
+    if {$txnenv == 1} {
+        wrap XmlTransaction xtxn [$mgr createTransaction]
+        $container addIndex $xtxn $uri $nodename $idx $uc
+        $container putDocument $xtxn $docname $content $uc
+	wrap XmlIndexLookup il [$mgr createIndexLookup $container $uri $nodename $idx]
+	wrap XmlResults xr [$il execute $xtxn $qc]
+	dbxml_error_check_bad "$testid - expected non-zero results (1)" [$xr size] 0
+        $xtxn commit
+        delete xtxn
+	delete il
+	delete xr
+    } else {
+        $container addIndex "" "n" "node-attribute-equality-string" $uc
+        $container putDocument $docname $content $uc
+	wrap XmlIndexLookup il [$mgr createIndexLookup $container $uri $nodename $idx]
+	wrap XmlResults xr [$il execute $qc]
+	dbxml_error_check_bad "$testid - expected non-zero results (1)" [$xr size] 0
+	delete il
+	delete xr
+    }
+
+    # now update the doc
+    wrap XmlDocument xd [$mgr createDocument]
+    $xd setName $docname
+    $xd setContent $newcontent
+
+    # perform the update and verify index entries
+    if {$txnenv == 1} {
+        wrap XmlTransaction xtxn [$mgr createTransaction]
+        $container updateDocument $xtxn $xd $uc
+	wrap XmlIndexLookup il [$mgr createIndexLookup $container $uri $nodename $idx]
+	wrap XmlResults xr [$il execute $xtxn $qc]
+	dbxml_error_check_bad "$testid - expected non-zero results (2)" [$xr size] 0
+        $xtxn commit
+        delete xtxn
+	delete il
+	delete xr
+    } else {
+        $container updateDocument $xd $uc
+	wrap XmlIndexLookup il [$mgr createIndexLookup $container $uri $nodename $idx]
+	wrap XmlResults xr [$il execute $qc]
+	dbxml_error_check_bad "$testid - expected non-zero results (2)" [$xr size] 0
+	delete il
+	delete xr
+    }
+
+    delete xd
+    # clean up
+    delete container
+    delete qc
+    delete uc
+    delete mgr
+}
+
+# 12.4  Updates and indexes and bug fixes
+proc xml012_4 { env txnenv basename oargs } {
+    puts "\t\t$basename: Updates and indexes"
+    xml012_4_1 $env $txnenv $basename.1 $oargs
+}
+
+
 # 2004-09-13 ARW maybe put these methods in xmlutils.tcl
 
 # check results by verifying count, a document name
