@@ -1,11 +1,10 @@
 # See the file LICENSE for redistribution information.
 #
-# Copyright (c) 2000-2005
-#	Sleepycat Software.  All rights reserved.
+# Copyright (c) 2000,2006 Oracle.  All rights reserved.
 #
-# $Id: xmlparams.tcl,v 1.13 2005/04/05 16:44:32 bostic Exp $
+# $Id: xmlparams.tcl,v 1.17 2006/10/30 17:46:10 bostic Exp $
 
-set test_names(xml)	[list xml003 xml006 xml008 xml009 xml010 xml011 xml012 xml013 xml014 xml017]
+set test_names(xml)	[list xml002 xml003 xml006 xml008 xml009 xml010 xml011 xml012 xml013 xml014 xml016 xml017]
 
 set xmlsubs {xml}
 
