@@ -1,9 +1,8 @@
 # See the file LICENSE for redistribution information.
 #
-# Copyright (c) 1996-2005
-#	Sleepycat Software.  All rights reserved.
+# Copyright (c) 1996,2006 Oracle.  All rights reserved.
 #
-# $Id: xmlutils.tcl,v 1.54 2005/04/20 18:31:42 bostic Exp $
+# $Id: xmlutils.tcl,v 1.65 2006/10/30 17:46:10 bostic Exp $
 #
 # DB XML test system utilities
 #
@@ -143,11 +142,11 @@ proc xml_dumploadtest { directory container pref quiet } {
 	foreach dbname [eval {exec $util_path/db_dump} $utilflag -l $container] {
 		# Don't check the dictionary or statistics databases - it's
 		# okay if the are different.  We can't easily check
-		# index_number - it has a custom bt_compare, which isn't
+		# indexes - they have a custom bt_compare, which isn't
 		# accessible from Tcl.
 		if { [ string first "primary_dictionary" $dbname ] != -1 ||
 		     [ string first "document_statistics" $dbname ] != -1 ||
-		     [ string first "index_number" $dbname ] != -1 } {
+		     [ string first "document_index" $dbname ] != -1 } {
 			continue
 		}
 
@@ -184,8 +183,18 @@ proc xml_dbcmp { dbname oldfile newfile } {
 	error_check_good olddb($oldfile:$dbname) [is_valid_db $olddb] TRUE
 
 	# Now open new database.
-	set newdb [eval {berkdb_open -rdonly} $dbarg $newfile $dbname]
-	error_check_good newdb($newfile:$dbname) [is_valid_db $newdb] TRUE
+  catch {eval {berkdb_open -rdonly} $dbarg $newfile $dbname} newdb
+  set valid [is_valid_db $newdb]
+
+  # Sometimes reindexing after a dbxml_load doesn't create the same
+  # index databases, as empty ones aren't recreated.
+  # In this case, we say that they databases compare equal
+  if { [binary_compare $valid TRUE] != 0 } {
+    if { [string equal -length 24 $dbname "secondary_document_index"] == 1 } {
+      return 0;
+    }
+  }
+	error_check_good newdb($newfile:$dbname) $valid TRUE
 
 	# Walk through olddb and newdb and make sure their contents
 	# are identical.
@@ -246,7 +255,7 @@ proc readFileUTF8 { filename } {
 }
 
 # returns 1 iff ok, 0 otherwise
-#   if the debug flag is set then the cause of failure is printed
+# if the debug flag is set then the cause of failure is printed
 proc putFileInContainer { database container filename {txn "NULL"} {debug 0} } {
     if { [catch {
 	wrap XmlUpdateContext uc [$database createUpdateContext]
@@ -328,6 +337,7 @@ proc xml_database { handle_r dir env { env_flags 0 } } {
 
     # the default container type is set globally by each test group
     $handle setDefaultContainerType $global_container_type
+    $handle setDefaultContainerFlags $global_container_flags
 
     # set up procedure (so that overloaded name works)
     proc $handle {cmd args} "eval XmlManager_\$cmd $handle \$args"
@@ -484,6 +494,10 @@ proc set_openargs { args } {
 		set openargs [expr $openargs + $DB_THREAD]
 	}
 
+  if { [lsearch -exact $args "-indexnodes"] >= 0 } {
+		set openargs [expr $openargs + $DBXML_INDEX_NODES]
+	}
+
 	return $openargs
 }
 
@@ -527,6 +541,10 @@ proc run_external_test_program { txnenv test_id program args} {
 	set args [concat $args "--nls"]
     }
 
+    if { $global_container_flags == $DBXML_INDEX_NODES } {
+	set args [concat $args "--indexnodes"]
+    }
+
     # escape square brackets and dollar signs
     regsub -all {\[} $args {\[} args
     regsub -all {\]} $args {\]} args
@@ -540,17 +558,17 @@ proc run_external_test_program { txnenv test_id program args} {
     set time_at_start_of_test [clock scan $now]
     puts "\t\tExecuting '$prog'"
 
-    #  puts "\t\t$xmltest_path/$prog $args"
+ #  puts "\t\t$xmltest_path/$prog $args"
 
     set ret [catch { eval exec $xmltest_path/$prog $args >> unit_tests.out 2>> unit_tests.err } res]
 
-    puts "\t\tChecking results"
-
     # verify success (1) - return code of external program
     set failed 0
     if { $ret != 0 } {
 	dbxml_error_check_good $res $ret 0
 
+    puts "\t\tChecking results"
+
 	# forcibly remove the TESTDIR to avoid inconsistent state for next test
 	fileremove -f $testdir
 	file mkdir $testdir
